#include "otf.h"

extern void ludcmp (double **, int, int *, double *);
extern void lubksb (double **, int, int *, double *);

 int *indx;
 double *tmax;
 double **a,*bb,**c;

int ChebychevSet(Y,C)
short int *C;
float *Y;
{
 int i,j,k,neq;
 double d,x;

 neq=order+1;
 a=dmatrix(1,neq,1,neq);
 c=dmatrix(0,H.nchan-1,1,neq);
 indx=ivector(1,neq);
 bb=dvector(1,neq);
 tmax=dvector(1,neq);

 for(i=1; i<=neq; i++) {
  for(j=1; j<=neq; j++) a[i][j]=0.0;
 }

 for(k=0; k<H.nchan; k++) {
   x=(double)k/((double)H.nchan-1.0);

   for(j=1; j<=neq; j++) {
	  switch (j) {
		case 1:
          c[k][j]=1.0;
		  break;
		case 2:
          c[k][j]=2.0 * x - 1.0;
		  break;
		case 3:
		  c[k][j]=1.0 + 8.0 * x * (x - 1.0);
		  break;
		case 4:
		  c[k][j]=-1.0 + x * (18.0 + x * (32.0 * x - 48.0));
		  break;
		case 5:
		  c[k][j]=1.0 + x * (-32.0 + x * (160.0 + x * (128.0 * x - 256.0)));
		  break;
		case 6:
		  c[k][j]=-1.0 + x * (50.0 + x * (-400.0 + x * (1120.0 + x * (512.0 * x - 1280.0))));
		  break;
		default:
		  fprintf(stderr,"\n Baseline order too high \n");
	      return(1);
	  }
   }

   if (C[k] && fabs(Y[k]) < 1.0e+30) {
     for(j=1; j<=neq; j++) {
       for(i=1; i<=neq; i++) a[j][i]+=c[k][i]*c[k][j];
	 }
   }
 }

 for(j=1; j<=neq; j++) {
   tmax[j]=0.0;
   for(i=1; i<=neq; i++) {
     if (a[j][i] > tmax[j]) tmax[j]=a[j][i];
   }
   if (tmax[j] == 0.0) {
     fprintf(stderr,"\n Baseline matrix singular\n");
     return(1);
   }
   for(i=1; i<=neq; i++) a[j][i]=a[j][i]/tmax[j];
 }
   
 ludcmp(a,neq,indx,&d);
 return(0);
}

int ChebychevFit1(Y,C)
short int *C;
float *Y;
{
 int i,j,k,neq;
   
 neq=order+1;
 for(i=1; i<=neq; i++) bb[i]=0.0;

 for(k=0; k<H.nchan; k++) {
  if (C[k] && fabs(Y[k]) < 1.0e+30) {
   for(j=1; j<=neq; j++) bb[j]+=Y[k]*c[k][j];
  }
 }

 for(j=1; j<=neq; j++) bb[j]=bb[j]/tmax[j];
   
 lubksb(a,neq,indx,bb);

 for(k=0; k<H.nchan; k++) {
  if (fabs(Y[k]) < 1.0e+30) {
   for(i=1; i<=neq; i++) Y[k]-=bb[i]*c[k][i];
  }
 }

 return(0);
}

int ChebychevFit2(Y,C)
short int *C;
float *Y;
{
 int i,j,k,neq;
 double d;
   
 neq=order+1;
 for(i=1; i<=neq; i++) {
  bb[i]=0.0;
  for(j=1; j<=neq; j++) a[i][j]=0.0;
 }

 for(k=0; k<H.nchan; k++) {
   if (C[k] && fabs(Y[k]) < 1.0e+30) {
     for(j=1; j<=neq; j++) {
	   bb[j]+=Y[k]*c[k][j];
       for(i=1; i<=neq; i++) a[j][i]+=c[k][i]*c[k][j];
	 }
   }
 }

 for(j=1; j<=neq; j++) {
   tmax[j]=0.0;
   for(i=1; i<=neq; i++) {
     if (a[j][i] > tmax[j]) tmax[j]=a[j][i];
   }
   if (tmax[j] == 0.0) {
     fprintf(stderr,"\n Baseline matrix singular\n");
     return(1);
   }
   bb[j]=bb[j]/tmax[j];
   for(i=1; i<=neq; i++) a[j][i]=a[j][i]/tmax[j];
 }
   
 ludcmp(a,neq,indx,&d);
   
 lubksb(a,neq,indx,bb);

 for(k=0; k<H.nchan; k++) {
  if (fabs(Y[k]) < 1.0e+30) {
   for(i=1; i<=neq; i++) Y[k]-=bb[i]*c[k][i];
  }
 }

 return(0);
}

void ChebychevFree()
{
 int neq;

 neq = order + 1;
 free_dmatrix(a,1,neq,1,neq);
 free_dmatrix(c,0,H.nchan-1,1,neq);
 free_ivector(indx,1,neq);
 free_dvector(bb,1,neq); 
 free_dvector(tmax,1,neq); 
}
