#ifdef GUI
#include "otftool.h"
#else
#include "otf.h"
#endif
#include "classio.h"

extern struct tm *caldate (double );
#ifdef GUI
extern void error_mes (gchar *, gchar *);
#endif

void ClassHEAD() {
  
  char telescope[12];
  int i;
  char datestring[15];
  char *month_name[] = { "JAN",
			 "FEB",
			 "MAR",
			 "APR",
			 "MAY",
			 "JUN",
			 "JUL",
			 "AUG",
			 "SEP",
			 "OCT",
			 "NOV",
			 "DEC"
  };
  struct tm *t1;
  int ERROR = 0;

  Rscan.PRESEC[GEN_SEC] = 1;
  Rscan.PRESEC[POS_SEC] = 1;
  Rscan.PRESEC[SPEC_SEC] = 1;
  Rscan.PRESEC[ORIG_SEC] = 0;

  
  Rscan.RXNUM = -1;

  Crpar.RSCAN = H.iscan;
  Crpar.RVER = 0;
  Crpar.RNUM = nfound;
  Com.CLUN = Com.OLUN;
  t1 = caldate (H.utd);
  sprintf(datestring,"%2d-%s-%4d",t1->tm_mday,month_name[t1->tm_mon],TM_YEAR_BASE+t1->tm_year);
  printf("%s\n",datestring);
  GAG_FROMDATE(datestring,Crpar.RDOBS,ERROR);
  Crpar.RKIND = KIND_SPEC;
  strcpy(Crpar.source.SOURCENAME,H.name);
  Crpar.REPOCH = H.epoch;
  switch(H.nchan) {
  case 64:
    sprintf(telescope,"FCRAO-QEF");
    break;
    /*  case 1024:
  case 2048:
    sprintf(telescope,"FCRAO-COR");
    break;
    */
  default:
    /* NB correlator */
    switch (H.iconf) {
    case 1:
      /* LOA */
      if (H.bw == 50.0) 
	sprintf(telescope, "NBCORLOA50");
      if (H.bw == 25.0)
	sprintf(telescope, "NBCORLOA25");
      if (H.bw == 12.5)
	sprintf(telescope, "NBCORLOA12.5");
      break;
    case 2:
      /* LOB */
      if (H.bw == 50.0) 
	sprintf(telescope, "NBCORLOB50");
      if (H.bw == 25.0)
	sprintf(telescope, "NBCORLOB25");
      if (H.bw == 12.5)
	sprintf(telescope, "NBCORLOB12.5");
      break;      
    }
    break;
  }
  strcpy(Crpar.tel.TELNAME,telescope);
  if (linename != NULL) {
    strcpy(Crpar.line.LINENAME,linename);
  }

  switch (H.telcoord) {
  case 2:
    Crpar.RTYPEC = TYPE_EQ;  
    break;
  case 3:
    Crpar.RTYPEC = TYPE_GA;
    break;
  default:
    Crpar.RTYPEC = TYPE_UN;
    break;    
  }			
  Crpar.RST = H.tsid;  Crpar.RUT = H.utd;
  Crpar.RTAU = H.tau;
  Crpar.RAZ = H.azs;  Crpar.REL = H.els;
  Crpar.RPROJ = P_RADIO;
  Crpar.RLAM = H.XCEN; Crpar.RBET = H.YCEN;
  Crpar.RRESTF = H.f0*1.e3; Crpar.RSKYFR = H.fsky*1.e3;  /*convert GHz to MHz for class*/
  Crpar.RNCHAN = H.nchan;
  Crpar.RRCHAN = (H.vlsr - H.xvel1)/H.dveldin + 1.0;
  Crpar.RVRES = H.dveldin; Crpar.RFRES = H.dfrdin*1.e3;
  Crpar.RVOFF = H.vlsr;
  /*  Crpar.RBAD = H.BLANK;*/
  Crpar.RVTYPE = VEL_LSR;   /*fix later with modcomp header value */
  Crpar.RH2OMM = H.h2omm; Crpar.RPAMB = H.pamb;
  Crpar.RTAMB = H.tamb;

  /* Fill in the velocity axis - this should be the same for all spectra */
  Crpar.RVMIN = (1. - Crpar.RRCHAN) * Crpar.RVRES;
  Crpar.RVMAX = Crpar.RVMIN;
  for (i = 0; i< Crpar.RNCHAN; i++) {
    Crdata.RDATAX[i] = ((float)(i+1) - Crpar.RRCHAN) * Crpar.RVRES;
    if (Crdata.RDATAX[i] < Crpar.RVMIN) Crpar.RVMIN = Crdata.RDATAX[i];
    if (Crdata.RDATAX[i] > Crpar.RVMAX) Crpar.RVMAX = Crdata.RDATAX[i];
  }

  ClassHIST();

  Rscan.PRESEC[PLOT_SEC] = 1;
  Rscan.PRESEC[CAL_SEC] = 1;
  Crpar.RH2OMM = 4.0;
  Rscan.PRESEC[BASE_SEC] = 1;
  /*  Crpar.RSIGFI = 0.3;
      Crpar.RNWIND = 5;*/ 

}

void ClassHIST() {
  /*  char *tok; */
  int i;
  int *scanhist;
  char temp[MAXPATHLEN];
  char errmsg[50];
  /*  if ((tok = (char *) malloc(64*sizeof(char)))  == NULL){
    perror("tok error: malloc");
    exit(3);
    } */
  int nseq,curr,diff, done;
  int rstart[MSEQ], rend[MSEQ];

  Rscan.PRESEC[ORIG_SEC] = 1;
  Rscan.PRESEC[DESCR_SEC] = 0;
  Crpar.R_NDUMP = 1;
  /* Fill in history of scans of the otf map */

  if ((scanhist = (int *) malloc (sizeof(int)*nfiles)) == NULL) {
#ifdef GUI
    sprintf(errmsg,"scanhist malloc error");
    error_mes("Error!", errmsg);
#else
    fprintf(stderr, "scanhist malloc error\n");
#endif
    return;
  }
    

  for (i = 0; i<nfiles; i++) {
    /*    strcpy(temp,rawdir);
	  strcat(temp,infile[i]); */
    strcpy (temp, infile[i]);
    if ((otffp=fopen(temp,"r"))==NULL) {
      sprintf(errmsg,"Input OTF file: %s",temp);
      strcat(errmsg,strerror(errno));
#ifdef GUI      
      error_mes("Error!", errmsg);
#endif      
      return;
    }
    ReadHEADER();
      /*    tok = strtok(infile[i],".");
	    tok = strtok(NULL, ".");*/
    scanhist[i] = H.iscan; 
    /* printf("%d\n",scanhist[i]);*/
    fclose(otffp);
  }

  if ((scanhist[nfiles-1]-scanhist[0]) == (nfiles-1)) {
    Crpar.RSTART[0] = scanhist[0]; Crpar.REND[0] = scanhist[nfiles-1];
    Crpar.RNSEQ = 1;
  } else {
    nseq = 0;
    done = 0;
    curr = 0;
    rstart[nseq] = scanhist[curr];
    while (1) {
      diff = scanhist[curr+1]-scanhist[curr];
      while (diff == 1) {
	curr++;
	/*	printf("%d %d\n",nseq,curr);*/
	if (curr >= (nfiles-1)) {
	  done = 1;
	  break;
	} else {
	  diff=scanhist[curr+1]-scanhist[curr];
	}
      }
      rend[nseq] = scanhist[curr];
      /*      printf("done with diff %d\n",nseq);*/
      nseq++;
      curr++;
      rstart[nseq] = scanhist[curr];
      if (done) break;
    }
    /*    nseq;*/

    if (nseq <= MSEQ) {
      Crpar.RNSEQ = nseq;
      for (i = 0; i<nseq; i++) {
	/*	printf("%d %d %d\n",i,rstart[i],rend[i]);*/
	Crpar.RSTART[i] = rstart[i]; Crpar.REND[i] = rend[i];
      }
    } else {
#ifdef GUI      
      append_log(2,1,1,otfgui->logtext,"\n History list overflowed Class origin buffer\n");
#else
      fprintf(stderr,"\n History list overflowed Class origin buffer\n");
#endif
      Crpar.RNSEQ = MSEQ;
      for (i = 0; i<MSEQ; i++) {
	Crpar.RSTART[i] = rstart[i]; Crpar.REND[i] = rstart[i];
      }
    }
  }      

  free(scanhist);
}
 

