#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "otf.h"

void Usage () {
  fprintf(stderr,"\n Usage: otfmap ");
  fprintf(stderr,"\n      -f input_list_filename");
  fprintf(stderr,"\n      -o output_filename");
  fprintf(stderr,"\n      -t outfile_type{fits:class}");
  fprintf(stderr,"\n      -wt ref_weight_mode{equal:biased}");
  fprintf(stderr,"\n      -cell cell_size (in arcsecs)");
  fprintf(stderr,"\n     OPTIONAL ARGUMENTS");
  fprintf(stderr,"\n      -mask horn_1,horn_2,...horn_n, default=all horns");
  fprintf(stderr,"\n      -kopt convolve_function {1:4}, default=1");
  fprintf(stderr,"\n      -base {yes:no}, default=no");
  fprintf(stderr,"\n      -order order, default=0");
  fprintf(stderr,"\n      -windtype {channel:velocity}");
  fprintf(stderr,"\n      -wind {xl1:xu1,xl2:xu2, ... xl5:xu5}");
  fprintf(stderr,"\n      -flatfield {yes:no}, default=no");
  fprintf(stderr,"\n      -calfile cal_filename, default=NULL");
  fprintf(stderr,"\n      -rmsweight {yes:no}, default=no");
  fprintf(stderr,"\n      -tsysweight {yes:no}, default=no");
  fprintf(stderr,"\n      -rmsthreshold threshold, default=1.0e+35");
  fprintf(stderr,"\n      -includemargin {yes:no}, default=no");
  fprintf(stderr,"\n      -l linename\n\n");
}

void ParseArguments(argc,argv,otffile,outfile)
char **argv,*otffile,*outfile;
int argc;
{
 char answer[8],ctmp,carg[64],*b;
 char outtype_char[5];
 char windtype_char[5];
 int nh, i,horn;
 long HornMASK;
 float w1tmp,w2tmp;

 i=1;
 while(i < argc) {
   sscanf(argv[i], "%s", carg);
   
   if (!strcmp(carg,"-cell")) {
    i++;
    THETA_N=atof(argv[i]);
    if (THETA_N < 0) {
      fprintf(stderr,"\n Grid size must be greater than 0\n");
      exit(2);
    }
   }
   else if (!strcmp(carg,"-o")) {
    i++;
    sscanf(argv[i], "%s", outfile);
   }
   else if (!strcmp(carg, "-t")) {
     i++;
     sscanf(argv[i],"%s", outtype_char);
     if (!strncmp(outtype_char,"fits",2) && !strncmp(outtype_char,"class",2)) {
       fprintf(stderr, "\n Unknown output filetype \n");
       Usage();
       exit(2);
     } else {
       if (!strncmp(outtype_char,"fits",2)) {
	 out_type = 1; 
       } else {
	 out_type = 2;
       } 
     }
   }
   else if (!strcmp(carg,"-wt")) {
    i++;
    sscanf(argv[i], "%s", WTmode);
    if (!strncmp(WTmode,"equal",2) && !strncmp(WTmode,"biased",2)) {
      fprintf(stderr,"\n Unknown REF weighting option\n");
      exit(2);
    }
   }
   else if (!strcmp(carg,"-f")) {
    i++;
    sscanf(argv[i], "%s", otffile);
   }
   else if (!strcmp(carg,"-calfile")) {
    i++;
    sscanf(argv[i], "%s", calfile);
   }
   else if (!strcmp(carg,"-kopt")) {
    i++;
    kopt=atoi(argv[i]);
    if (kopt < 1 || kopt > 4) {
      fprintf(stderr,"\n kopt value is out of range\n");
      exit(2);
    }
   }
   else if (!strcmp(carg,"-mask")) {
    i++;
    nh=0;
    CmdLineMASK=0;
    do {
      if (nh) {  
        b=strtok(NULL,",");
      }
      else b=strtok(argv[i],",");
      if (b==NULL) break; 
      nh++;
      horn=atoi(b);
      if (abs(horn) > MAXHORNS) {
        fprintf(stderr,"\n Illegal horn to mask\n");
        exit(2);
      }
      HornMASK=1<<(abs(horn)-1);
      CmdLineMASK=CmdLineMASK|HornMASK;
    } while(b!=NULL);
   }
   else if (!strcmp(carg,"-h")) {
     Usage();
     exit(0);
   }
   if (!strcmp(carg,"-order")) {
    i++;
    ORDER=atoi(argv[i]);
    if (ORDER > MAXORDER) {
      fprintf(stderr,"\n Max. allowed Baseline order is %d\n",(int) MAXORDER);
      exit(2);
    }
   }
   else if (!strcmp(carg,"-rmsthreshold")) {
    i++;
    rmsthreshold=atof(argv[i]);
    if (rmsthreshold <= 0) {
      fprintf(stderr,"\n rms threshold must be greater than 0\n");
      exit(2);
    }
   }
   else if (!strcmp(carg, "-windtype")) {
     i++;
     sscanf(argv[i],"%s", windtype_char);
     if (!strncmp(windtype_char,"channel",2) && !strncmp(windtype_char,"velocity",2)) {
       fprintf(stderr, "\n Unknown  window type \n");
       Usage();
       exit(2);
     } else {
       if (!strncmp(windtype_char,"channel",2)) {
	 wind_type = 1; 
       } else {
	 wind_type = 2;
       } 
     }
   }
   else if (!strcmp(carg,"-wind")) {
    i++;
    nh=0;
    do {
      if (nh) {  
        b=strtok(NULL,",");
      }
      else b=strtok(argv[i],",");
      if (b==NULL) break; 
      sscanf(b,"%f%c%f",&w1tmp,&ctmp,&w2tmp);
      W1[nh]=w1tmp;
      W2[nh]=w2tmp;
      nh++;
      if (nh > MAXWIND) {
        fprintf(stderr,"\n Too many windows defined. MAX=%2d\n",(int)MAXWIND);
        exit(2);
      }
    } while(b!=NULL);
    NWIND=nh;
   }
   else if (!strcmp(carg,"-flatfield")) {
    i++;
    sscanf(argv[i], "%s", answer);
    flatfield=0;
    if (!strncmp(answer,"yes",3)) flatfield=1;
   }     
   else if (!strcmp(carg,"-base")) {
    i++;
    sscanf(argv[i], "%s", answer);
    base=0;
    if (!strncmp(answer,"yes",3)) base=1;
   }     
   else if (!strcmp(carg,"-rmsweight")) {
    i++;
    sscanf(argv[i], "%s", answer);
    rmsweight=0;
    if (!strncmp(answer,"yes",3)) {
       rmsweight=1;
       noise_weight=1;
    }
   }     
   else if (!strcmp(carg,"-includemargin")) {
    i++;
    sscanf(argv[i], "%s", answer);
    include_margin=0;
    if (!strncmp(answer,"yes",3)) include_margin=1;
   }     
   else if (!strcmp(carg,"-tsysweight")) {
    i++;
    sscanf(argv[i], "%s", answer);
    tsysweight=0;
    if (!strncmp(answer,"yes",3)) {
      tsysweight=1;
      noise_weight=1;
    }
   }     
   else if (!strcmp(carg,"-l")) {
    i++;
    sscanf(argv[i], "%s", linename);
   }     
   else {
    fprintf(stderr,"\n  %s is not an option", carg);
   }
   i++;
 }
/* Make sure the 4 required command line arguments were given */
 if (!strcmp(outfile,"")) {
  fprintf(stderr,"\n Command line must contain output file option\n");
  Usage();
  exit(1);
 }
 if (!strcmp(otffile,"")) {
  fprintf(stderr,"\n Command line must contain in_file_list option\n");
  Usage();
  exit(1);
 }
 if (THETA_N==0.0) {
  fprintf(stderr,"\n Command line must contain cellsize option\n");
  Usage();
  exit(1);
 }
/* Check for inconsistentcies or missing optional arguments */
  if (rmsweight*tsysweight==1) {
    fprintf(stderr,"\n Can NOT noise weight by both tsys AND rms \n");
    Usage();
    exit(1);
  }
  if (base || rmsweight) {
    if (!NWIND) {               /* No windows defined */
      fprintf(stderr,"\n Baseline Windows must be set for baseline subtraction or rms noise weighting\n");
      Usage();
      exit(1);
    }
    if (!wind_type) {
      fprintf(stderr,"\n Window type not defined (channel | velocity) \n");
      Usage();
      exit(1);
    }
  }
}
