#ifdef GUI
#include "otftool.h"
#else
#include "otf.h"
#endif

#ifdef GUI
extern struct tm *caldate (double );

char 
*radtoRA (double rad)
{
  static char rastring[15];
  int RAh, RAm;
  float RAs;
  double RA;

  RA = rad * 12.0 / M_PIl;
  RAh = (int) RA;
  RAm = (int) (60.0 * (RA - RAh));
  RAs = (3600. * (RA - RAh - RAm / 60.0));	
  sprintf(rastring,"%02d:%02d:%06.3f",RAh, RAm, RAs);
  return &rastring[0];
}
  
char 
*radtoDEC (double rad)
{
  static char decstring[15];
  int DECd, DECm;
  float DECs;
  double DEC;
  
  DEC = rad * 180.0 / M_PIl;
  DECd = (int) DEC;
  DECm = (int)(60. * (DEC - DECd));		
  DECs = (3600. * (DEC - DECd - DECm / 60.0));		
  /*  if the Dec is negative, take the negative sign off the m and s  */		
  if (DECd < 0)	{		
      DECm = -DECm;		
      DECs = -DECs;		
  }		
  sprintf(decstring,"%02d:%02d:%06.3f",DECd, DECm, DECs);
  return &decstring[0];
}

  

void
GtkPrintHEADER(gchar *text)
{
  GtkWidget *headerwin;
  char title[250];
  GtkWidget *vbox;
  GtkWidget *scrolled_window;
  GtkWidget *txtw;
  GtkWidget *button;
  struct tm *t1;

  strcpy(title,"Header for ");
  sprintf(logtext_string, "%s", text);
  strcat(title,logtext_string);
  headerwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (headerwin), title);

  gtk_signal_connect (GTK_OBJECT (headerwin), "destroy",
                      GTK_SIGNAL_FUNC (gtk_widget_destroyed), &headerwin);
  gtk_container_border_width (GTK_CONTAINER (headerwin), 5);
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add (GTK_CONTAINER (headerwin), vbox);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                  GTK_POLICY_NEVER,
                                  GTK_POLICY_ALWAYS);
  gtk_widget_set_usize (scrolled_window, 400, 600);
  gtk_widget_show (scrolled_window);

  txtw = gtk_text_new (NULL, NULL);
  gtk_text_set_editable (GTK_TEXT (txtw), FALSE);
  gtk_container_add (GTK_CONTAINER (scrolled_window), txtw);
  gtk_widget_grab_focus (txtw);
  gtk_widget_show (txtw);
  gtk_text_set_word_wrap(GTK_TEXT (txtw), TRUE);

  append_log (2, 1, 2, txtw, "\n Header Information for file %s \n",text);
  append_log (2, 1, 1, txtw, "\n OTFDataREV: ");
  append_log (0, 1, 1, txtw, "%s ",H.otflab);
  append_log (2, 1, 1, txtw, "\n floatopt: ");
  append_log (0, 1, 1, txtw, " %s ",H.floatopt);
  append_log (2, 1, 1, txtw, "\n observerID: ");
  append_log (0, 1, 1, txtw, " %s ",H.observer);
  append_log (2, 1, 1, txtw, "\n UT Date: ");
  append_log (0, 1, 1, txtw, "  %12.7f ",H.utd);
  t1 = caldate (H.utd);
  append_log (0, 1, 1, txtw, " (%d/%d/%d) ",t1->tm_mon+1,t1->tm_mday,TM_YEAR_BASE+t1->tm_year);
  append_log (2, 1, 1, txtw, "\n LST: ");
  append_log (0, 1, 1, txtw, "%12.7f ",H.tsid);
  append_log (2, 1, 1, txtw, "\n Scan: ");
  append_log (0, 1, 1, txtw, " %5d", H.iscan);
  append_log (2, 1, 1, txtw, "\n Source: ");
  append_log (0, 1, 1, txtw, " %s ", H.name);
  append_log (2, 1, 1, txtw, "\n nhorns: ");
  append_log (0, 1, 1, txtw, "%2d ", H.nhorns);
  append_log (2, 1, 1, txtw, "   nchannels: ");
  append_log (0, 1, 1, txtw, " %4d ", H.nchan);
  append_log (2, 1, 1, txtw, "\n nsample: ");
  append_log (0, 1, 1, txtw, " %3d ", H.nsample);
  append_log (2, 1, 1, txtw, "\n tdump: ");
  append_log (0, 1, 1, txtw, "%4.2f secs ", H.tdump);
  append_log (2, 1, 1, txtw, "   nos: ");
  append_log (0, 1, 1, txtw, "%4.1f ", H.NOS);
  append_log (2, 1, 1, txtw, "\n Antenna Rate: ");
  append_log (0, 1, 1, txtw, "%4.1f arcmin/min", H.telrate);
  append_log (2, 1, 1, txtw, "\n ctype1: ");
  append_log (0, 1, 1, txtw, "%s ", H.vlabel);
  append_log (2, 1, 1, txtw, "\n ctype2: ");
  append_log (0, 1, 1, txtw, "%s", H.xlabel);
  append_log (2, 1, 1, txtw, "  ctype3: ");
  append_log (0, 1, 1, txtw, "%s ", H.ylabel);
  append_log (2, 1, 1, txtw, "\n  Center Position: ");
  append_log (0, 1, 1, txtw, "(%s, %s)", radtoRA(H.XCEN),radtoDEC(H.YCEN));
  append_log (2, 1, 1, txtw, "\n  AZ,EL: ");
  append_log (0, 1, 1, txtw, "%8.3f, %8.3f", H.azs*180./M_PI, H.els*180./M_PI);
  append_log (2, 1, 1, txtw, "\n  Requested Map Size: ");
  append_log (0, 1, 1, txtw, "%5.1f, %5.1f", H.XSIZE,H.YSIZE);
  append_log (2, 1, 1, txtw, "\n  Requested Map Offsets: ");
  append_log (0, 1, 1, txtw, "%5.2f, %5.2f", H.XOFFMAP,H.YOFFMAP);
  append_log (2, 1, 1, txtw, "\n  Ramp Size: ");
  append_log (0, 1, 1, txtw, "%5.2f, %5.2f", H.XRAMP,H.YRAMP);
  append_log (2, 1, 1, txtw, "\n  Map Angle: ");
  append_log (0, 1, 1, txtw, "%5.2f deg", H.rotang);
  append_log (0, 1, 1, txtw, "\n ");
  
  append_log (2, 1, 1, txtw, "\n VLSR: ");
  append_log (0, 1, 1, txtw, "%g km/s", H.vlsr);
  append_log (2, 1, 1, txtw, "\n Vel. Res: ");
  append_log (0, 1, 1, txtw, "%g km/s", H.dveldin);
  append_log (2, 1, 1, txtw, "\n Line Freq: ");
  append_log (0, 1, 1, txtw, "%g GHz", H.f0);
  append_log (2, 1, 1, txtw, "\n Sky  Freq: ");
  append_log (0, 1, 1, txtw, "%g GHz", H.fsky);



  button = gtk_button_new_with_label ("Dismiss");
  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (headerwin));
  gtk_widget_show (button);
  gtk_widget_show (vbox);
  gtk_widget_show (headerwin);
}

#endif

void PrintHEADER()
{
 int ihorn,idmp;

 fprintf(stdout,"\n ");
 fprintf(stdout,"\n OTFDataREV %s ",H.otflab);
 fprintf(stdout,"\n floatopt   %s ",H.floatopt);
 fprintf(stdout,"\n observer ID   %s ",H.observer);
 fprintf(stdout,"\n name   %s ",H.name);
 fprintf(stdout,"\n bunit   %s ",H.bunit);
 fprintf(stdout,"\n vlabel   %s ",H.vlabel);
 fprintf(stdout,"\n xlabel   %s ",H.xlabel);
 fprintf(stdout,"\n ylabel   %s ",H.ylabel);

 fprintf(stdout,"\n ");
 fprintf(stdout,"\n UT Date  %12.7f ",H.utd);
 fprintf(stdout,"\n Line Freq  %12.7f ",H.f0);
 fprintf(stdout,"\n Sky Freq  %12.7f ",H.fsky);
 fprintf(stdout,"\n azs  %12.7f ",H.azs);
 fprintf(stdout,"\n els  %12.7f ",H.els);
 fprintf(stdout,"\n XCEN  %12.7f ",H.XCEN);
 fprintf(stdout,"\n YCEN  %12.7f ",H.YCEN);
 fprintf(stdout,"\n xvel1  %12.7f ",H.xvel1);
 fprintf(stdout,"\n xfr1  %12.7f ",H.xfr1);
 fprintf(stdout,"\n dveldin  %12.7f ",H.dveldin);
 fprintf(stdout,"\n dfrdin  %12.7f ",H.dfrdin);
 fprintf(stdout,"\n fr1  %12.7f ",H.fr1);
 fprintf(stdout,"\n fr2  %12.7f ",H.fr1);

 fprintf(stdout,"\n ");
 fprintf(stdout,"\n LST      %12.7f ",H.tsid);
 fprintf(stdout,"\n utc      %12.7f ",H.utc);
 fprintf(stdout,"\n tdump    %12.7f ",H.tdump);
 fprintf(stdout,"\n tref    %12.7f ",H.tref);
 fprintf(stdout,"\n epoch      %12.7f ",H.epoch);
 fprintf(stdout,"\n bw      %12.7f ",H.bw);
 fprintf(stdout,"\n vlsr      %12.7f ",H.vlsr);
 fprintf(stdout,"\n vclsr      %12.7f ",H.vclsr);
 fprintf(stdout,"\n vcsun      %12.7f ",H.vcsun);
 fprintf(stdout,"\n NOS      %12.7f ",H.NOS);
 fprintf(stdout,"\n tau      %12.7f ",H.tau);
 fprintf(stdout,"\n h2omm      %12.7f ",H.h2omm);
 fprintf(stdout,"\n pamb      %12.7f ",H.pamb);
 fprintf(stdout,"\n tamb      %12.7f ",H.tamb);
 fprintf(stdout,"\n rotang      %12.7f ",H.rotang);
 fprintf(stdout,"\n XSIZE      %12.7f ",H.XSIZE);
 fprintf(stdout,"\n YSIZE      %12.7f ",H.YSIZE);
 fprintf(stdout,"\n XOFFMAP      %12.7f ",H.XOFFMAP);
 fprintf(stdout,"\n YOFFMAP      %12.7f ",H.YOFFMAP);
 fprintf(stdout,"\n XRAMP      %12.7f ",H.XRAMP);
 fprintf(stdout,"\n YRAMP      %12.7f ",H.YRAMP);


 fprintf(stdout,"\n ");
 fprintf(stdout,"\n Scan %5d ",H.iscan);
 fprintf(stdout,"\n Nhorns %2d ", H.nhorns);
 fprintf(stdout,"\n nchan %4d ", H.nchan);
 fprintf(stdout,"\n nsample %3d ", H.nsample);
 fprintf(stdout,"\n nblock %3d ", H.nblock);
 fprintf(stdout,"\n blksize %3d ", H.blksize);
 fprintf(stdout,"\n telcoord %3d ", H.telcoord);
 fprintf(stdout,"\n kx %3d ", H.kx);
 fprintf(stdout,"\n ky %3d ", H.ky);
 fprintf(stdout,"\n klo %3d ", H.klo);
 fprintf(stdout,"\n khm %3d ", H.khm);
 fprintf(stdout,"\n harm %3d ", H.harm);
 fprintf(stdout,"\n krecv %3d ", H.krecv);
 fprintf(stdout,"\n khorn %3d ", H.khorn);
 fprintf(stdout,"\n caltype %3d ", H.caltype);
 fprintf(stdout,"\n calscan %5d ", H.calscan);
 fprintf(stdout,"\n cfact %5d ", H.cfact);
 fprintf(stdout,"\n etab %5d ", H.etab);
 fprintf(stdout,"\n kmap %5d ", H.kmap);
 fprintf(stdout,"\n raoff %5d ", H.raoff);
 fprintf(stdout,"\n decoff %5d ", H.decoff);
 fprintf(stdout,"\n fsrate %5d ", H.fsrate);
 fprintf(stdout,"\n stype %5d ", H.stype);
 fprintf(stdout,"\n iconf %5d ", H.iconf);


 fprintf(stdout,"\n ");
 fprintf(stdout,"\n Pixel   HornID  Tsys     GAIN    DXHORNS  DYHORNS ");
 for(ihorn=0; ihorn<H.nhorns; ihorn++) fprintf(stdout,"\n  %2d      %2d    %5d    %5d    %5d    %5d",
    ihorn+1,H.hornID[ihorn],H.tsys[ihorn],H.gain[ihorn],H.DXHORNS[ihorn],H.DYHORNS[ihorn]);
     
 fprintf(stdout,"\n ");
 fprintf(stdout,"\n ");
 fprintf(stdout,"\n ---- X OFF ----- \n ");
 for(idmp=0; idmp<H.nsample; idmp++) {
  if (fmod((double)idmp,16.0)==0.0) fprintf(stdout,"\n %3d: ",idmp);
    fprintf(stdout," %6.2f",OTFPOS.XOFF[idmp]);
 }
 fprintf(stdout,"\n ");
 fprintf(stdout,"\n ---- Y OFF ----- \n ");
 for(idmp=0; idmp<H.nsample; idmp++) {
  if (fmod((double)idmp,16.0)==0.0) fprintf(stdout,"\n %3d: ",idmp);
    fprintf(stdout," %6.2f",OTFPOS.YOFF[idmp]);
 }
 fprintf(stdout,"\n ");
 fprintf(stdout,"\n ---- ANGLES (radians) ---\n ");
 for(idmp=0; idmp<H.nsample; idmp++) {
  if (fmod((double)idmp,16.0)==0.0) fprintf(stdout,"\n %3d: ",idmp);
    fprintf(stdout," %6.5f",OTFPOS.ANGLE[idmp]);
 }
 fprintf(stdout,"\n ");
 fprintf(stdout,"\n ---- X Positions ----- \n ");
 for(idmp=0; idmp<H.nsample; idmp++) {
  fprintf(stdout,"\n %3d: ",idmp);
  for(ihorn=0; ihorn<H.nhorns; ihorn++) 
    fprintf(stdout," %6.2f",XPOS[ihorn][idmp]);
 }
 fprintf(stdout,"\n ");
 fprintf(stdout,"\n ---- Y Positions ----- \n ");
 for(idmp=0; idmp<H.nsample; idmp++) {
  fprintf(stdout,"\n %3d: ",idmp);
  for(ihorn=0; ihorn<H.nhorns; ihorn++) 
    fprintf(stdout," %6.2f",YPOS[ihorn][idmp]);
 }
 fprintf(stdout,"\n ");
}
