#ifdef GUI
#include "otftool.h"
#else
#include "otf.h"
#endif

extern void UpdateHeader (float);

int ReadHEADER()
{
char dumstr1[8],dumstr2[8];
int iblreq,iblcur,offset,ih,idmp,jdmp;
float otfdatarev;
double dx,dy;

 if (fread(H.otflab,sizeof(char), 16, otffp) != 16) return(-1);
 /* Decode otflabel and check */
 sscanf(H.otflab,"%s %s %f", dumstr1,dumstr2,&otfdatarev);
 if (strncmp(dumstr1,"OTF",3) || strncmp(dumstr2,"REV",3) ) {
#ifdef GUI
   append_log(2,1,1,otfgui->logtext, "\n ERROR: Unrecognized Data file");
#else
   fprintf(stderr,"\n ERROR***\n Unrecognized Data file");
#endif
   return (-5);
 } 

/* check for rev of data with program */
  if (otfdatarev > (float)OTFREV) {
#ifdef GUI
    append_log(0,1,1,otfgui->logtext,"\n ERROR***\n Data file is Rev %4.2f", 
	       otfdatarev);
    append_log(0,1,1,otfgui->logtext,"\n Program Rev %4.2f", OTFREV);
    append_log(0,1,1,otfgui->logtext,
	       "\n Data file is incompatible with this version of the OTF program.  Exiting \n");
#else
    fprintf(stderr,"\n ERROR***\n Data file is Rev %4.2f", otfdatarev);
    fprintf(stderr,"\n Program Rev %4.2f", OTFREV);
    fprintf(stderr,"\n Data file is incompatible with this version of the OTF program.  Exiting\n");
#endif
    return(-5);
  }

/* Read basic header */
  rewind(otffp);
  if (fread(H.otflab,sizeof(OTFHeader_t),1, otffp) != 1) return (-1);
/* If otfdatarev less than 2.2 -- patch up header */
  if (otfdatarev < 2.15) UpdateHeader(otfdatarev);

/* check for floating point mode */
  if (!strncmp(H.floatopt,(char *)FLOATPT,4)) {
#ifdef GUI
    append_log(0,1,1,otfgui->logtext,"\n Input file floating point is %s", 
	       H.floatopt);
    append_log(0,1,1,otfgui->logtext,"\n Converting to %s", FLOATPT);
    append_log(0,1,1,otfgui->logtext,"\n Conversion not yet implemented ");
#else
    fprintf(stdout,"\n Input file floating point is %s", H.floatopt);
    fprintf(stdout,"\n Converting to %s", FLOATPT);
    fprintf(stdout,"\n Conversion not yet implemented ");
#endif
    return (-5);
  }




/* Allocate memory for position arrays */
   for(ih=0; ih<H.nhorns; ih++) {

     if ((XPOS[ih] = (float *) malloc(sizeof(float)*H.nsample))==NULL) {
#ifdef GUI
       strcpy(logtext_string,"\n Can not allocate memory for array\n");
       append_log(2,1,1,otfgui->logtext,logtext_string);
       return(-1);
#else
       fprintf(stderr,"\n Can not allocate memory for array\n");
       exit(3);
#endif
     }
     if ((YPOS[ih] = (float *) malloc(sizeof(float)*H.nsample))==NULL) {
#ifdef GUI
       strcpy(logtext_string,"\n Can not allocate memory for array\n");
       append_log(2,1,1,otfgui->logtext,logtext_string);
       return(-1);
#else
       fprintf(stderr,"\n Can not allocate memory for array\n");
       exit(3);
#endif
     }
   }

/* Read position information */
   idmp=0;
   iblcur=-1;
   do {
     iblreq=idmp/H.blksize;
     if (iblreq != iblcur) {
       offset=HEADERSIZE+iblreq*( POSITIONHEADERSIZE +
               sizeof(float)*H.nchan*H.nhorns*H.blksize);
       fseek(otffp,offset,SEEK_SET);
       fread(OTFPOS.ANGLE,sizeof(OTFPositions_t),1,otffp);
       iblcur=iblreq;
     }

/*   Fill in position arrays */
     jdmp=(int)fmod((double)idmp,(double)H.blksize);
     if (OTFPOS.XOFF[jdmp] < 21600) {
/*    Fix 2.0 data */
      if (otfdatarev < 2.05) 
        OTFPOS.XOFF[jdmp]=OTFPOS.XOFF[jdmp]*cos(H.YCEN+2.90888209e-04*OTFPOS.YOFF[jdmp]);
      for(ih=0; ih<H.nhorns; ih++) {
       if (H.kmap == 1 || H.kmap == 3) {
         dx=OTFPOS.XOFF[jdmp]+(H.DXHORNS[ih]-H.DXHORNS[H.khorn-1])*60.0/1.0e+04;
         dy=OTFPOS.YOFF[jdmp]+(H.DYHORNS[ih]-H.DYHORNS[H.khorn-1])*60.0/1.0e+04;
         XPOS[ih][idmp]=-dx*cos(OTFPOS.ANGLE[jdmp])+dy*sin(OTFPOS.ANGLE[jdmp]);
         YPOS[ih][idmp]=dx*sin(OTFPOS.ANGLE[jdmp])+dy*cos(OTFPOS.ANGLE[jdmp]);
	 if(H.telcoord == 3)
	   XPOS[ih][idmp] /= cos(H.YCEN+2.90888209e-04*YPOS[ih][idmp]);
	 /* line above added following Mike Brewer's correction of 
	    converting azoff/eloff to DL/DB by dividing by Cos(B).
	    See mail from Brewer dated Mar 3, 2005 */
/*
         fprintf(stderr,"\n %3d %3d %2d %4.2f %4.2f %5.4f %6.3f %6.3f %6.2f %6.2f",
            jdmp,idmp,ih,OTFPOS.XOFF[jdmp],OTFPOS.YOFF[jdmp],OTFPOS.ANGLE[jdmp],
            dx,dy,XPOS[ih][idmp],YPOS[ih][idmp]);
*/
       }
       else {
         dx=(H.DXHORNS[ih]-H.DXHORNS[H.khorn-1])*60.0/1.0e+04;
         dy=(H.DYHORNS[ih]-H.DYHORNS[H.khorn-1])*60.0/1.0e+04;

         YPOS[ih][idmp]=OTFPOS.YOFF[jdmp]+
                  dx*sin(OTFPOS.ANGLE[jdmp])+dy*cos(OTFPOS.ANGLE[jdmp]);
	 if(H.telcoord == 3)
           XPOS[ih][idmp] = OTFPOS.XOFF[jdmp] + 
	     (dy*sin(OTFPOS.ANGLE[jdmp]) - dx*cos(OTFPOS.ANGLE[jdmp])) 
	     / cos(H.YCEN+2.90888209e-04*YPOS[ih][idmp]);
         else
           XPOS[ih][idmp] = OTFPOS.XOFF[jdmp] 
	     + dy*sin(OTFPOS.ANGLE[jdmp]) 
	     - dx*cos(OTFPOS.ANGLE[jdmp]);
	 /* line above added following Mike Brewer's correction of 
	    converting azoff/eloff to DL/DB by dividing by Cos(B).
	    See mail from Brewer dated Mar 3, 2005 */

       }
      }
     }
     else {                   /* these are cal or ref data */
       for(ih=0; ih<H.nhorns; ih++) {
         XPOS[ih][idmp]=OTFPOS.XOFF[jdmp];
         YPOS[ih][idmp]=OTFPOS.YOFF[jdmp];
       }
     }
     idmp++;
   } while(idmp < H.nsample);

   return(0);
}
