#include "otftool.h"

/* Use parsecfg.c & parsecfg.h to parse the configuration files
 * parsecfg is a library for parsing a configuration file written by
 * Yuuki NINOMIYA <gm@debian.or.jp>
 * we will use this api here
 * Gopal Narayanan <gopal@astro.umass.edu>
 */

#include "parsecfg.h"

/* array of cfgStruct */

cfgStruct otftoolcfg[] = {
  /* parameter  type       address of variable */
  {"OUTFILE",     CFG_STRING,   &outfile},
  {"OUTTYPE",     CFG_INT,      &out_type},
  {"OTFLISTFILE", CFG_STRING,   &otffile},
  {"LINENAME",    CFG_STRING,   &linename_sav},
  {"WEIGHTMODE",  CFG_INT,      &weight_mode},
  {"KOPT",        CFG_INT,      &kopt},
  {"THETA_N",     CFG_FLOAT,    &THETA_N},
  {"RMSTHRESHLD", CFG_FLOAT,    &rmsthreshold},
  {"OUTPUTTYPE",  CFG_INT,      &out_type},
  {"LOGMODE",     CFG_SHORT,    &log_mode},
  {"ORDER",       CFG_SHORT,    &ORDER},
  {"NWIND",       CFG_SHORT,    &NWIND}, 
  {"WINDOWTYPE",  CFG_SHORT,    &wind_type},
  {"ASSESSMODE",  CFG_SHORT,    &assess_mode},
  {"RMSWEIGHT",   CFG_SHORT,    &rmsweight},
  {"TSYSWEIGHT",  CFG_SHORT,    &tsysweight},
  {"NOISEWEIGHT", CFG_SHORT,    &noise_weight},
  {"FLATFIELD",   CFG_SHORT,    &flatfield},
  {"BASELINESPEC",CFG_SHORT,    &base},
  {"IFSELECTION", CFG_SHORT,    &lo_sel},
  {"BWSELECTION", CFG_SHORT,    &bw_sel},
  {"FILTSNAME",  CFG_STRING,   &sname},
  {"FILT_STARTSCAN", CFG_SHORT, &filter.startscan},
  {"FILT_ENDSCAN", CFG_SHORT,   &filter.endscan},
  {"FILT_F0",      CFG_FLOAT,   &filter.f0},
  {"FILT_DELTAF",  CFG_FLOAT,   &filter.deltaf},
  {"INCLUDE_MARGIN", CFG_SHORT, &include_margin},
  {"START_YEAR",   CFG_INT,     &start_tm.tm_year},
  {"START_MONTH",  CFG_INT,     &start_tm.tm_mon},
  {"START_DAY",    CFG_INT,     &start_tm.tm_mday},
  {"END_YEAR",     CFG_INT,     &end_tm.tm_year},
  {"END_MONTH",    CFG_INT,     &end_tm.tm_mon},
  {"END_DAY",      CFG_INT,     &end_tm.tm_mday},
  {"MAIN_WINDOW_WIDTH",CFG_INT, &main_window_width},
  {"MAIN_WINDOW_HEIGHT",CFG_INT, &main_window_height},  
  {NULL,          CFG_END,      NULL}   /* no more params */
};

int
save_defaults_as (char *path) 
{
  int ret;

  ret = cfgDump (path, otftoolcfg, CFG_SIMPLE, 0);
  if (ret == 0) {
    append_log (0, 1, 1, otfgui->logtext, "Saved default configs to %s\n", path);
  } else {
    append_log (2, 1, 1, otfgui->logtext, "Could not save default configs to %s\n", path);
  }
  return ret;
}

int save_defaults ()
{
  int ret;

  ret = save_defaults_as ("otftool.sav");
  return ret;
}

void
getcfgvar () 
{
  char *txt;

  if (fetchVarFromCfgFile("otftool.sav", "OUTFILE", &txt, CFG_STRING, CFG_SIMPLE, 0, NULL) == 0) {
                printf("OUTFILE: %s\n", txt);
  } else {
    printf("Failed \n");
  }
}


int
config_parse (char *path)
{
  int ret;

  ret = cfgParse(path, otftoolcfg, CFG_SIMPLE);
  return ret;
}

int
config_parse_defaults () 
{

  if (config_parse ("otftool.sav") == -1) {
    printf("Warning: No default otftool.sav found. You may proceed\n");
    return 1;
  } else {
    return 0;
  }

}
