/*
 * index_inc.h
 * Derived from GAG/linux-intel/astro/class/index.inc
 * fortran include file
 * Gopal Narayanan <gopal@astro.umass.edu>
 * Sept 4, 2001
 */

#ifndef __INDEX_INC_H_
#define __INDEX_INC_H_

#include <cfortran.h>

#define M_IX 100000          /* max number of scans in input file: increased 1.25-fold from class*/
#define M_OX 100000          /* max number of scans in output file: increased 1.25-fold from class*/
#define M_CX 100000          /* max number of scans in current index: increased 1.25-fold from class*/

typedef struct {
  union {
    long int xindex[32];          /* search key values */
    long int xbloc;
    struct {
      long int dummy;
      long int xnum;
    } s1;
    struct {
      long int dummy[2];
      long int xver;
    } s2;
    struct {
      long int dummy[3];
      long int xsourc[3];
      char xcsour[12];
    } s3;
    struct {
      long int dummy[6];
      long int xline[3];
      char xcline[12];
    } s4;
    struct {
      long int dummy[9];
      long int xteles[3];
      char xctele[12];
    } s5;
    struct {
      long int dummy[12];
      long int xdobs;
    } s6;
    struct {
      long int dummy[13];
      long int xdred;
    } s7;
    struct {
      long int dummy[14];      
      long int xoff1;
    } s8;
    struct {
      long int dummy[15];
      long int xoff2;
    } s9;
    struct {
      long int dummy[16];
      long int xtype;
    } s10;
    struct {
      long int dummy[17];
      long int xkind;
    } s11;
    struct {
      long int dummy[18];
      long int xqual;
    } s12;
    struct {
      long int dummy[19];
      long int xscan;
    } s13;
    struct {
      long int dummy[20];
      long int xposa;
    } s14;
  } xindex_union;
  union {
    long int lindex[32];          /* values read into index */
    long int lbloc;
    struct {
      long int dummy;
      long int lnum;
    } s1;
    struct {
      long int dummy[2];
      long int lver;
    } s2;
    struct {
      long int dummy[3];
      long int lsourc[3];
      char lcsour[12];
    } s3;
    struct {
      long int dummy[6];
      long int lline[3];
      char lcline[12];
    } s4;
    struct {
      long int dummy[9];
      long int lteles[3];
      char lctele[12];
    } s5;
    struct {
      long int dummy[12];
      long int ldobs;
    } s6;
    struct {
      long int dummy[13];
      long int ldred;
    } s7;
    struct {
      long int dummy[14];      
      long int loff1;
    } s8;
    struct {
      long int dummy[15];
      long int loff2;
    } s9;
    struct {
      long int dummy[16];
      long int ltypec;
    } s10;
    struct {
      long int dummy[17];
      long int lkind;
    } s11;
    struct {
      long int dummy[18];
      long int lqual;
    } s12;
    struct {
      long int dummy[19];
      long int lscan;
    } s13;
    struct {
      long int dummy[20];
      long int lposa;
    } s14;
  } lindex_union;
  long int YNUM, YVER, YDOBS, YDRED;
  float YOFF1, YOFF2;
  long int YSCAN;
  float YPOSA;
  int LAST, FNUM, FVER, FSOURC, FLINE, FTELES, FDOBS, FDRED, FSCAN, FPOSA; /* flags */
  int FOFF1, FOFF2;
  int IX_BLOC[M_IX], OX_BLOC[M_OX], CX_BLOC[M_CX], IX_KIND[M_IX], IX_QUAL[M_IX];
  long int IX_NUM[M_IX], OX_NUM[M_OX], CX_NUM[M_CX];
  int IX_VER[M_IX], OX_VER[M_OX], CX_VER[M_CX], CX_IND[M_CX];
  long int CXNEXT;
  int ISOURC, ILINE, ITELES;
  int IX_DOBS[M_IX], IX_SCAN[M_IX];
} IND_DEF;

#define Ind COMMON_BLOCK(IND,ind)
COMMON_BLOCK_DEF(IND_DEF,Ind);


#define XINDEX xindex_union.xindex
#define XBLOC xindex_union.xbloc
#define XNUM xindex_union.s1.xnum
#define XVER xindex_union.s2.xver
#define XSOURC xindex_union.s3.xsourc
#define XCSOUR  xindex_union.s3.xcsour
#define XLINE xindex_union.s4.xline
#define XCLINE xindex_union.s4.xcline
#define XTELES xindex_union.s5.xteles
#define XCTELE xindex_union.s5.xctele
#define XDOBS xindex_union.s6.xdobs
#define XDRED xindex_union.s7.xdred
#define XOFF1 xindex_union.s8.xoff1
#define XOFF2 xindex_union.s9.xoff2
#define XTYPE xindex_union.s10.xtype
#define XKIND xindex_union.s11.xkind
#define XQUAL xindex_union.s12.xqual
#define XSCAN xindex_union.s13.xscan
#define XPOSA xindex_union.s14.xposa

#define LINDEX lindex_union.lindex
#define LBLOC lindex_union.lbloc
#define LNUM lindex_union.s1.lnum
#define LVER lindex_union.s2.lver
#define LSOURC lindex_union.s3.lsourc
#define LCSOUR  lindex_union.s3.lcsour
#define LLINE lindex_union.s4.lline
#define LCLINE lindex_union.s4.lcline
#define LTELES lindex_union.s5.lteles
#define LCTELE lindex_union.s5.lctele
#define LDOBS lindex_union.s6.ldobs
#define LDRED lindex_union.s7.ldred
#define LOFF1 lindex_union.s8.loff1
#define LOFF2 lindex_union.s9.loff2
#define LTYPEC lindex_union.s10.ltypec
#define LKIND lindex_union.s11.lkind
#define LQUAL lindex_union.s12.lqual
#define LSCAN lindex_union.s13.lscan
#define LPOSA lindex_union.s14.lposa
  

#endif
