#include <stdio.h>
#include <math.h>
#include <stdlib.h>
void read_beams(char cen_name[25]);
int read_pattern(char pat_name[25]);
double parallactic(void);
void shifter(int pattern);
void rotate(int pattern);
void rotate_offset(int pattern);
void integrate(int nrpt, int tmain, int num_point);
void MakeImage(int tmain);

/* This program is to calculate the sensitivity of a SEQUOIA     */
/* observation for various configurations of the 32 pixels       */
/* and using various patterns to map a single SEQUOIA footprint. */

/*  Chris Taylor (FCRAO), Jan 2000                               */

/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */
/*                  GLOBAL VARIABLES                            */

int X0[32], Y0[32],xtemp,ytemp;
float X1[32], Y1[32];
int xcen[32], ycen[32];
float xcen1[32], ycen1[32];
int xshift[99], yshift[99];
float xshift1[99], yshift1[99];
float sky[600][600] = { {0, 0} };
float lat;
double HA, DEC, pangle = 0.0;
double PI = 3.141592653589793;
char askrot;
   
/* xcen, ycen = the central positions of a beam                   */
/* xcen1, ycen1 = the central positions of a beam after rotation  */
/*                through the parallactic angle                   */
/* xshift, yshift = position shifts for mapping a footprint       */
/*                    usually pixel 11                            */
/* sky = plane of the sky;                                        */
/* HA = hour angle in decimal hours                               */
/* DEC = declination in decimal degrees                           */
/* lat = observer's latitude                                      */
/* pangle = parallactic angle                                     */
/* PI = pi                                                        */
/* askrot -- if = 'y' then do sky rotation, if 'n' then don't     */

/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */


main(argc,argv)
int argc;
char *argv[];
{
/*                DECLARE VARIABLES                              */

     FILE *out1, *in1, *beams; 
     int tmain, n_obs, count;
     int num_point, nrpt, nmaps, maps; 
     char out_name1[25], in_name[25], cen_name[25], pat_name[25]; 
     char Hsign, Dsign, beam_name[] = "beamfile";
     float Hhour, Hmin, Hsec, Ddeg, Dmin, Dsec;

/* out1 = pointer to output file name                            */
/* n_obs = number of observations (array pointings) to be done   */
/* num_point = number of positions in the mapping pattern        */
/* X1, Y1 = counters used to write out sky array                 */
/* out_name1 = name of output binary file                         */
/* Hhour, Hmin, Hsec = hours, minutes, seconds for hour angle    */
/* Ddeg, Dmin, Dsec = degrees, minutes, seconds for declination  */


/*                BEGIN                                          */

       printf ("Enter name of input file: \n");
       scanf ("%24s", in_name);


/*                OPEN FILE CONTAINING START INFO                 */

     if ( (in1 = fopen (in_name,"r")) == NULL)
        { printf ("This file does not exist. Quitting now.\n");
          exit(0);}

/*                READ START INFO                                 */

     fscanf (in1, " %24s %*[^\n]\n", &out_name1);
     fscanf (in1, " %c %*[^\n]\n", &askrot);
     fscanf (in1, " %d %*[^\n]\n", &nmaps);
     fscanf (in1, " %d %*[^\n]\n", &nrpt);
     fscanf (in1, " %d %*[^\n]\n", &tmain);
     fscanf (in1, " %c%f:%f:%f %c%f:%f:%f %*[^\n]\n", &Hsign, &Hhour, &Hmin, 
                                       &Hsec, &Dsign, &Ddeg, &Dmin, &Dsec);
     fscanf (in1, " %f %*[^\n]\n", &lat);
     fscanf (in1, " %24s %*[^\n]\n", &cen_name);
     fscanf (in1, " %24s %*[^\n]\n", &pat_name);


     fclose(in1);


/* convert to decimal hours and decimal degrees                  */

    if (Hsign == '-')
       {
       HA = -Hhour - (Hmin/60) - (Hsec/3600);
       }
    else
       {
       HA = Hhour + (Hmin/60) + (Hsec/3600);
       }

    if (Dsign == '-')
       {
       DEC = -Ddeg - (Dmin/60) - (Dsec/3600);
       }
    else
       {
       DEC = Ddeg + (Dmin/60) + (Dsec/3600);
       }

    printf ("Output file name = %s\n", out_name1);
    printf ("askrot = %c\n", askrot);
    printf ("nmaps = %d\n", nmaps);
    printf ("nrpt = %d\n", nrpt);
    printf ("tmain = %d\n", tmain);
    printf ("Hsign = %c, Dsign = %c\n", Hsign, Dsign); 
    printf ("HA = %f, DEC = %f, latitude = %f\n", HA, DEC, lat);
    printf ("beam file name = %s\n", cen_name);
    printf ("pattern file name = %s\n", pat_name);

/* convert to radians                                            */

    HA = HA * 15 * PI / 180;
    DEC = DEC * PI /180;
    lat = lat * PI /180;

/*  destroy old output beamfile if it exists and create new one  */

    beams = fopen (beam_name,"w");
    fclose(beams);


/*                READ IN POSITIONS OF SEQUOIA BEAMS             */


    read_beams(cen_name);


/*                READ IN SHIFTS FOR MAPPING PATTERN             */


    num_point = read_pattern(pat_name);


/* determine starting parallactic angle                          */

    pangle = parallactic();


/*               INTEGRATE                                       */


/* integrate on the sky array.  Any sky pixels which overlap     */
/* with pixels in the sequoia beams (gaussian shaped) have their */
/* values incremented for each integration interval they overlap */


    for (maps=1; maps<=nmaps; maps++)
        {
        integrate(nrpt, tmain, num_point);
        }


/*                WRITE OUTPUT                                   */


/* open the output file for writing                              */

    if ( (out1 = fopen (out_name1,"wb")) == NULL)
        printf ("Creating file %s\n",out_name1);
    else
        printf ("Opening file %s\n",out_name1);

/* write the sky array as a binary image                         */

    fwrite(sky, sizeof(float), 600*600, out1);
    fclose(out1);


/*                 END OF PROGRAM                               */


    return;

}

/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */

void read_beams(char cen_name[25])
{

#include <stdio.h>
#include <math.h>
#include <stdlib.h>


/*                DECLARE FUNCTION VARIABLES                    */


     FILE *center;
     int count, num_beam = 32;

/* center = pointer to file containing beam centers for beams    */
/* cen_name = name of file pointed to by centers                 */
/* count = counter for reading beam center file                  */
/* num_beam = number of beams SEQUOIA has                        */
/* X0,Y0 = the central positions of a beam                  */


/*                          NOTE!!!                              */

/* THE INPUT BEAM FILE SHOULD SHOULD LIST THE ORIGINAL 16        */
/* SEQUOIA PIXELS FIRST, IN ORDER, THEN FOLLOW WITH THE NEW ONES */



/*                READ IN SEQUOIA BEAM POSITIONS                 */

/* open the file with beam positions                             */

    if ( (center = fopen (cen_name,"r")) == NULL)
       { printf ("This file does not exist. Quitting now.\n");
         exit(0);}

/* read the center position of each beam                         */

    for (count=0; count<=31; count++)
        {
        fscanf (center, "%d %d", &X0[count], &Y0[count]);
        if (count == 10) {
          xtemp=X0[10]; ytemp=Y0[10];
        }
        }
    for (count=0; count<=31; count++)
        {
        X0[count]=X0[count]-xtemp;
        Y0[count]=Y0[count]-ytemp;
        }

/* close the file with beam positions                            */

    fclose(center);


    return;
}


/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */

int read_pattern(char pat_name[25])
{

#include <stdio.h>
#include <math.h>
#include <stdlib.h>


/*                DECLARE FUNCTION VARIABLES                    */


     FILE *center;
     int num_point = 0;

/* center = pointer to file containing pattern for moving        */
/*           telescope                                           */
/* pat_name = name of file pointed to by centers                 */
/* num_point = number of pointings in the current pattern        */
/* xcen, ycen = the central positions of a beam                  */


/*                READ IN PATTERN FOR MOVING TELESCOPE           */


/* open the file with position shifts                            */

    if ( (center = fopen (pat_name,"r")) == NULL)
       { printf ("This file does not exist. Quitting now.\n");
         exit(0);}
    do
     {
/* read the x,y shift of each position                           */

      fscanf (center, "%d %d", &xshift[num_point], &yshift[num_point]);
      num_point++; 
     }
    while (!feof(center)); 

/* close the file with position shifts                           */

    fclose(center);

    num_point = num_point - 1;
      fprintf(stderr,"\num_point = %d", num_point);

    return (num_point);
}


/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */

double parallactic(void)
{

/* HA = hour angle                                               */
/* DEC = declination                                             */
/* lat = observer latitude                                       */

/* This function calculates the parallactic angle (angle between */
/* equatorial RA and DEC axes and the alt-azimuth axes.          */


#include <stdio.h>
#include <math.h>
#include <stdlib.h>


/*                DECLARE FUNCTION VARIABLES                     */

     double ypa, xpa, pa;

/* parallactic angle is the inverse tangent of the ratio of two  */
/* functions                                                     */


/*                CALCULATE PARALLACTIC ANGLE                    */

     ypa = sin(HA);
     xpa = cos(DEC)*tan(lat) - sin(DEC)*cos(HA);
     pa = atan2(ypa,xpa);

     return (pa);

}

/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */

void shifter (int pattern)
{

/* A function to shift SEQUOIA around in x and y  using xshift[] */
/* and yshift[]                                                  */

    int shift = 0;

/*                SHIFT BEAM POSITIONS FOR MAPPING               */

    fprintf(stderr,"\n Pattern %2d: %d %d %5.0f %5.0f", pattern, X0[10],Y0[10],
     xshift[pattern],yshift[pattern]);
         X1[10] = (float)X0[10] + xshift[pattern]*cos(pangle)+
                  yshift[pattern]*sin(pangle);
         Y1[10] = (float)Y0[10] - xshift[pattern]*sin(pangle)+
                  yshift[pattern]*cos(pangle);
    fprintf(stderr,"--->  %5.0f %5.0f",X1[10],Y1[10]);

    return;
}

/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */

void rotate (int pattern)
{

/* This function performs the rotation of the beam positions     */
/* relative to pixel 11, using the parallactic angle             */


    FILE *beams;
    char beam_name[] = "beamfile";
    int shift = 0;
    float xzero, yzero;

    beams = fopen (beam_name,"a");

    for (shift = 0; shift<= 31; shift++)
        {
 
/*                ROTATE BEAM PATTERN AROUND PIXEL 11            */

/* these lines set the center of the rotation to pixel 11        */

         xcen1[shift] = X1[10]+ (float)X0[shift] * cos(pangle) 
                         + (float)Y0[shift] * sin(pangle);

         ycen1[shift] = Y1[10]-(float)X0[shift] * sin(pangle) 
                         + (float)Y0[shift] * cos(pangle);
         fprintf (beams, "%f %f %d\n", xcen1[shift], ycen1[shift], pattern);
        }

    fclose(beams);
    
    return;

}

/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */

void integrate (int nrpt, int tmain, int num_point)
{

/* nrpt = number of repeats of tmain seconds in duration to do   */
/*        reading scan out                                       */
/* tmain = time spent on source                                  */
/* num_point = number of positions to observe to complete map    */


/* This function reads the position of the SEQUOIA beams from   */
/* a file and determines which pixels in the sky array overlap  */
/* with them.  It then increases the value of those "observed"  */
/* regions of sky by one unit (of integration time)             */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>


/*                DECLARE FUNCTION VARIABLES                     */

     int x=0, y=0, n_beam;
     int count, rpt, pattern, shift;
     int xdif = 0, ydif = 0;
     int pix = 0, Sigma = 7;
     float sigma = 6.79, part1=0.0, part2 = 0.0;
     float t_incr = 0.0;
     double dHA = 0.0; 

/* n_beam = number of beams SEQUOIA has                          */
/* x, y = pixels in a given beam (x -xcen)^2 + (y-ycen)^2 = r^2  */
/* count = counter for number of beams in SEQUOIA footprint      */
/* pattern = counter for the number of pointings observed        */
/* shift = counter for shifting xcen[] and ycen[]                */
/* rpt = counter for number of repeats on a given pointing       */
/* xdif, ydif = placeholders for differnces                      */
/* pix = distance of pixels in a beam from xcen, ycen            */
/* sigma = sigma of a gaussian beam with FWHM 45" (14m telescope)*/
/*         in units of pixels                                    */ 
/* t_incr = the amount of integration time added to a given pixel*/
/*          This is determined by a gaussian which is normalized */
/*          to tmain at the center of the beam                   */
/* pangle = parallactic angle                                    */
/* t_main = the amount of time spent on source each single       */
/*            integration                                        */


/*               STEP THROUGH EACH POSITION FROM PATTERN FILE    */

    dHA = ((float) tmain * 15 /3600) * PI / 180;

    for (pattern = 0; pattern < num_point; pattern++) 
        {


        printf ("starting pointing number= %i\n", pattern);


/* integrate within the beams for nrpt repeats, int time intervals */
/* equal to tmain                                                 */


        for (rpt = 1; rpt <= nrpt; rpt++)
           {

/* calculate the parallactic angle so the rotation of the source */
/* with respect to alt-az coordinates can be determined          */
/* the beam locations are rotated around pixel 11                */

           if (askrot == 'y')
              {
              pangle = parallactic ();

/*   increase the hour angle with each second of integration time  */

/*   2 * dHA is to take into account the time spent on reference   */
/*   for position switched observations, assuming tmain = tref     */


              HA = HA + 2* dHA;

              printf ("pangle =  %f, new HA = %f\n", pangle, HA); 
              }


           shifter(pattern);
           rotate(pattern);
           MakeImage(tmain);
       }
    }
}


/*                INTEGRATE ON SKY UNDER EACH BEAM               */
void MakeImage(tmain)
int tmain;
{

     int x=0, y=0, n_beam;
     int count, rpt, pattern, shift;
     int xdif = 0, ydif = 0;
     int pix = 0, Sigma = 7;
     float sigma = 6.79, part1=0.0, part2 = 0.0;
     float t_incr = 0.0,xc[32],yc[32];
     double dHA = 0.0; 

/* use a 2-D gaussian to increment the integration time count in */
/* array elements that are inside a circle of radius 3 sigma     */
/* (inside a SEQUIOA beam.   This is normalized to 1 second at   */
/* beam center. Do this once for each beam                       */
     
           for (count=0; count<=31; count++) {
             xc[count]=xcen1[count]+xtemp;
             yc[count]=ycen1[count]+ytemp;
           }

           for (count=0; count<=31; count++)
             {


             for (x=xc[count]-(3*Sigma); x<=xc[count]+(3*Sigma); x++)
               {
               for (y=yc[count]-(3*Sigma); y<=yc[count]+ (3*Sigma); y++) 
                 { 
                 xdif = x-xc[count];
                 ydif = y-yc[count];
                 pix = (xdif*xdif) + (ydif*ydif);

                 if (pix <= 9*Sigma*Sigma)
                    {

/*      calculating gaussian                                     */

                    part2 = ((float) pix/(sigma*sigma*2));
                    t_incr = exp(-part2);

/*      increment the pixels within the beam                     */

                    sky[y][x] = sky[y][x] + t_incr * tmain;
                    }
                 }
               } 
           
           }

/*                END OF FUNCTION                                */

     return;

}
