#include <stdio.h>
#include <stdlib.h>
#include <cpgplot.h>
#include <math.h>
#include "otf.h"

#define BLANK -5.0

FILE *fp;

void ReadHEADER();
void PrintHEADER();
void GetREFERENCE(int,float*,float*);
void GetDUMP(int,int,float*);
void WeightREF(char*,int,float*,float*,float*);
int index(int,int,int);

main(argc,argv)
int argc;
char *argv[];
{
 char WTmode[8],ch,title[32],comment[64];
 int i,ii,j,k,idmp,ndmp,id1,id2;
 int IF,IHORN;
 int just,axis;
 float *R1,*R2,*R,*M,*TA;
 float v1,v2,tamin,tamax,tr[6];


 if (argc < 5) {
  fprintf(stderr,"\n otfreduce  filename IF IHORN REFWeight(equal:biased) \n");
  exit(1);
 }

 if ((fp=fopen(argv[1],"r"))==NULL) {
  fprintf(stderr,"\n Can not open file %s\n", argv[1]);
  exit(1);
 }
 ReadHEADER();

 IF=atoi(argv[2]);
 if (IF <1 || IF > h.H.nifs) {
  fprintf(stderr,"\n IF selection out of range \n");
  exit(2);
 }
 IF=IF-1;
 IHORN=atoi(argv[3]);
 if (IHORN <1 || IHORN > h.H.nhorns) {
  fprintf(stderr,"\n PIXEL selection out of range \n");
  exit(2);
 }
 IHORN=IHORN-1;

 sscanf(argv[4],"%s",WTmode);

 if ( (R1 = (float *)malloc(sizeof(float)*h.H.nchan*h.H.nhorns))==NULL) {
   fprintf(stderr,"\n Can not allocate memory for REF 1  data array\n");
   exit(3);
 }
 if ( (R2 = (float *)malloc(sizeof(float)*h.H.nchan*h.H.nhorns))==NULL) {
   fprintf(stderr,"\n Can not allocate memory for REF 2  data array\n");
   exit(3);
 }
 if ( (R = (float *)malloc(sizeof(float)*h.H.nchan*h.H.nhorns))==NULL) {
   fprintf(stderr,"\n Can not allocate memory for REF  data array\n");
   exit(3);
 }
 if ( (M = (float *)malloc(sizeof(float)*h.H.nchan*h.H.nhorns))==NULL) {
   fprintf(stderr,"\n Can not allocate memory for MAIN data array\n");
   exit(3);
 }
 if ( (TA = (float *)malloc(sizeof(float)*h.H.nsample*h.H.nchan))==NULL) {
   fprintf(stderr,"\n Can not allocate memory for MAIN data array\n");
   exit(3);
 }


 fprintf(stderr,"\n Getting ref data");
 GetREFERENCE(IF,R1,R2);

/* Now start gathering ON data */
 tamin=1.0e+20; tamax=-1.0e+20;
 ndmp=h.H.nsample;
 for(idmp=1; idmp<ndmp; idmp++) {   /* MAIN data in idmp 1:ndumps-1 */

 fprintf(stderr,"\n Getting main data %d",idmp);
  GetDUMP(IF,idmp,M);
 fprintf(stderr,"\n Weighting ref data");
  WeightREF(WTmode,idmp,R1,R2,R);

 fprintf(stderr,"\n Subtracting ref from main");
  for(k=0; k<h.H.nchan; k++) {
    i=index(k,idmp,h.H.nchan);
    ii=index(k,IHORN,h.H.nchan);
    TA[i]=M[ii]-R[ii];
    if (TA[i] < tamin) tamin=TA[i];
    if (TA[i] > tamax) tamax=TA[i];

/*  for real: 
       TA[i]=BLANK;
       if (R[k] != 0.0) TA[i]=h.H.tsys[IF][IHORN]*(M[k]-R[k])/R[k];
*/
  }
 }


 id1=0; id2=0;
 tr[0]=h.H.xvel1[IF]; tr[1]=h.H.dveldin[IF]; tr[2]=0.0;
 tr[3]=1.0; tr[4]=0.0; tr[5]=1.0;
 just=0; axis=0;
 fprintf(stderr,"\n min/max=%g   %g ",tamin,tamax);
 /* Open graphics device. */
 id1=cpgopen("/xwin");
 if (id1 < 1) exit(1);

 v1=h.H.xvel1[IF]; v2=v1+(h.H.nchan-1.0)*h.H.dveldin[IF];
 cpgenv(v1,v2,1,ndmp,just,axis);
 sprintf(comment,"OTF Scan %d  IF %1d HORN %2d",h.H.iscan,IF+1,IHORN+1);
 cpglab(h.H.ctype1,"DUMP", comment);
 cpggray(TA,h.H.nchan,h.H.nsample,1,h.H.nchan,1,h.H.nsample,tamin,tamax,tr);

/* Close the graphics device */
  cpgslct(id1);
  cpgclos();

/* Free memory */
  free(R1);
  free(R2);
  free(R);
  free(M);
  free(TA);
}

void ReadHEADER()
{
int nmem;

  nmem=HEADERSIZE/4;
  fread(h.buf,sizeof(long),nmem, fp);
  PrintHEADER();
}

void PrintHEADER()
{
 int ihorn,jf,i;

 fprintf(stdout,"\n ");
 fprintf(stdout,"\n OTFDataREV %3.1f ",h.H.otfdatarev);
 fprintf(stdout,"\n Scan %5d   %s ",h.H.iscan, h.H.name);
 fprintf(stdout,"\n nhorns %2d ", h.H.nhorns);
 fprintf(stdout,"  nifs %1d ", h.H.nifs);
 fprintf(stdout,"  nchannels %4d ", h.H.nchan);
 fprintf(stdout,"\n nsample %3d ", h.H.nsample);
 fprintf(stdout,"   ndumps %3d ", h.H.ndumps);
 fprintf(stdout,"\n ctype1: %s ", h.H.ctype1);
 fprintf(stdout,"   ctype2: %s   ctype3: %s ", h.H.ctype2,h.H.ctype3);
 fprintf(stdout,"\n ");

 if (h.H.nifs == 2) fprintf(stdout,"\n     xvel1(if1)  dveldin(if1)   
   xvel1(if2) dveldin(ifs) \n ");
 if (h.H.nifs == 1) fprintf(stdout,"\n     xvel1(if1)  dveldin(if1) \n ");  
 for(jf=0; jf<h.H.nifs; jf++) 
  fprintf(stdout,"     %8.3f  %5.3f ",h.H.xvel1[jf],h.H.dveldin[jf]);

 fprintf(stdout,"\n ");
 fprintf(stdout,"\n           ");
 for(ihorn=0; ihorn<h.H.nhorns; ihorn++) fprintf(stdout,"  %2d  ",ihorn+1);
 for(jf=0; jf<h.H.nifs; jf++) {
  fprintf(stdout,"\n Tsys(IF_%1d)",jf+1);
  for(ihorn=0; ihorn<h.H.nhorns; ihorn++) fprintf(stdout,"%5d ",h.H.tsys[jf][ihorn]);
 }
     
 fprintf(stdout,"\n ");
 fprintf(stdout,"\n ");
}

void GetREFERENCE(IF,R1,R2)
int IF;
float *R1,*R2;
{
 int k;
 long offset,nmem;

 nmem=h.H.nhorns*h.H.nchan;
 offset=HEADERSIZE+sizeof(float)*IF*nmem;
 fseek(fp,offset,SEEK_SET);
 fread(R1,sizeof(float),nmem,fp); 

 offset=HEADERSIZE+sizeof(float)*nmem*(h.H.nifs*h.H.ndumps-1+IF);
 fseek(fp,offset,SEEK_SET);
 fread(R2,sizeof(float),nmem,fp); 

}
void GetDUMP(IF,idmp,M)
int IF,idmp;
float *M;
{
 int k;
 long offset,nmem;

 nmem=h.H.nhorns*h.H.nchan;
 offset=HEADERSIZE+sizeof(float)*nmem*(h.H.nifs*idmp+IF);
 fseek(fp,offset,SEEK_SET);
 fread(M,sizeof(float),nmem,fp); 
}

void WeightREF(WTmode,idmp,R1,R2,R)
char *WTmode;
float *R1,*R2,*R;
{
 int k;
 float wt1,wt2;

 if (!strncmp(WTmode,"equal",2)) {
  wt1=0.5;
  wt2=0.5;
 }
 else if (!strncmp(WTmode,"biased",2)) {
  wt1=1.0-(float)idmp/h.H.nsample;
  wt2=1.0-wt1;
 }
 for(k=0; k<h.H.nhorns*h.H.nchan; k++) 
  R[k]= wt1*R1[k]+wt2*R2[k];
}


int index(i,j,NX)
int i,j,NX;
{
  return(i+j*NX);
}
