#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "fitsio.h"

/*
#if defined(linux) || defined(LINUX)
#define main	MAIN__
#endif
*/

int status,nhorn=16;
long naxes[2];
long nelements,fpixel=1,naxis=2;
float dnu=50.0e+03;
float TDUMP=0.25;
float TDUMPMIN=0.1,MAXTIMEREF=120.0,RateMAX=100.;
float DSUM;
float eta=2.71;
float N_c=16.0;
float tslew2ref=10.0;
float tcal=15.0;
float xa[32],ya[32],tsys[32],T[256][256],COL[256],ROW[256];
float XE,XS,YE,YS,xs,xe,r,DX,dx,tscan,ttotal;
float crval1,crval2;
float SCAN_ANGLE,LX,LY,XOFF,YOFF,THETA_N,RMS,ROWSTEP; 
double WT[256][256];
double weights[256];
double DEC=30.000; 
double LAT=0.73987416;   /* FCRAO latitude in radians */
double PI = 3.141592653589793;

void FindRampSize(float,float*,float*);
void MakeWeights(float);
double parallactic(double);
double jinc(double);
double bessj1(double);
void convolve(int,float,float);
void printerror(int);
int index(int,int,int);
double GetWeight(float);

main(argc,argv)
int argc;
char *argv[];
{
 FILE *fp,*fp1,*fp2;
 fitsfile *fptr;
 int ier,i,icol,irow,ix,iy;
 double pa,ha,HA,HAS;
 float tsysAV,x1,y1;
 float targetrms, RAMPX[2],RAMPY[2];
 float s1,s0,xmin,xmax,ymin,ymax,scan_angle;
 char MAPCOORD[8];
 float nos,R,tcell,tref,nsample;
 float *sigma;
 float t,X,Y,x,y;
 float xref,yref;
 int NRPO,NRPC;
 float x0,y0;
  
/* Get input parameters */
  if (argc < 13) {
    fprintf(stderr,"\n OTF PARAMETERS: ");
    fprintf(stderr,"\n SCAN COORDINATE (az,el,ra,dec,l,b) ");
    fprintf(stderr,"\n SCAN ANGLE");
    fprintf(stderr,"\n XOFF ");
    fprintf(stderr,"\n YOFF");
    fprintf(stderr,"\n LX (arcminutes)");
    fprintf(stderr,"\n LY (arcminutes)");
    fprintf(stderr,"\n NRPO rows per off (integer)");
    fprintf(stderr,"\n NRPC rows per cal (integer)");
    fprintf(stderr,"\n Output sampling cell (arcsec) ");
    fprintf(stderr,"\n Target RMS (K)");
    fprintf(stderr,"\n ROWSTEP (beams) ");
    fprintf(stderr,"\n Starting Hour Angle (hrs)");
    fprintf(stderr,"\n Exiting\n");
    exit(0);
  } 
  sscanf(argv[1],"%s",MAPCOORD);
  SCAN_ANGLE=atof(argv[2]);
  XOFF=atof(argv[3]);
  YOFF=atof(argv[4]);
  LX=atof(argv[5]);
  LY=atof(argv[6]);
  NRPO=atoi(argv[7]);
  NRPC=atoi(argv[8]);
  THETA_N=atof(argv[9]);
  RMS=atof(argv[10]);
  ROWSTEP=atof(argv[11]);
  HAS=atof(argv[12]);

  scan_angle=SCAN_ANGLE*PI/180.;
/* SEQUOIA positions relative to pixel 11 */
  xa[0]=-2.952; xa[1]=-1.476; xa[2]=0.0; xa[3]=1.476;
  xa[4]=-2.952; xa[5]=-1.476; xa[6]=0.0; xa[7]=1.476;
  xa[8]=-2.952; xa[9]=-1.476; xa[10]=0.0; xa[11]=1.476;
  xa[12]=-2.952; xa[13]=-1.476; xa[14]=0.0; xa[15]=1.476;
  ya[0]=2.952; ya[1]=2.952; ya[2]=2.952; ya[3]=2.952;
  ya[4]=1.476; ya[5]=1.476; ya[6]=1.476; ya[7]=1.476;
  ya[8]=0.0; ya[9]=0.0; ya[10]=0.0; ya[11]=0.0;
  ya[12]=-1.476; ya[13]=-1.476; ya[14]=-1.476; ya[15]=-1.476;
/* SEQUOIA TSYS*/
  tsys[0]=300.; tsys[1]=307; tsys[2]=286; tsys[3]=290;
  tsys[4]=307; tsys[5]=309; tsys[6]=317; tsys[7]=304;
  tsys[8]=318; tsys[9]=341; tsys[10]=272; tsys[11]=325;
  tsys[12]=348; tsys[13]=282; tsys[14]=281; tsys[15]=329;

  if ( (fp = fopen("otf.wip", "w")) == NULL) {
   printf("\n Can not open file otf.wip\n");
   exit(2);
  }
  if ( (fp1 = fopen("pos.dat", "w")) == NULL) {
   printf("\n Can not open file pos.dat\n");
   exit(2);
  }
  if ( (fp2 = fopen("cuts.dat", "w")) == NULL) {
   printf("\n Can not open file cuts.dat\n");
   exit(2);
  }

/* Make lookup table for convolving kernel */
   MakeWeights(THETA_N);

  HA=HAS;
  ha=15.0*HA*PI/180.;
  pa=parallactic(ha);
  FindRampSize(pa,RAMPX,RAMPY);
  
  s1=0.0; s0=0.0;
  for(i=0; i<nhorn; i++) {
    s1+=tsys[i];
    s0+=1.0;
  }
  tsysAV=s1/s0;

/* Make some guess as to the beam redundancy given ROWSTEP > 0.5 */
   if (ROWSTEP > 0.5 && ROWSTEP <= 8) N_c=8.0/ROWSTEP;
   if (ROWSTEP <= 0.5) N_c=16;
   if (ROWSTEP > 8) N_c=1;

   targetrms=RMS;
   nos=2.0;
   TDUMP=pow(tsysAV/RMS,2.)/(N_c*eta)/(nos*dnu);
   if (TDUMP < TDUMPMIN) {
     fprintf(stderr,"\n Cannot not run the correlators faster than 10 Hz");
     fprintf(stderr,"\n Setting TDUMP=0.1 secs"); 
     fprintf(stderr,"\n New Target RMS = %6.2f K", RMS); 
     TDUMP=0.1;
     targetrms=RMS;
     RMS=tsysAV/sqrt(TDUMP*N_c*eta*nos*dnu);
   }
   tcell=pow(tsysAV/RMS,2.)/N_c/dnu;
   nsample=60.0*NRPO*nos*(LX+RAMPX[0]+RAMPX[1])/(THETA_N);
   if (nsample*TDUMP > MAXTIMEREF) {
     fprintf(stderr,"\n Time between OFF measurements exceeds recommended maximum%4.0f secs",MAXTIMEREF);
     fprintf(stderr,"\n Increase target rms and make repeated maps\n");
     exit(1);
   }
   tref=0.5*TDUMP*sqrt(nsample);
   R=THETA_N/(60.0*nos*TDUMP);  /* antenna rate in arcmin/sec */
   if (R > RateMAX) {
     fprintf(stderr,"\n Adjusting over-sampling rate");
     do {
       nos+=0.5;
       R=THETA_N/(60.0*nos*TDUMP);  /* antenna rate in arcmin/sec */
       TDUMP=pow(tsysAV/RMS,2.)/(N_c*eta)/(nos*dnu);
       if (R < RateMAX) break;
     } while(nos <=4);
   }
   if (nos > 4) {
     fprintf(stderr,"\n Sampling rate too large -- exitting\n");
     exit(1);
   }

   fprintf(stderr,"\n Parallactic Angle=%4.0f ",pa*180/PI);
   fprintf(stderr,"\n Nc=%4.1f ",N_c);
   fprintf(stderr,"\n nos=%6.2f ",nos);
   fprintf(stderr,"\n DUMP RATE=%4.2f HZ",1.0/TDUMP);
   fprintf(stderr,"\n R=%6.2f arcmin/sec",R);
   fprintf(stderr,"\n <TSYS>=%6.0f K",tsysAV);
   fprintf(stderr,"\n tcell=%6.3f sec",tcell);
   fprintf(stderr,"\n nsample=%6.0f ",nsample);
   fprintf(stderr,"\n tref=%6.2f sec",tref);
   fprintf(stderr,"\n Estimated time between OFF measurements=%6.2f sec",2.*tslew2ref+nsample*TDUMP);
   fprintf(stderr,"\n Type any key to continue: ");
  
   getchar();
   
   YS=YOFF*LY+LY/2.0+RAMPY[0]; YE=YOFF*LY-LY/2.0-RAMPY[1];
   XS=XOFF*LX+LX/2.0+RAMPX[0]; XE=XOFF*LX-LX/2.0-RAMPX[1];
   crval1=XS+RAMPX[0];
   crval2=YE-RAMPY[1];

   naxes[0]=60.0*(LX+2.0*fabs(RAMPX[0]+RAMPX[1]))/THETA_N;
   naxes[1]=60.0*(LY+2.0*fabs(RAMPY[0]+RAMPY[1]))/THETA_N;
   nelements=naxes[0]*naxes[1];
  fprintf(fp,"\n fill 2");
  fprintf(fp,"\n viewport .15 .9 .5 .9");
  fprintf(fp,"\n pal 1");
  fprintf(fp,"\n symbol -1");
  fprintf(fp,"\n panel 2 1 1");
  fprintf(fp,"\n winadj %6.1f %6.1f %6.1f %6.1f", 1.4*XS,1.3*XE,1.3*YE,1.4*YS);
  fprintf(fp,"\n limi  %6.1f %6.1f %6.1f %6.1f", 1.4*XS,1.3*XE,1.3*YE,1.4*YS);
  fprintf(fp,"\n box");
  fprintf(fp,"\n input pos.dat");

   if (naxes[0] > 256 || naxes[1] > 256) {
     fprintf(stderr,"\n Output image exceeds limits (256x256)");
     fprintf(stderr,"\n Exiting");
     exit(1);
   }
   if ( (sigma = (float *)malloc(sizeof(float)*nelements))==NULL) {
     fprintf(stderr,"\n Can not allocate memory for data array\n");
     exit(3);
   }

   if (fits_create_file(&fptr, "!rms.fits", &status)) 
         printerror( status );    
   if (fits_create_img(fptr,FLOAT_IMG,naxis,naxes,&status))
         printerror( status );    

/* Initialize T array */
   for(iy=0; iy<naxes[1]; iy++) {
     for(ix=0; ix<naxes[0]; ix++) {
      T[iy][ix]=0.0;
      WT[iy][ix]=0.0;
     }
   }

   y0=YS;
   irow=0;
   ttotal=0.0;
   DSUM=0.0;
   while(y0 > YE) {
     Y=y0;
     if (fmod((double)irow,(double)NRPC)==0.0) {
       fprintf(stderr,"\n CAL measurement");
       X=xref; Y=yref; ttotal+=tslew2ref; HA+=tslew2ref/3600.;
       ttotal+=tcal; HA+=tcal/3600.;
     }
     if (fmod((double)irow,(double)NRPO)==0.0) {
       fprintf(stderr,"\n REF measurement");
       if (X!=xref || Y!=yref) {
         ttotal+=tslew2ref; HA+=tslew2ref/3600.;
       }
       X=xref; Y=yref; 
       ttotal+=tref;
       HA+=tref/3600.;
       ttotal+=tslew2ref; 
       HA+=tslew2ref/3600.;
       DSUM+=nhorn*2048*4.0;
     }

     irow++;
     fprintf(stderr,"\n Row: %4d  Hour Angle %6.2f hr PA = %4.0f", irow,HA,pa*180/PI);
     if (fmod((double)irow,2.0)==1.0) {
       xs=XS; xe=XE; r=R;
     }
     if (fmod((double)irow,2.0)==0.0) {
       xs=XE; xe=XS; r=-R;
     }
     DX=fabs(xe-xs);
     dx=0.0;
     x0=xs; 
     tscan=0.0;
     icol=0; 
     while(dx <= DX) {
      fprintf(stderr,".");
      icol++;
      DSUM+=nhorn*2048*4.0;
      tscan+=TDUMP;
      x0-=r*TDUMP;
      X=x0*cos(scan_angle)+y0*sin(scan_angle);
      Y=y0*cos(scan_angle)-x0*sin(scan_angle);
      dx+=fabs(R*TDUMP);
/* Update pa  */
      HA+=TDUMP/3600.;
      ha=15.0*HA*PI/180.;
      pa=parallactic(ha);
      for(i=0; i<nhorn; i++) {
       x1=X-xa[i]*cos(pa)+ya[i]*sin(pa);
       y1=Y+xa[i]*sin(pa)+ya[i]*cos(pa);
       if (irow==1 && icol==1) {
        fprintf(fp1, "\n color 2");
        fprintf(fp1,"\n move %5.1f %5.1f", x1,y1);
        fprintf(fp1,"\n arc .738 .738");
        fprintf(fp1, "\n color 1");
       }
       fprintf(fp1,"\n move %5.1f %5.1f", x1,y1);
       fprintf(fp1,"\n dot");

/*     convolve this position into output image */
       convolve(i,x1,y1);

      }  /* horn loop */
     }   /* scan loop */
     y0-=ROWSTEP*2.0*THETA_N/60.;   /*   y0-=THETA_N/60.;   */
     ttotal+=tscan;
   }   /* MAP is finished */

/* Be sure to end with a OFF measurement  */
   if (X!=xref || Y!=yref) {
     ttotal+=tslew2ref; HA+=tslew2ref/3600.;
     fprintf(stderr,"\n REF measurement");
     X=xref; Y=yref; 
     ttotal+=tref; HA+=tref/3600.;
     ttotal+=tslew2ref; HA+=tslew2ref/3600.;
     DSUM+=nhorn*2048*4.0;
   }


   for(iy=0; iy<naxes[1]; iy++) {
    for(ix=0; ix<naxes[0]; ix++) {
      i=index(ix,iy,naxes[0]);
      sigma[i]=0.0;
      if (T[iy][ix]!=0.0 && WT[iy][ix]!=0.0) {
        sigma[i]=1.0/sqrt(dnu*T[iy][ix]);
      }
    }
   }
   for(ix=0; ix<naxes[0]; ix++) {
    i=index(ix,(int)naxes[1]/2,naxes[0]);
    ROW[ix]=sigma[i];
   }
   for(iy=0; iy<naxes[1]; iy++) {
    i=index((int)naxes[0]/2,iy,naxes[0]);
    COL[iy]=sigma[i];
   }
   for(ix=0; ix<naxes[0]; ix++)
    fprintf(fp2,"\n %d %6.3f %6.3f", ix,ROW[ix], COL[ix]);

   fpixel=1;
    if (fits_write_img(fptr, TFLOAT,fpixel,nelements,&sigma[0], &status))
     printerror( status );    
   if (fits_close_file(fptr,&status)) 
         printerror( status );    

#
fprintf(fp,"\n panel 2 1 2");
fprintf(fp,"\n image rms.fits ");
fprintf(fp,"\n transfer %7.3f %7.3f 0.0 %7.3f 0.0 %7.3f  ",
       crval1+THETA_N/60.,-THETA_N/60.,crval2,THETA_N/60.);
  fprintf(fp,"\n winadj %6.1f %6.1f %6.1f %6.1f", 1.4*XS,1.3*XE,1.3*YE,1.4*YS);
  fprintf(fp,"\n limi  %6.1f %6.1f %6.1f %6.1f", 1.4*XS,1.3*XE,1.3*YE,1.4*YS);
fprintf(fp,"\n halftone %6.2f %6.2f",0.5*RMS,1.5*RMS);
fprintf(fp,"\n box");
fprintf(fp,"\n wedge T 1 3 ");
fprintf(fp,"\n color 7");
fprintf(fp,"\n rect %6.2f %6.2f %6.2f %6.2f", LX/2.,-LX/2.,-LY/2.,LY/2.);
fprintf(fp,"\n color 1");
fprintf(fp,"\n mtext B 3.5 .5 .5 X");
fprintf(fp,"\n mtext R 3.5 .5 .5 Y");
#
  fprintf(fp,"\n viewport .3 .7 .15 .4");
  fprintf(fp,"\n data cuts.dat ");
  fprintf(fp,"\n xcol 1 ");
  fprintf(fp,"\n ycol 2 ");
  fprintf(fp,"\n limi 1 %d %6.2f %6.2f ",naxes[0],0.5*RMS,2.0*RMS);
  fprintf(fp,"\n box");
fprintf(fp,"\n mtext B 3.5 .5 .5 pixel");
fprintf(fp,"\n mtext L 3.5 .5 .5 RMS");
  fprintf(fp,"\n color 2");
  fprintf(fp,"\n connect");
  fprintf(fp,"\n color 4");
  fprintf(fp,"\n ycol 3");
  fprintf(fp,"\n connect");
  fprintf(fp,"\n color 7");
  fprintf(fp,"\n lstyle 4");
  fprintf(fp,"\n move 0 %6.3f", targetrms);
  fprintf(fp,"\n draw %d %6.3f",naxes[0], targetrms);

   fclose(fp);
   fclose(fp1);
   fclose(fp2);
   free(sigma);
   fprintf(stderr,"\n\n ******************************************"); 
   fprintf(stderr,"\n TOTAL TIME        :%6.1f minutes",ttotal/60.);
   fprintf(stderr,"\n RAW DATA COLLECTED:%5.1f Mbytes ",DSUM/1.0e+06); 
   fprintf(stderr,"\n REGRIDDED DATA    :%5.1f Mbytes ",
              (float)nelements*4.0*2048./1.0e+06); 
   fprintf(stderr,"\n ******************************************\n\n"); 
}

void convolve(ihorn,x1,y1)
int ihorn;
float x1,y1;
{
int ix,iy;
float arg,deltax,deltay,x,y,radius;
double wt;

 for(iy=0; iy<naxes[1]; iy++) {
  y=crval2+iy*THETA_N/60.;
  deltay=60.0*fabs(y-y1);
  for(ix=0; ix<naxes[0]; ix++) {
   x=crval1-ix*THETA_N/60.;
   deltax=60.0*fabs(x-x1);
   radius=sqrt(pow(deltax,2.0)+pow(deltay,2.0));
   if (radius < 8.0*THETA_N) {   
    arg=radius/(2.0*THETA_N);
    wt=GetWeight(arg); 
    T[iy][ix]+=(float)wt*TDUMP/pow(tsys[ihorn],2.);
    WT[iy][ix]+=wt;
   }
  }
 }
}
double parallactic(HA)
double HA;
{
  double ypa, xpa, pa;

  ypa = sin(HA);
  xpa = cos(DEC)*tan(LAT) - sin(DEC)*cos(HA);
  pa = atan2(ypa,xpa);

  return (pa);
}

void printerror( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
    {
       fits_report_error(stderr, status); /* print error report */

       exit( status );    /* terminate the program, returning error status */
    }
    return;
}

double jinc(x)
double x;
{
  double X,Y;
  double J;

  if (x!=0.0) {
    J=2.0*bessj1(4.44*x)/(4.44*x);
  }
  else {
    J=1.0;
  }
  return(J);
}

double bessj1(x)
double x;
{
        double ax,z;
        double xx,y,ans,ans1,ans2;

        if ((ax=fabs(x)) < 8.0) {
                y=x*x;
                ans1=x*(72362614232.0+y*(-7895059235.0+y*(242396853.1
                        +y*(-2972611.439+y*(15704.48260+y*(-30.16036606))))));
                ans2=144725228442.0+y*(2300535178.0+y*(18583304.74
                        +y*(99447.43394+y*(376.9991397+y*1.0))));
                ans=ans1/ans2;
        } else {
                z=8.0/ax;
                y=z*z;
                xx=ax-2.356194491;
                ans1=1.0+y*(0.183105e-2+y*(-0.3516396496e-4
                        +y*(0.2457520174e-5+y*(-0.240337019e-6))));
                ans2=0.04687499995+y*(-0.2002690873e-3
                        +y*(0.8449199096e-5+y*(-0.88228987e-6
                        +y*0.105787412e-6)));
                ans=sqrt(0.636619772/ax)*(cos(xx)*ans1-z*sin(xx)*ans2);
                if (x < 0.0) ans = -ans;
        }
        return ans;
}

int index(i,j,NX)
int i,j;
{
  return(i+j*NX);
}
  
void FindRampSize(pa,RAMPX,RAMPY)
float pa,*RAMPX,*RAMPY;
{
  int i;
  float x1,y1,xmin,xmax,ymin,ymax;
  
/* Find the extreme, rotated positions within the array at this angle */
  xmin=1.0e+20; xmax=-1.0e+20; ymin=1.0e+20; ymax=-1.0e+20;
  for(i=0; i<nhorn; i++) {
    x1=-xa[i]*cos(pa)+ya[i]*sin(pa);
    if (x1 < xmin) xmin=x1;
    if (x1 > xmax) xmax=x1;

    y1=xa[i]*sin(pa)+ya[i]*cos(pa);
    if (y1 < ymin) ymin=y1;
    if (y1 > ymax) ymax=y1;
  }
/* Set default ramp offset */
   RAMPX[0]=fabs(xmin);
   RAMPX[1]=fabs(xmax);
   RAMPY[0]=fabs(ymin);
   RAMPY[1]=fabs(ymax);
}

void MakeWeights(bw)
float bw;
{
int i;
double x,y,y2;
double y1,y3,y4;

  for(i=0; i<256; i++) {
    x=-5.0+0.0390625*i;
    weights[i]=jinc(x);      /* jinc  */
  }
}

double GetWeight(arg)
float arg;
{
 int is,ie;
 float rxs,rxe;
 double w;

 is=(arg-(-5.0))/0.0390625;
 ie=is+1;
 rxs=-5.0+0.0390625*is;
 rxe=-5.0+0.0390625*ie;
 if (is<0 || is >254) {
   fprintf(stderr,"\n Weighting arguments out of range\n");
   exit(2);
 }
 w=weights[is]+(arg-rxs)*((weights[ie]-weights[is])/(rxe-rxs));
 return(w);
}
