#ifndef __OTF_H_
#define __OTF_H_

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <cpgplot.h>
#include <errno.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/param.h>
#include <sys/times.h>
#include <unistd.h>
#include <libgen.h>


#include "fitsio.h"
#include "nrutil.h"

extern int errno;

#ifndef X86 
#define FLOATPT "IEEE"
#else
#define FLOATPT "INTL"
#endif

#ifndef M_PIl
# define M_PIl          3.1415926535897932384626433832795029L  /* pi */
#endif

#define OTFREV 2.2
#define HEADERSIZE 4096
#define POSITIONHEADERSIZE 12288
#define XHEADERSIZE 5120
#define MAXFILES 5000
#define MAXCHAR 128
#define MAXSCANSIZE 1024
#define MAXHORNS 32
#define MAXDIM 350
#define MAXWIND 5
#define MAXORDER 5
#define MAXWEIGHT 0.25
#define NAN 1.0e+35;
#define RMAX 3.0
#define PI 3.1415926535;

#define TM_YEAR_BASE 1900
#define DDUMMYS   19
#define RDUMMYS    9
#define IDUMMYS   72
#define MC_DDUMS  17
#define MC_RDUMS  49
#define MC_IDUMS  92
 
/** structure for the OTF data file header
 */
struct OTFHeaderStruct
{
   /* 128 bytes */
   char otflab[16],floatopt[8],name[16],bunit[16],vlabel[16],
     xlabel[16],ylabel[16],observer[8],cdum[16]; 
   /* 32*8=256 bytes */
   double utd,f0,fsky,azs,els,XCEN,YCEN,xvel1,xfr1,dveldin,dfrdin,fr1,fr2,
     ddum[DDUMMYS]; 
   /* 32*4=128 bytes */
   float tsid,utc,targetrms,tdump,tref,telrate,epoch,bw,vlsr,vclsr,vcsun,
     NOS,tau,h2omm,pamb,tamb,rotang,
     XSIZE,YSIZE,XOFFMAP,YOFFMAP,XRAMP,YRAMP,rdum[RDUMMYS];        
   /* 256*2=512 bytes */
   short int iscan,nhorns,nchan,nsample,nblock,blksize,telcoord,kx,ky,klo,khm,
	 harm,krecv,khorn,caltype,calscan,cfact,etab,kmap,raoff,decoff,fsrate,stype,
	 iconf,tsys[MAXHORNS],gain[MAXHORNS], 
     DXHORNS[MAXHORNS],DYHORNS[MAXHORNS],hornID[MAXHORNS],idum[IDUMMYS]; 
};
/** type for OTF data file header struct
 */
typedef struct OTFHeaderStruct OTFHeader_t; 
OTFHeader_t H;

/** structure for the OTF position array 
*/
struct OTFPositionStruct
{
   /* 12288 bytes */ 
   float ANGLE[MAXSCANSIZE], XOFF[MAXSCANSIZE],YOFF[MAXSCANSIZE];      
};
/** type for OTF data file position struct
 */
typedef struct OTFPositionStruct OTFPositions_t; 
OTFPositions_t OTFPOS;

union XOTFHEADER {
 long ebuf[1280];   
 struct XHEADER
 {
  long MASK[MAXSCANSIZE];                           /* 4096 bytes */
  short int iscan,order,w1[MAXWIND],w2[MAXWIND],nwind,modified,idum[498]; /* 1024 bytes */
 } XH;
} xotf;

struct STATISTICS {
  float s0[MAXFILES][MAXHORNS],s1[MAXFILES][MAXHORNS],s2[MAXFILES][MAXHORNS],
      ave[MAXFILES][MAXHORNS],sd[MAXFILES][MAXHORNS],
      file_ave[MAXFILES],file_sd[MAXFILES],horn_ave[MAXHORNS],horn_sd[MAXHORNS],
      AVE,SD;
} rmsstat;


void ParseArguments(int,char **,char*,char*);
int ReadList(char*);
int GetXYLimits(float*,float*,float*,float*);
int ReadHEADER();
void PrintHEADER();
int SetFITSHEADERS(float,float,float,float);
void CreateArray(long,float *);
int ReGridData(char *);
void GetREFERENCE(int,float *);
void GetDUMP(int,float *);
void WeightREF(char *,int,float *,float *,float *);
void MakeWeights();
float GetRMS(float*,short int*);
double jinc(double);
double bessj1(double);
void convolve(int,int,float *,float *,
             float *, float *,float *,float *);
void printerror(int);
void PrintHelp();
long Getindex(int,int,int);
double GetWeight(float);
void ClassHEAD();
void ClassHIST();
int User_function(char*);

/* Gobal Variables */
FILE *otffp,*otffp1;
fitsfile *fptr;
char **infile,WTmode[8],calfile[64];
char ctype1[16],ctype2[16],ctype3[16];
short int flatfield,base,rmsweight,tsysweight,include_margin;
short noise_weight;
short int ORDER,NWIND,order,nwind,wind_type;
float W1[MAXWIND],W2[MAXWIND],w1[MAXWIND],w2[MAXWIND];
float *XPOS[MAXHORNS],*YPOS[MAXHORNS];
float rms[MAXHORNS][MAXSCANSIZE];
int IF,out_type,kopt;
int id1,id2;
int nfiles,status;
long fpixel,nelements,naxes[3]; 
long naxis;
long CmdLineMASK;
float rmsthreshold;
float crval1,crval2,crval3,crpix1,crpix2,crpix3,
      cdelt1,cdelt2,cdelt3;
float weights[256];
float THETA_N;
int nfound; /*class obs. number*/
char linename[12]; /*class linename*/
char rawdir[MAXPATHLEN];   /*directory for rawfiles */
int *indx;
double *tmax;
double **a,*bb,**c;

/*Timing stuff */
struct tms StartTime;
struct tms EndTime;
time_t STime, ETime;

#define RAD2DEG 180.0/M_PI
#define DEG2RAD M_PI/180.0

#endif   /*ifndef __OTF_H_ */

