#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "fitsio.h"
#include "otf.h"

#define BLANK 0.0
#define RMAX 3.0

FILE *p,*fp;

void ReadHEADER();
void PrintHEADER();
void GetXYLimits(float*,float*,float*,float*);
void GetREFERENCE(int,int,float*);
void GetDUMP(int,int,float*);
void WeightREF(char*,int,float*,float*,float*);
void MakeWeights();
double jinc(double);
double bessj1(double);
void convolve(float,float,float *);
void printerror(int);
int index(int,int,int);
double GetWeight(float);

char infile[64][64],outtype,outfile[64],WTmode[8];
int IF;
int nfiles,status;
long fpixel,nelements,naxis=3,naxes[3]; 
long ICUR;
float crval1,crval2,crval3,crpix1,crpix2,crpix3,
      cdelt1,cdelt2,cdelt3;
float weights[256];
float *WT,*T;
float THETA_N=23.0;

main(argc,argv)
int argc;
char *argv[];
{
 fitsfile *fptr;
 char otfconfigfile;
 int i,ii,ih,idmp,j,ix,iy,k;
 int NX;
 long jj;
 float xpos,ypos;
 float *R1,*R2,*R,*M,*Z;
 float xmin,xmax,ymin,ymax;

  
/* Get input parameters from config file */
/*
  sscanf(argv[1],"%s",otfconfig);
  if ((fp=fopen(otfconfigfile,"r")==NULL) {
    fprintf(stderr,"\n Can not open OTF configuration file %s\n", otfconfig);
    exit(0);
  }
  ReadConfigFile(otfconfig,&otfinput);
*/
  strcpy(&infile[0][0],"cepheus.2001.otf");
  strcpy(&infile[1][0],"cepheus.2002.otf");
  strcpy(&infile[2][0],"cepheus.2003.otf");
  strcpy(&infile[3][0],"cepheus.2004.otf");
  strcpy(&infile[4][0],"cepheus.2005.otf");
  strcpy(&infile[5][0],"cepheus.2006.otf");
  nfiles=6;
  IF=1; IF=IF-1;
  strcpy(outfile,"!ta.fits");
  strcpy(WTmode,"equal");

  xmin=1.0e+30;  xmax=-1.0e+30;  ymin=1.0e+30; ymax=-1.0e+30;
  for(i=0; i<nfiles; i++) {
   if ((fp=fopen(&infile[i][0],"r"))==NULL) {
     fprintf(stderr,"\n Can not open file %s\n", infile[i][0]);
     exit(1);
   }
   GetXYLimits(&xmin,&xmax,&ymin,&ymax);
   fclose(fp);
  }
  fprintf(stderr,"\n Position Limits: %f %f %f %f", xmin,xmax,ymin,ymax);
  naxes[0]=h.H.nchan;
  naxes[1]=(xmax-xmin)/(THETA_N/60.0)+1; 
  naxes[2]=(ymax-ymin)/(THETA_N/60.0)+1; 
  crval1=h.H.xvel1[IF];  cdelt1=h.H.dveldin[IF];  crpix1=1.0;
  crval2=0.0;  crval3=0.0;
  cdelt2=-THETA_N/3600.0;  cdelt3=THETA_N/3600.;
  crpix2=60.0*xmax/THETA_N;   crpix3=60.0*fabs(ymin)/THETA_N;

  fprintf(stderr,"\n Output Cube dimensions: %3d x %3d x %3d", naxes[0],naxes[1],naxes[2]);
  fprintf(stderr,"\n crval1 = %6.3f", crval1);
  fprintf(stderr,"\n cdelt1 = %6.3f", cdelt1);
  fprintf(stderr,"\n crpix1 = %6.3f", crpix1);
  fprintf(stderr,"\n crval2 = %6.3f", crval2);
  fprintf(stderr,"\n cdelt2 = %6.3f", cdelt2);
  fprintf(stderr,"\n crpix2 = %6.3f", crpix2);
  fprintf(stderr,"\n crval3 = %6.3f", crval3);
  fprintf(stderr,"\n cdelt3 = %6.3f", cdelt3);
  fprintf(stderr,"\n crpix3 = %6.3f", crpix3);

  if (naxes[1] > 256 || naxes[2] > 256) {
    fprintf(stderr,"\n Output image exceeds limits (256x256)");
    fprintf(stderr,"\n Exiting");
    exit(1);
  }
  if ( (WT = (float *)malloc(sizeof(float)*naxes[1]*naxes[2]))==NULL) {
    fprintf(stderr,"\n Can not allocate memory for Z data array\n");
    exit(3);
  }
  if ( (T = (float *)malloc(sizeof(float)*naxes[0]*naxes[1]*naxes[2]))==NULL) {
    fprintf(stderr,"\n Can not allocate memory for Z data array\n");
    exit(3);
  }
  if ( (R1 = (float *)malloc(sizeof(float)*h.H.nchan*h.H.nhorns))==NULL) {
    fprintf(stderr,"\n Can not allocate memory for REF 1  data array\n");
    exit(3);
  }
  if ( (R2 = (float *)malloc(sizeof(float)*h.H.nchan*h.H.nhorns))==NULL) {
    fprintf(stderr,"\n Can not allocate memory for REF 2  data array\n");
    exit(3);
  }
  if ( (R = (float *)malloc(sizeof(float)*h.H.nchan*h.H.nhorns))==NULL) {
    fprintf(stderr,"\n Can not allocate memory for REF  data array\n");
    exit(3);
  }
  if ( (M = (float *)malloc(sizeof(float)*h.H.nchan*h.H.nhorns))==NULL) {
    fprintf(stderr,"\n Can not allocate memory for MAIN data array\n");
    exit(3);
  }
  if ( (Z = (float *)malloc(sizeof(float)*h.H.nchan))==NULL) {
    fprintf(stderr,"\n Can not allocate memory for Z data array\n");
    exit(3);
  }

/* Initialize WT, T arrays */
   for(iy=0; iy<naxes[2]; iy++) {
     for(ix=0; ix<naxes[1]; ix++) {
      ii=index(ix,iy,naxes[1]);
      WT[ii]=0.0;
      jj=ii*naxes[0];
      for(k=0; k<naxes[0]; k++) T[jj+k]=0.0;
     }
   }
/* Make lookup table for convolving kernel */
   MakeWeights();

   for(i=0; i<nfiles; i++) {
    fprintf(stderr,"\n Processing: %s",&infile[i][0]);
    if ((fp=fopen(&infile[i][0],"r"))==NULL) {
      fprintf(stderr,"\n Can not open file %s\n", infile[i][0]);
      exit(1);
    }

    ReadHEADER();
    GetREFERENCE(IF,1,R1);
    GetREFERENCE(IF,2,R2);

    for(idmp=0; idmp<h.H.nsample; idmp++) {
      for(ih=0; ih<h.H.nhorns; ih++) {
/*      check mask for this horn at this position */
        if (CheckMASK(idmp,ih)) {
          GetDUMP(IF,idmp+1,M);
          WeightREF(WTmode,idmp+1,R1,R2,R);
          for(k=0; k<h.H.nchan; k++) {
            ii=index(k,ih,h.H.nchan);
            Z[k]=M[ii]-R[ii];     /* ok for now but this needs to be changed 
                                     for real data */
          }
/*        convolve this position into output image */
          xpos=h.H.XPOS[ih][idmp]; ypos=h.H.YPOS[ih][idmp];
          convolve(xpos,ypos,Z);
        }
      }
    }

    fclose(fp);
   }   /* end of infile loop */
  free(R1);
  free(R2);
  free(R);
  free(M);
  free(Z);

/* Normalize DATA */
   for(iy=0; iy<naxes[2]; iy++) {
     for(ix=0; ix<naxes[1]; ix++) {
       ii=index(ix,iy,naxes[1]);
       jj=ii*naxes[0];
       if (WT[ii]!=0.0) {
         for(k=0; k<naxes[0]; k++) T[jj+k]=T[jj+k]/WT[ii];
       }
       else {
         for(k=0; k<naxes[0]; k++) T[jj+k]=BLANK;
       }
     }
   }

   nelements=naxes[0];
   for(iy=10; iy<11; iy++) {
    for(ix=10; ix<11; ix++) {
     ii=index(ix,iy,naxes[1]);
     jj=ii*naxes[0];
for(k=0; k<naxes[0]; k++) fprintf(stderr,"\n %d %d %d %6.3f", ii,jj,k,T[jj+k]);

    }
   }

  free(WT);
  free(T);
  
}

int CheckMASK(idmp,ihorn)
int idmp,ihorn;
{
 long f;

 f=pow(2.,ihorn);
 if (h.H.MASK[idmp]&f == f) return(1);
 else return(0);
}

void GetXYLimits(xmin,xmax,ymin,ymax)
float *xmin,*xmax,*ymin,*ymax;
{
 int i,ih,idmp,k;


   ReadHEADER();
   for(idmp=0; idmp<h.H.nsample; idmp++) {
     for(ih=0; ih<h.H.nhorns; ih++) {
       if (CheckMASK(idmp,ih)) {
         if (h.H.XPOS[ih][idmp] < *xmin) *xmin=h.H.XPOS[ih][idmp];
         if (h.H.XPOS[ih][idmp] > *xmax) *xmax=h.H.XPOS[ih][idmp];
         if (h.H.YPOS[ih][idmp] < *ymin) *ymin=h.H.YPOS[ih][idmp];
         if (h.H.YPOS[ih][idmp] > *ymax) *ymax=h.H.YPOS[ih][idmp];
       }
     }
   }
}

void convolve(xpos,ypos,Z)
float xpos,ypos,*Z;
{
int ind,i,ii,j,ix,iy,k;
long jj;
float arg,deltax,deltay,x,y,radius;
double wt;

/* distribute this value to the output image */
 for(iy=10; iy<11; iy++) {
  y=60.0*crval3+(iy-crpix3)*THETA_N/60.0;
  deltay=60.0*fabs(y-ypos);
  for(ix=10; ix<11; ix++) {
   x=60.0*crval2-(ix-crpix2)*THETA_N/60.;
   deltax=60.0*fabs(x-xpos);
   radius=sqrt(pow(deltax,2.0)+pow(deltay,2.0));
   arg=radius/(2.0*THETA_N);
   if (arg < RMAX) {
     fprintf(stderr,"\n %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f: ", 
        xpos,ypos,x,y,deltax,deltay);
     wt=GetWeight(arg); 
     ii=index(ix,iy,naxes[1]);
    WT[ii]+=wt;
     jj=ii*naxes[0];
     fprintf(stderr," %5.3f %6.4f %6.4f %d %d", radius,wt,WT[ii],ii,jj);
     for(k=0; k<naxes[0];k++) T[jj+k]+=(float)wt*Z[k];
     fprintf(stderr," %6.4f %6.4f ", Z[128],T[jj+128]);
   }
  }
 }
}

void printerror( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
    {
       fits_report_error(stderr, status); /* print error report */

       exit( status );    /* terminate the program, returning error status */
    }
    return;
}

void MakeWeights()
{
int i;
double x;
double y1,y2,y3,y4;
double alpha=4.44,beta=4.75;
double PI=3.141592654;

  for(i=0; i<256; i++) {
    x=-5.0+0.0390625*i;
    weights[i]=2.0*jinc(4.44*x)*exp(-2.77*pow(x,2.)); ;
    
/*
    y1=jinc(x/alpha);     
    y2=exp(-pow(2.0*x/beta,2.)); 
    y3=jinc(3.831706*x/(2.0*PI*RMAX));     
    y4=1.0;
    if (fabs(x) > RMAX) y4=0.0;
    weights[i]=y1*y2*y3*y4;  
*/
  }
}

double jinc(r)
double r;
{
 double arg,f1,theta,j;

 if (r <=3.0) {
   
  arg=pow(r/3.0,2.0);
  j=0.5+arg*(-0.56249985+arg*(0.21093573+arg*(-0.03954289 
       +arg*(0.00443319+arg*(-0.00031761+arg*0.00001109)))));
 }
 else {
  arg=3.0/r;
  f1=0.79788456+arg*(0.00000156+arg*(0.01659667+arg*  
       (0.00017105+arg*(-.00249511+arg*(0.00113653-arg*  
        0.00020033)))));
   
  theta=r-2.35619449+arg*(0.12499612+arg*(0.00005650+arg* 
        (-.00637879+arg*(0.00074348+arg*(0.00079824-arg* 
         0.00029166))))) ;
   
  j=f1*cos(theta)/(r*sqrt(r));
 }
 return(j);
}

double GetWeight(arg)
float arg;
{
 int is,ie;
 float rxs,rxe;
 double w;

 is=(arg-(-5.0))/0.0390625;
 ie=is+1;
 rxs=-5.0+0.0390625*is;
 rxe=-5.0+0.0390625*ie;
 if (is<0 || is >254) {
   fprintf(stderr,"\n Weighting arguments out of range\n");
   exit(2);
 }
 w=weights[is]+(arg-rxs)*((weights[ie]-weights[is])/(rxe-rxs));
 return(w);
}
