#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <cpgplot.h>
#include "fitsio.h"

/*
#if defined(linux) || defined(LINUX)
#define main	MAIN__
#endif
*/

int status,nhorn=16;
long naxes[2],anaxes[2];
long nelements,fpixel=1,naxis=2;
float dnu=50.0e+03;
float TDUMP=0.25;
float TDUMPMIN=0.1,MAXTIMEREF=120.0,RateMAX=100.;
float DSUM;
float eta=2.71;
float N_c=16.0;
float tslew2ref=10.0;
float tcal=15.0;
float xa[32],ya[32],tsys[32],T[256][256],COL[256],ROW[256];
float XE,XS,YE,YS,xs,xe,r,DX,dx,tscan,ttotal;
float crval1,crval2;
float SCAN_ANGLE,LX,LY,XOFF,YOFF,THETA_N,RMS,ROWSTEP; 
double WT[256][256];
double weights[256];
double DEC=30.000; 
double LAT=0.73987416;   /* FCRAO latitude in radians */
double PI = 3.141592653589793;
double acrval1,acrval2,acrpix1,acrpix2,acdelt1,acdelt2,acrota;
char acoordtype[16];
int MAXDIM=2,anynul;
int idum=-1;
float *I,nulval;
long firstpixel[2];

void FindRampSize(float,float*,float*);
void MakeWeights();
double parallactic(double);
double jinc(double);
double bessj1(double);
float convolve_0(float,float);
void convolve(int,float,float);
void printerror(int);
int index(int,int,int);
double GetWeight(float);
double gasdev(int*);

main(argc,argv)
int argc;
char *argv[];
{
 FILE *fp,*fp1,*fp2;
 fitsfile *fptr,*ifptr;
 int ier,i,icol,irow,ix,iy;
 double pa,ha,HA,HAS;
 float tsysAV,x1,y1;
 float targetrms, RAMPX[2],RAMPY[2];
 float s1,s0,xmin,xmax,ymin,ymax,scan_angle;
 char infile[64],MAPCOORD[8];
 float nos,R,tcell,tref,nsample;
 float *sigma;
 float tr[6];
 float t,X,Y,x,y;
 float xref,yref;
 float TMAX,TMIN;
 int NRPO,NRPC;
 int just,axis;
 float x0,y0;
  
/* Get input parameters */
  if (argc < 14) {
    fprintf(stderr,"\n OTF PARAMETERS: ");
    fprintf(stderr,"\n SCAN COORDINATE (az,el,ra,dec,l,b) ");
    fprintf(stderr,"\n SCAN ANGLE");
    fprintf(stderr,"\n XOFF ");
    fprintf(stderr,"\n YOFF");
    fprintf(stderr,"\n LX (arcminutes)");
    fprintf(stderr,"\n LY (arcminutes)");
    fprintf(stderr,"\n NRPO rows per off (integer)");
    fprintf(stderr,"\n NRPC rows per cal (integer)");
    fprintf(stderr,"\n Output sampling cell (arcsec) ");
    fprintf(stderr,"\n Target RMS (K)");
    fprintf(stderr,"\n ROWSTEP (beams) ");
    fprintf(stderr,"\n Starting Hour Angle (hrs)");
    fprintf(stderr,"\n Input FITS image to observe");
    fprintf(stderr,"\n Exiting\n");
    exit(0);
  } 
  sscanf(argv[1],"%s",MAPCOORD);
  SCAN_ANGLE=atof(argv[2]);
  XOFF=atof(argv[3]);
  YOFF=atof(argv[4]);
  LX=atof(argv[5]);
  LY=atof(argv[6]);
  NRPO=atoi(argv[7]);
  NRPC=atoi(argv[8]);
  THETA_N=atof(argv[9]);
  RMS=atof(argv[10]);
  ROWSTEP=atof(argv[11]);
  HAS=atof(argv[12]);
  sscanf(argv[13],"%s",infile);

  scan_angle=SCAN_ANGLE*PI/180.;
/* SEQUOIA positions relative to pixel 11 */
  xa[0]=-2.952; xa[1]=-1.476; xa[2]=0.0; xa[3]=1.476;
  xa[4]=-2.952; xa[5]=-1.476; xa[6]=0.0; xa[7]=1.476;
  xa[8]=-2.952; xa[9]=-1.476; xa[10]=0.0; xa[11]=1.476;
  xa[12]=-2.952; xa[13]=-1.476; xa[14]=0.0; xa[15]=1.476;
  ya[0]=2.952; ya[1]=2.952; ya[2]=2.952; ya[3]=2.952;
  ya[4]=1.476; ya[5]=1.476; ya[6]=1.476; ya[7]=1.476;
  ya[8]=0.0; ya[9]=0.0; ya[10]=0.0; ya[11]=0.0;
  ya[12]=-1.476; ya[13]=-1.476; ya[14]=-1.476; ya[15]=-1.476;
/* SEQUOIA TSYS*/
  tsys[0]=300.; tsys[1]=307; tsys[2]=286; tsys[3]=290;
  tsys[4]=307; tsys[5]=309; tsys[6]=317; tsys[7]=304;
  tsys[8]=318; tsys[9]=341; tsys[10]=272; tsys[11]=325;
  tsys[12]=348; tsys[13]=282; tsys[14]=281; tsys[15]=329;

/* Open input image to observe */
  fits_open_file(&ifptr,argv[13],READONLY, &status);
  fits_get_img_size(ifptr, MAXDIM, anaxes, &status);
  nelements=anaxes[0]*anaxes[1];
  if ( (I = (float *)malloc(sizeof(float)*nelements))==NULL) {
     fprintf(stderr,"\n Can not allocate memory for data array\n");
     exit(3);
   }

/* Get WCS coordinate headers */
  fits_read_img_coord(ifptr, &acrval1,&acrval2,&acrpix1,&acrpix2,
                      &acdelt1,&acdelt2,&acrota,acoordtype,&status);
  acrval1=acrval1*60.0; acrval2=acrval2*60.0;
  acdelt1=acdelt1*60.0; acdelt2=acdelt2*60.0;

  firstpixel[0]=1; firstpixel[1]=1;
  nulval=0.0;
  status=0;
  if (fits_read_pix(ifptr,TFLOAT,firstpixel,nelements,&nulval,I,&anynul, &status
))
  printerror( status );


/* Make lookup table for convolving kernel */
   MakeWeights();

  HA=HAS;
  ha=15.0*HA*PI/180.;
  pa=parallactic(ha);
  FindRampSize(pa,RAMPX,RAMPY);
  
  s1=0.0; s0=0.0;
  for(i=0; i<nhorn; i++) {
    s1+=tsys[i];
    s0+=1.0;
  }
  tsysAV=s1/s0;

/* Make some guess as to the beam redundancy given ROWSTEP > 0.5 */
   if (ROWSTEP > 0.5 && ROWSTEP <= 8) N_c=8.0/ROWSTEP;
   if (ROWSTEP <= 0.5) N_c=16;
   if (ROWSTEP > 8) N_c=1;

   targetrms=RMS;
   nos=2.0;
   TDUMP=pow(tsysAV/RMS,2.)/(N_c*eta)/(nos*dnu);
   if (TDUMP < TDUMPMIN) {
     fprintf(stderr,"\n Cannot not run the correlators faster than 10 Hz");
     fprintf(stderr,"\n Setting TDUMP=0.1 secs"); 
     fprintf(stderr,"\n New Target RMS = %6.2f K", RMS); 
     TDUMP=0.1;
     targetrms=RMS;
     RMS=tsysAV/sqrt(TDUMP*N_c*eta*nos*dnu);
   }
   tcell=pow(tsysAV/RMS,2.)/N_c/dnu;
   nsample=60.0*NRPO*nos*(LX+RAMPX[0]+RAMPX[1])/(THETA_N);
   if (nsample*TDUMP > MAXTIMEREF) {
     fprintf(stderr,"\n Time between OFF measurements exceeds recommended maximum%4.0f secs",MAXTIMEREF);
     fprintf(stderr,"\n Increase target rms and make repeated maps\n");
     exit(1);
   }
   tref=0.5*TDUMP*sqrt(nsample);
   R=THETA_N/(60.0*nos*TDUMP);  /* antenna rate in arcmin/sec */
   if (R > RateMAX) {
     fprintf(stderr,"\n Adjusting over-sampling rate");
     do {
       nos+=0.5;
       R=THETA_N/(60.0*nos*TDUMP);  /* antenna rate in arcmin/sec */
       TDUMP=pow(tsysAV/RMS,2.)/(N_c*eta)/(nos*dnu);
       if (R < RateMAX) break;
     } while(nos <=4);
   }
   if (nos > 4) {
     fprintf(stderr,"\n Sampling rate too large -- exitting\n");
     exit(1);
   }

   fprintf(stderr,"\n Parallactic Angle=%4.0f ",pa*180/PI);
   fprintf(stderr,"\n Nc=%4.1f ",N_c);
   fprintf(stderr,"\n nos=%6.2f ",nos);
   fprintf(stderr,"\n DUMP RATE=%4.2f HZ",1.0/TDUMP);
   fprintf(stderr,"\n R=%6.2f arcmin/sec",R);
   fprintf(stderr,"\n <TSYS>=%6.0f K",tsysAV);
   fprintf(stderr,"\n tcell=%6.3f sec",tcell);
   fprintf(stderr,"\n nsample=%6.0f ",nsample);
   fprintf(stderr,"\n tref=%6.2f sec",tref);
   fprintf(stderr,"\n Estimated time between OFF measurements=%6.2f sec",2.*tslew2ref+nsample*TDUMP);
   fprintf(stderr,"\n Estimated time between CAL measurements=%6.2f sec",2.*tslew2ref+nsample*TDUMP*(float)(NRPC/NRPO));
  
   
   YS=YOFF*LY+LY/2.0+RAMPY[0]; YE=YOFF*LY-LY/2.0-RAMPY[1];
   XS=XOFF*LX+LX/2.0+RAMPX[0]; XE=XOFF*LX-LX/2.0-RAMPX[1];
   crval1=XS+RAMPX[0];
   crval2=YE-RAMPY[1];

   naxes[0]=60.0*(LX+2.0*fabs(RAMPX[0]+RAMPX[1]))/THETA_N;
   naxes[1]=60.0*(LY+2.0*fabs(RAMPY[0]+RAMPY[1]))/THETA_N;
   nelements=naxes[0]*naxes[1];
   fprintf(stderr,"\n Convolved Image %d x %d", naxes[0],naxes[1]);
   fprintf(stderr,"\n crval1 = %6.2f crval2 = %6.2f", crval1,crval2);
   fprintf(stderr,"\n Ramp Sizes: %5.2f %5.2f %5.2f %5.2f", RAMPX[0],RAMPX[1],
                   RAMPY[0],RAMPY[1]);

   fprintf(stderr,"\n Type any key to continue: ");
   getchar();

   if (naxes[0] > 256 || naxes[1] > 256) {
     fprintf(stderr,"\n Output image exceeds limits (256x256)");
     fprintf(stderr,"\n Exiting");
     exit(1);
   }
   if ( (sigma = (float *)malloc(sizeof(float)*nelements))==NULL) {
     fprintf(stderr,"\n Can not allocate memory for data array\n");
     exit(3);
   }
   tr[0]=crval1; tr[1]=-THETA_N/60.; tr[2]=0.0;
   tr[3]=crval2; tr[4]=0.0; tr[5]=-tr[1];



   if (fits_create_file(&fptr, "!ta.fits", &status)) 
         printerror( status );    
   if (fits_create_img(fptr,FLOAT_IMG,naxis,naxes,&status))
         printerror( status );    

/* Initialize T array */
   for(iy=0; iy<naxes[1]; iy++) {
     for(ix=0; ix<naxes[0]; ix++) {
      T[iy][ix]=0.0;
      WT[iy][ix]=0.0;
     }
   }

   y0=YS;
   irow=0;
   ttotal=0.0;
   DSUM=0.0;


    /* Open graphics device. */
    cpgopen("/xwin");

    just=1; axis=0;
    cpgenv(crval1,crval1-naxes[0]*THETA_N/60.,
           crval2,crval2+naxes[1]*THETA_N/60.,
           just,axis); 
    cpglab("x", "y", "OTF Image");


/* Start the map */
   while(y0 > YE) {
     Y=y0;
     if (fmod((double)irow,(double)NRPC)==0.0) {
       fprintf(stderr,"\n CAL measurement");
       X=xref; Y=yref; ttotal+=tslew2ref; HA+=tslew2ref/3600.;
       ttotal+=tcal; HA+=tcal/3600.;
     }
     if (fmod((double)irow,(double)NRPO)==0.0) {
       fprintf(stderr,"\n REF measurement");
       if (X!=xref || Y!=yref) {
         ttotal+=tslew2ref; HA+=tslew2ref/3600.;
       }
       X=xref; Y=yref; 
       ttotal+=tref;
       HA+=tref/3600.;
       ttotal+=tslew2ref; 
       HA+=tslew2ref/3600.;
       DSUM+=nhorn*2048*4.0;
     }

     irow++;
     fprintf(stderr,"\n Row: %4d  Hour Angle %6.2f hr PA = %4.0f", irow,HA,pa*180/PI);
     if (fmod((double)irow,2.0)==1.0) {
       xs=XS; xe=XE; r=R;
     }
     if (fmod((double)irow,2.0)==0.0) {
       xs=XE; xe=XS; r=-R;
     }
     DX=fabs(xe-xs);
     dx=0.0;
     x0=xs; 
     tscan=0.0;
     icol=0; 
     while(dx <= DX) {
      fprintf(stderr,".");
      icol++;
      DSUM+=nhorn*2048*4.0;
      tscan+=TDUMP;
      x0-=r*TDUMP;
      X=x0*cos(scan_angle)+y0*sin(scan_angle);
      Y=y0*cos(scan_angle)-x0*sin(scan_angle);
      dx+=fabs(R*TDUMP);
/* Update pa  */
      HA+=TDUMP/3600.;
      ha=15.0*HA*PI/180.;
      pa=parallactic(ha);
      for(i=0; i<nhorn; i++) {
       x1=X-xa[i]*cos(pa)+ya[i]*sin(pa);
       y1=Y+xa[i]*sin(pa)+ya[i]*cos(pa);

/*     convolve this position into output image */
       convolve(i,x1,y1);

      }  /* horn loop */
   s1=0.0; s0=0.0;
   for(iy=0; iy<naxes[1]; iy++) {
    for(ix=0; ix<naxes[0]; ix++) {
      i=index(ix,iy,naxes[0]);
      sigma[i]=0.0;
      if (WT[iy][ix]!=0.0) {
        sigma[i]=T[iy][ix]/WT[iy][ix];
        if (sigma[i]!=0.0) { s1+=sigma[i]; s0+=1.0;}
      }
    }
   }
   cpggray(sigma,naxes[0],naxes[1],1,naxes[0],1,naxes[1],-3.0*RMS,10.0*RMS,tr);

     }   /* scan loop */
     y0-=ROWSTEP*2.0*THETA_N/60.;   /*   y0-=THETA_N/60.;   */
     ttotal+=tscan;
   }   /* MAP is finished */

/* Be sure to end with a OFF measurement  */
   if (X!=xref || Y!=yref) {
     ttotal+=tslew2ref; HA+=tslew2ref/3600.;
     fprintf(stderr,"\n REF measurement");
     X=xref; Y=yref; 
     ttotal+=tref; HA+=tref/3600.;
     ttotal+=tslew2ref; HA+=tslew2ref/3600.;
     DSUM+=nhorn*2048*4.0;
   }


   s1=0.0; s0=0.0;
   for(iy=0; iy<naxes[1]; iy++) {
    for(ix=0; ix<naxes[0]; ix++) {
      i=index(ix,iy,naxes[0]);
      sigma[i]=0.0;
      if (WT[iy][ix]!=0.0) {
        sigma[i]=T[iy][ix]/WT[iy][ix];
        if (sigma[i]!=0.0) { s1+=sigma[i]; s0+=1.0;}
      }
    }
   }
   cpggray(sigma,naxes[0],naxes[1],1,naxes[0],1,naxes[1],-3.0*RMS,10.0*RMS,tr);

   fprintf(stderr,"\n Type any key to continue: ");
   getchar();


   fprintf(stderr,"\n Closing graphics window ");
   cpgclos();
   fprintf(stderr,"\n Writing FITS file");
   fpixel=1;
   if (fits_write_img(fptr, TFLOAT,fpixel,nelements,&sigma[0], &status))
     printerror( status );    
   if (fits_close_file(fptr,&status)) 
         printerror( status );    

   free(sigma);
   fprintf(stderr,"\n\n ******************************************"); 
   fprintf(stderr,"\n TOTAL TIME        :%6.1f minutes",ttotal/60.);
   fprintf(stderr,"\n RAW DATA COLLECTED:%5.1f Mbytes ",DSUM/1.0e+06); 
   fprintf(stderr,"\n REGRIDDED DATA    :%5.1f Mbytes ",
              (float)nelements*4.0*2048./1.0e+06); 
   fprintf(stderr,"\n ******************************************\n\n"); 
}

void convolve(ihorn,x1,y1)
int ihorn;
float x1,y1;
{
int ind, i,j,ix,iy;
float arg,deltax,deltay,x,y,radius;
float noise;
float TA,TANOISE,TTT;
double wt;

/* First convolve true distribution with FCRAO telescope beam */
 TA=convolve_0(x1,y1);
/* Add some noise */
 noise=tsys[ihorn]/sqrt(dnu*TDUMP);
 TANOISE=noise*(float)gasdev(&idum);
 TTT=TA+TANOISE;


/* now distribute this value to the output image */
 for(iy=0; iy<naxes[1]; iy++) {
  y=crval2+iy*THETA_N/60.;
  deltay=60.0*fabs(y-y1);
  for(ix=0; ix<naxes[0]; ix++) {
   x=crval1-ix*THETA_N/60.;
   deltax=60.0*fabs(x-x1);
   radius=sqrt(pow(deltax,2.0)+pow(deltay,2.0))/THETA_N;   /* in cell units */
   if (radius < 5.0) {   
    wt=GetWeight(radius); 
    T[iy][ix]+=(float)wt*TTT;
    WT[iy][ix]+=wt;
   }
  }
 }
}
double parallactic(HA)
double HA;
{
  double ypa, xpa, pa;

  ypa = sin(HA);
  xpa = cos(DEC)*tan(LAT) - sin(DEC)*cos(HA);
  pa = atan2(ypa,xpa);

  return (pa);
}

void printerror( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
    {
       fits_report_error(stderr, status); /* print error report */

       exit( status );    /* terminate the program, returning error status */
    }
    return;
}

void MakeWeights()
{
int i;
double x;
double y1,y2,y3,y4;
double alpha=1.55,beta=2.52,RMAX=3.0;
double PI=3.141592654;

  for(i=0; i<256; i++) {
    x=0.01953125*i;
    weights[i]=2.0*jinc(x/alpha)*exp(-pow(x/beta,2.));  /* NRAO function */
    
/*
    y1=jinc(x/alpha);     
    y2=exp(-pow(2.0*x/beta,2.)); 
    y3=jinc(3.831706*x/(2.0*PI*RMAX));     
    y4=1.0;
    if (fabs(x) > RMAX) y4=0.0;
    weights[i]=y1*y2*y3*y4;  
*/
  }
}

double jinc(r)
double r;
{
 double arg,f1,theta,j;

 if (r <=3.0) {
   
  arg=pow(r/3.0,2.0);
  j=0.5+arg*(-0.56249985+arg*(0.21093573+arg*(-0.03954289 
       +arg*(0.00443319+arg*(-0.00031761+arg*0.00001109)))));
 }
 else {
  arg=3.0/r;
  f1=0.79788456+arg*(0.00000156+arg*(0.01659667+arg*  
       (0.00017105+arg*(-.00249511+arg*(0.00113653-arg*  
        0.00020033)))));
   
  theta=r-2.35619449+arg*(0.12499612+arg*(0.00005650+arg* 
        (-.00637879+arg*(0.00074348+arg*(0.00079824-arg* 
         0.00029166))))) ;
   
  j=f1*cos(theta)/(r*sqrt(r));
 }
 return(j);
}

int index(i,j,NX)
int i,j,NX;
{
  return(i+j*NX);
}
  
void FindRampSize(pa,RAMPX,RAMPY)
float pa,*RAMPX,*RAMPY;
{
  int i;
  float x1,y1,xmin,xmax,ymin,ymax;
  
/* Find the extreme, rotated positions within the array at this angle */
  xmin=1.0e+20; xmax=-1.0e+20; ymin=1.0e+20; ymax=-1.0e+20;
  for(i=0; i<nhorn; i++) {
    x1=-xa[i]*cos(pa)+ya[i]*sin(pa);
    if (x1 < xmin) xmin=x1;
    if (x1 > xmax) xmax=x1;

    y1=xa[i]*sin(pa)+ya[i]*cos(pa);
    if (y1 < ymin) ymin=y1;
    if (y1 > ymax) ymax=y1;
  }
/* Set default ramp offset */
   RAMPX[0]=fabs(xmin);
   RAMPX[1]=fabs(xmax);
   RAMPY[0]=fabs(ymin);
   RAMPY[1]=fabs(ymax);

   RAMPY[1]=RAMPY[0];
   RAMPX[1]=RAMPX[0];
}

double GetWeight(arg)
float arg;
{
 int is;
 float rxs,rxe;
 double w;

 is=(arg/0.01953125);
 if (is<0 || is >255) {
   fprintf(stderr,"\n Weighting arguments out of range\n");
   exit(2);
 }
 w=weights[is];
 return(w);
}

float convolve_0(x1,y1)
float x1,y1;
{
 int i,j,ind,ix,iy,ix1,ix2,iy1,iy2;
 float TT,SUM,X,Y,XMAX,XMIN,YMAX,YMIN;
 float x,y,radius,deltax,deltay,arg, S,WSUM,W,FWHM=0.75;

 XMAX=x1+3.0*FWHM; XMIN=x1-3.0*FWHM;
 YMAX=y1+3.0*FWHM; YMIN=y1-3.0*FWHM;
 ix1=acrpix1+(XMAX-acrval1)/acdelt1-1;
 ix2=acrpix1+(XMIN-acrval1)/acdelt1-1;
 iy1=acrpix2+(YMIN-acrval2)/acdelt2-1;
 iy2=acrpix2+(YMAX-acrval2)/acdelt2-1;
 if (iy1 < 0) iy1=0;  if (iy2 > anaxes[1]) iy2=anaxes[1];
 if (ix1 < 0) ix1=0;  if (ix2 > anaxes[0]) ix2=anaxes[0];

 S=0.0; WSUM=0.0;
 for(iy=iy1; iy<=iy2; iy++) {
  y=acrval2+(iy-acrpix2)*acdelt2;
  deltay=fabs(y-y1);
  for(ix=ix1; ix<=ix2; ix++) {
   x=acrval1+(ix-acrpix1)*acdelt1;
   deltax=fabs(x-x1);
   radius=sqrt(pow(deltax,2.0)+pow(deltay,2.0));
   if (radius < 10.0*FWHM) {   
    ind=index(ix,iy,anaxes[0]);
    arg=2.77*pow(radius/FWHM,2.0);
    W=exp(-arg);
    S+=(float)W*I[ind];
    WSUM+=W;
   }
  }
 }
 TT=0.0;
 if (WSUM !=0.0) TT=S/WSUM;
 return(TT);
}

