#include "otf.h"


FILE *fp;
fitsfile *ifptr;
char actype1[16],actype2[16],actype3[16];
char acoordtype[16];
int anynul;
int ndumps;
int idum=-1;
int maxdim=3;
long naxes[3],anaxes[3];
long nelements,fpixel=1,naxis=2;
float nos;
float dnu=18.4e+03;
float TDUMPMIN=0.1,MAXTIMEREF=120.0,RateMAX=300.;
float RMS,RMS1,TDUMP=0.1;
float DSUM;
float eta=2.71;
float N_c=16.0;
float tref,tslew2ref=10.0;
float tcal=15.0;
float XA,YA,X,Y,XE,XS,YE,YS,xs,xe,r,DX,dx,tscan,ttotal;
float xref,yref;
float SCAN_ANGLE,LX,LY,XOFF,YOFF,ROWSTEP; 
double XCENTER,YCENTER,DEC=30.000; 
double LAT=0.73987416;   /* FCRAO latitude in radians */
double HA;

void DoCALMeasurement();
void DoREFMeasurement(float*);
void WriteDATA(float*);
void WriteHEADER();
void CreateScanFile();
void FindRampSize(float,float*,float*);
void printerror(int);
void convolve_0(float,float,int,double,double,double,double,double,double,float*);
double gasdev(int*);
double parallactic(double);
double GetAngle(double,float,float);

main(argc,argv)
int argc;
char *argv[];
{
 char infitsfile[64],MAPCOORD[8];
 char comment[64];
 int k,iw,ier,i,icol,irow,ix,iy,itdump;
 int idmp,ih; 
 int iscan1,NRPO,NRPC;
 int just,axis;
 float tsysAV,x1,y1,x2,y2;
 float RAMPX[2],RAMPY[2];
 float s1,s0,xmin,xmax,ymin,ymax,scan_angle;
 float R,tcell;
 float tr[6];
 float crval11,crval22; 
 float t,x,y;
 float dxh,dyh;
 float TMAX,TMIN;
 float x0,y0;
 float TT[4096];
 float acrval1,acrval2,acrval3,acrpix1,acrpix2,acrpix3,
       acdelt1,acdelt2,acdelt3;
 float theta_n=23.0;
 double a,pa,ha,HAS;
  
/* Get input parameters */
  if (argc < 16) {
    fprintf(stderr,"\n OTF PARAMETERS: ");
    fprintf(stderr,"\n RA/L position of center position in degrees");
    fprintf(stderr,"\n DEC/B position of center position in degrees ");
    fprintf(stderr,"\n SCAN COORDINATE (equa | gala) ");
    fprintf(stderr,"\n SCAN ANGLE");
    fprintf(stderr,"\n XOFF ");
    fprintf(stderr,"\n YOFF");
    fprintf(stderr,"\n LX (arcminutes)");
    fprintf(stderr,"\n LY (arcminutes)");
    fprintf(stderr,"\n NRPO rows per off (integer)");
    fprintf(stderr,"\n NRPC rows per cal (integer)");
    fprintf(stderr,"\n Target RMS (K)");
    fprintf(stderr,"\n ROWSTEP (beams) ");
    fprintf(stderr,"\n Starting Hour Angle (hrs)");
    fprintf(stderr,"\n Input FITS image to observe");
    fprintf(stderr,"\n Starting scan number ");
    fprintf(stderr,"\n Exiting\n");
    exit(0);
  } 
  XCENTER=atof(argv[1]);
  YCENTER=atof(argv[2]);
  sscanf(argv[3],"%s",MAPCOORD);
  if (!strncmp(MAPCOORD,"equa",4)) {
    H.kmap=2;
  }
  else if (!strncmp(MAPCOORD,"gala",4)) {
   H.kmap=3;
  }
  else {
    fprintf(stderr,"\n Unrecognized map coordinate");
    fprintf(stderr,"\n  --  only \"equa\" or \"gala\" allowed ");
    exit(1);
  } 
  SCAN_ANGLE=atof(argv[4]);
  XOFF=atof(argv[5]);
  YOFF=atof(argv[6]);
  LX=atof(argv[7]);
  LY=atof(argv[8]);
  NRPO=atoi(argv[9]);
  NRPC=atoi(argv[10]);
  RMS=atof(argv[11]);
  ROWSTEP=atof(argv[12]);
  HAS=atof(argv[13]);
  sscanf(argv[14],"%s",infitsfile);
  fprintf(stderr,"\n %s", infitsfile);
  iscan1=atoi(argv[15]);

  scan_angle=SCAN_ANGLE*3.1415927/180.;

   H.nhorns=16;
   H.khorn=11;
/* SEQUOIA positions relative to pixel 11 */
  H.DXHORNS[0]=-492; H.DXHORNS[1]=-246; H.DXHORNS[2]=0; H.DXHORNS[3]=246;
  H.DXHORNS[4]=-492; H.DXHORNS[5]=-246; H.DXHORNS[6]=0; H.DXHORNS[7]=246;
  H.DXHORNS[8]=-492; H.DXHORNS[9]=-246; H.DXHORNS[10]=0; H.DXHORNS[11]=246;
  H.DXHORNS[12]=-492; H.DXHORNS[13]=-246; H.DXHORNS[14]=0; H.DXHORNS[15]=246;
  H.DYHORNS[0]=492; H.DYHORNS[1]=492; H.DYHORNS[2]=492; H.DYHORNS[3]=492;
  H.DYHORNS[4]=246; H.DYHORNS[5]=246; H.DYHORNS[6]=246; H.DYHORNS[7]=246;
  H.DYHORNS[8]=0; H.DYHORNS[9]=0; H.DYHORNS[10]=0; H.DYHORNS[11]=0;
  H.DYHORNS[12]=-246; H.DYHORNS[13]=-246; H.DYHORNS[14]=-246; H.DYHORNS[15]=-246;


/* Open input image to observe */
  fprintf(stderr,"\n OPening file %s", infitsfile);
  fits_open_file(&ifptr,infitsfile,READONLY, &status);
  fits_get_img_size(ifptr, maxdim, anaxes, &status);
  nelements=anaxes[0];

/* Get WCS coordinate headers */
  fits_read_key(ifptr, TFLOAT, "CRVAL1",&acrval1,comment,&status);
  fits_read_key(ifptr, TFLOAT, "CRPIX1",&acrpix1,comment,&status);
  fits_read_key(ifptr, TFLOAT, "CDELT1",&acdelt1,comment,&status);

  fits_read_key(ifptr, TFLOAT, "CRVAL2",&acrval2,comment,&status);
  fits_read_key(ifptr, TFLOAT, "CRPIX2",&acrpix2,comment,&status);
  fits_read_key(ifptr, TFLOAT, "CDELT2",&acdelt2,comment,&status);

  fits_read_key(ifptr, TFLOAT, "CRVAL3",&acrval3,comment,&status);
  fits_read_key(ifptr, TFLOAT, "CRPIX3",&acrpix3,comment,&status);
  fits_read_key(ifptr, TFLOAT, "CDELT3",&acdelt3,comment,&status);
/* use llc offset as reference point and transform into arcminutes */
  acdelt2=acdelt2*60.0; acdelt3=acdelt3*60.0;
  acrval2=-acdelt2*(acrpix2-1); acrval3=-acdelt3*(acrpix3-1);
  acrpix2=0.0; acrpix3=0.0;
  fprintf(stderr,"\n LLC OFFSETS: %7.3f %7.3f", acrval2,acrval3);

  HA=HAS;
  ha=15.0*HA*3.1415927/180.;
  fprintf(stderr,"\n Finding ramp sizes");
  FindRampSize(ha,RAMPX,RAMPY);
  fprintf(stderr,"\n %f %f %f %f", RAMPX[0],RAMPX[1],RAMPY[0],RAMPY[1]);
  

/* Make some guess as to the beam redundancy given ROWSTEP > 0.5 */
   if (ROWSTEP > 0.5 && ROWSTEP <= 8) N_c=8.0/ROWSTEP;
   if (ROWSTEP <= 0.5) N_c=16;
   if (ROWSTEP > 8) N_c=1;

/* Do a CAL to estimate OTF paramters from mean Tsys */
  DoCALMeasurement();
  s1=0.0; s0=0.0;
  for(i=0; i<H.nhorns; i++) {
    s1+=H.tsys[i];
    s0+=1.0;
  }
  if (RMS == 0.0) {
    RMS1=RMS;
    RMS=5.0;
  }
  tsysAV=s1/s0;
  fprintf(stderr,"\n Mean Tsys = %6.1f", tsysAV);
  nos=2.0;
  TDUMP=0.1;
  THETA_N=theta_n;
  TDUMP=pow((tsysAV/RMS),2.)/(dnu);
  itdump=(TDUMP/0.05);
  TDUMP=0.05*itdump;

  if (TDUMP < TDUMPMIN) {
     fprintf(stderr,"\n Cannot not run the correlators faster than 10 Hz");
     fprintf(stderr,"\n Setting TDUMP=0.1 secs"); 
     TDUMP=0.1;
     itdump=(TDUMP/0.05);
     TDUMP=0.05*itdump;
     RMS=tsysAV/sqrt(TDUMP*dnu);
     fprintf(stderr,"\n New Target RMS = %6.2f K", RMS); 
   }
   tcell=pow(tsysAV/RMS,2.)/N_c/dnu;
   ndumps=60.0*NRPO*nos*(LX+RAMPX[0]+RAMPX[1])/(THETA_N)-1;
   if (ndumps*+2 > MAXSCANSIZE) {
     fprintf(stderr,"\n Scan Length exceeds maximum (%d)",MAXSCANSIZE);
     exit(1);
   }
   if (ndumps*TDUMP > MAXTIMEREF) {
     fprintf(stderr,"\n Time between OFF measurements exceeds recommended maximum%4.0f secs",MAXTIMEREF);
     fprintf(stderr,"\n Increase target rms and make repeated maps\n");
     exit(1);
   }
   tref=0.5*TDUMP*sqrt(ndumps);
   R=THETA_N/(60.0*nos*TDUMP);  /* antenna rate in arcmin/sec */
   if (R > RateMAX) {
     fprintf(stderr,"\n Adjusting over-sampling rate");
     do {
       nos+=0.5;
       R=THETA_N/(60.0*nos*TDUMP);  /* antenna rate in arcmin/sec */
       TDUMP=pow(tsysAV/RMS,2.)/(N_c*eta)/(nos*dnu);
       itdump=(TDUMP/0.05);
       TDUMP=0.05*itdump;
       if (R < RateMAX) break;
     } while(nos <=4);
   }
   if (nos > 4) {
     fprintf(stderr,"\n Sampling rate too large -- exitting\n");
     exit(1);
   }
   if (ndumps > MAXSCANSIZE) {
     fprintf(stderr,"\n Number of samples is too large (>%d) -- exitting\n",MAXSCANSIZE);
     exit(1);
   }
/* Establish some basic header information */
   sprintf(H.otflab,"OTF REV %3.1",(float)OTFREV);
   strcpy(H.floatopt,"INTL");
   strcpy(H.name,"cepheus");
   strcpy(H.bunit,"K       ");
   strcpy(H.vlabel,"VELO-LSR");
   strcpy(H.xlabel,"RA---SIN");
   strcpy(H.ylabel,"DEC--SIN");


   H.tsid=XCENTER+HAS*15.0;   /* degrees */
   H.utc=H.tsid+5.0*15.0;   /* degrees */
   H.targetrms=RMS;
   H.tdump=TDUMP;
   H.tref=tref;
   H.telrate=R;
   H.epoch=2000.0;
   H.bw=H.nchan*dnu/1.0e+06;
   H.vlsr=acrval1+acdelt1*((anaxes[0]/2)-acrpix1);
   H.vclsr=0.0;
   H.vcsun=0.0;
   H.NOS=nos;
   H.tau=0.5;
   H.h2omm=2.5;
   H.pamb=1.0;
   H.tamb=300.0;
   H.rotang=scan_angle;
   H.XSIZE=LX;
   H.YSIZE=LY;
   H.XOFFMAP=XOFF;
   H.YOFFMAP=YOFF;
   H.XRAMP=RAMPX[0];
   H.YRAMP=RAMPY[0];
   for(ih=0; ih<MAXHORNS; ih++) H.gain[ih]=1000;

   H.utd=2001.43017712;        /* fractional year */
   H.f0=110.20137;
   H.fsky=110.20137;
   H.azs=90.0;
   H.els=45.0;
   H.XCEN=XCENTER;
   H.YCEN=YCENTER;
   H.xvel1=acrval1;
   H.xfr1=H.fsky-0.5*H.bw/1.0e+03;
   H.dveldin=acdelt1;
   H.dfrdin=dnu;
   H.fr1=0.0;
   H.fr2=0.0;

   H.iscan=iscan1;
   H.nchan=anaxes[0];
   H.blksize=(int)MAXSCANSIZE;
   H.telcoord=H.kmap;
   H.kx=0;
   H.ky=0;
   H.klo=1;
   H.khm=1;
   H.harm=1;
   H.krecv=5;
   H.caltype=0;
   H.calscan=iscan1;
   H.cfact=1000;
   H.etab=1000;
   H.raoff=0;
   H.decoff=0;
   H.fsrate=0;
   H.stype=0;
   fprintf(stderr,"\n Nc=%4.1f ",N_c);
   fprintf(stderr,"\n nos=%6.2f ",nos);
   fprintf(stderr,"\n DUMP RATE=%4.2f HZ",1.0/TDUMP);
   fprintf(stderr,"\n R=%6.2f arcmin/sec",R);
   fprintf(stderr,"\n <TSYS>=%6.0f K",tsysAV);
   fprintf(stderr,"\n tcell=%6.3f sec",tcell);
   fprintf(stderr,"\n ndumps=%4d ",ndumps);
   fprintf(stderr,"\n tref=%6.2f sec",tref);
   fprintf(stderr,"\n Estimated time between OFF measurements=%6.2f sec",2.*tslew2ref+ndumps*TDUMP);
   fprintf(stderr,"\n Estimated time between CAL measurements=%6.2f sec",2.*tslew2ref+ndumps*TDUMP*(float)(NRPC/NRPO));
  
   
   YS=YOFF*LY+LY/2.0+RAMPY[0]; YE=YOFF*LY-LY/2.0-RAMPY[1];
   XS=XOFF*LX+LX/2.0+RAMPX[0]; XE=XOFF*LX-LX/2.0-RAMPX[1];

   x1=XS*cos(scan_angle)+YS*sin(scan_angle);
   x2=XE*cos(scan_angle)+YE*sin(scan_angle);
   crval1=x1;
   crval11=x2;
   if (x2 > x1) {
      crval1=x2;
      crval11=x1;
   }
   y1=YE*cos(scan_angle)-XE*sin(scan_angle);
   y2=YS*cos(scan_angle)-XS*sin(scan_angle);
   crval2=y1; crval22=y2;
   if (y2 < y1) {
     crval2=y2;
     crval22=y1;
   }


   naxes[0]=60.0*(crval1-crval11)/THETA_N;
   naxes[1]=60.0*(crval22-crval2)/THETA_N;
   fprintf(stderr,"\n Convolved Image %d x %d", naxes[0],naxes[1]);
   fprintf(stderr,"\n crval1 = %6.2f crval2 = %6.2f", crval1,crval2);
   fprintf(stderr,"\n Ramp Sizes: %5.2f %5.2f %5.2f %5.2f", RAMPX[0],RAMPX[1],
                   RAMPY[0],RAMPY[1]);

   fprintf(stderr,"\n Type any key to continue: ");
   getchar();

   tr[0]=crval1; tr[1]=-THETA_N/60.; tr[2]=0.0;
   tr[3]=crval2; tr[4]=0.0; tr[5]=-tr[1];

/* Open graphics device. */
   cpgopen("/xwin");
   just=1; axis=0;
   cpgenv(crval1,crval1-naxes[0]*THETA_N/60.,
           crval2,crval2+naxes[1]*THETA_N/60.,
           just,axis); 
   cpglab("x", "y", "OTF Points");

/* Create first scan file and do the initial REF measurement */
   CreateScanFile();
   icol=0;
   DoREFMeasurement(TT);
   OTFPOS.ANGLE[icol]=24000;
   OTFPOS.XOFF[icol]=24000;
   OTFPOS.YOFF[icol]=24000;
   WriteDATA(TT);

/* Start the map */
   y0=YS;
   irow=0;
   ttotal=0.0;
   DSUM=0.0;
   while(y0 > YE) {
     Y=y0;
     irow++;
     fprintf(stderr,"\n Row: %4d  Hour Angle %6.2f hr PA = %4.0f", irow,HA,a*180/3.1415927);
     if (fmod((double)irow,2.0)==1.0) {
       xs=XS; xe=XE; r=R;
     }
     if (fmod((double)irow,2.0)==0.0) {
       xs=XE; xe=XS; r=-R;
     }
     DX=fabs(xe-xs);
     dx=0.0;
     x0=xs; 
     tscan=0.0;
     while(dx <= DX) {
      icol++;
      DSUM+=H.nhorns*H.nchan*4.0;
      tscan+=TDUMP;
      x0-=r*TDUMP;
      X=x0*cos(scan_angle)+y0*sin(scan_angle);
      Y=y0*cos(scan_angle)-x0*sin(scan_angle);
      dx+=fabs(R*TDUMP);
/* Update angle  */
      HA+=TDUMP/3600.;
      ha=15.0*HA*3.1415927/180.;
      YA=YCENTER+Y/60.0;
      XA=XCENTER+X/60.0/cos(YA);
      a=GetAngle(ha,XA,YA);
      OTFPOS.ANGLE[icol]=a;
      OTFPOS.XOFF[icol]=X;
      OTFPOS.YOFF[icol]=Y;
      for(i=0; i<H.nhorns; i++) {
       dxh=(H.DXHORNS[i]-H.DXHORNS[H.khorn])*60.0/1.0e+04;
       dyh=(H.DYHORNS[i]-H.DYHORNS[H.khorn])*60.0/1.0e+04;
       x1=X-dxh*cos(a)+dyh*sin(a);
       y1=Y+dxh*sin(a)+dyh*cos(a);
       cpgpt1(x1,y1,-2);
       convolve_0(x1,y1,i,acrval2,acrpix2,acdelt2,acrval3,acrpix3,acdelt3,TT);

      }  /* horn loop */
      WriteDATA(TT);

     }   /* scanning  loop */

     if (fmod((double)irow,(double)NRPO)==0.0 && y0 > YE) {
      icol++;
      DoREFMeasurement(TT);
      OTFPOS.ANGLE[icol]=24000;
      OTFPOS.XOFF[icol]=24000;
      OTFPOS.YOFF[icol]=24000;
      WriteDATA(TT);
      H.nsample=icol;
      H.nblock=H.nsample/H.blksize+1;
      WriteHEADER();
      fclose(otffp);

      H.iscan=H.iscan+1;
      icol=0; 
      if (H.iscan > 30000) H.iscan=1001;
      CreateScanFile();
      OTFPOS.ANGLE[icol]=24000;
      OTFPOS.XOFF[icol]=24000;
      OTFPOS.YOFF[icol]=24000;

      if (fmod((double)irow,(double)NRPC)==0.0) {
       DoCALMeasurement();
       H.calscan=H.iscan;
       icol=0; 
       DoREFMeasurement(TT);    /*  a new ref measure is required after a CAL */
       OTFPOS.ANGLE[icol]=24000;
       OTFPOS.XOFF[icol]=24000;
       OTFPOS.YOFF[icol]=24000;
      }
      WriteDATA(TT);
     }

     y0-=ROWSTEP*2.0*THETA_N/60.;   /*   y0-=THETA_N/60.;   */
     ttotal+=tscan;
   }   /* MAP is finished */

/* Be sure to end with a OFF measurement  */
   if (fmod((double)irow,(double)NRPO)!=0.0) {
     icol++;
     DoREFMeasurement(TT);
     OTFPOS.ANGLE[icol]=24000;
     OTFPOS.XOFF[icol]=24000;
     OTFPOS.YOFF[icol]=24000;
     WriteDATA(TT);
     WriteHEADER();
     fclose(fp);
   }


/*
   fprintf(stderr,"\n Closing graphics window ");
   cpgclos();
*/

   fprintf(stderr,"\n\n ******************************************"); 
   fprintf(stderr,"\n TOTAL TIME        :%6.1f minutes",ttotal/60.);
   fprintf(stderr,"\n RAW DATA COLLECTED:%5.1f Mbytes ",DSUM/1.0e+06); 
   fprintf(stderr,"\n REGRIDDED DATA    :%5.1f Mbytes ",
              (float)naxes[0]*naxes[1]*4.0*H.nchan/1.0e+06); 
   fprintf(stderr,"\n ******************************************\n\n"); 
}

double GetAngle(HA,X,Y)
double HA;
float X,Y;
{
 float XR,YR;
 double A,ypa, xpa, PA;
 double ra1,dec1,xga,yga,dga;
 double gzra=4.649646134;
 double gzdec=-5.050315348;
 double gpra=3.3660333150;
 double gpdec=0.4734787069;
 double eqzl=1.681402682;
 double eqzb=-1.050486996;
 double eqpl=2.145566646;
 double eqpb=0.4734787069;

 XR=X/57.29578;  YR=Y/57.29578; 
 ypa = sin(HA);
 xpa = cos(YR)*tan(LAT) - sin(YR)*cos(HA);
 PA = atan2(ypa,xpa);
 switch (H.kmap) {
   case 2:
     A=PA;
     break;
   case 3:

/*   convert current l,b coordinates to ra,dec j2000 */
     coord(eqzl,eqzb,eqpl,eqpb,XR,YR,&ra1,&dec1);
     ra1=ra1*57.29578; dec1=dec1*57.29578;

     yga=sin(ra1-gpra);
     xga=cos(dec1)*tan(gpdec)-sin(dec1)*cos(ra1-gpra);
     dga=atan2(yga,xga);
     A=PA+dga;
     break;
  }

  return (A);
}

double parallactic(HA)
double HA;
{
  double ypa, xpa, pa;

  ypa = sin(HA);
  xpa = cos(DEC)*tan(LAT) - sin(DEC)*cos(HA);
  pa = atan2(ypa,xpa);

  return (pa);
}

void printerror( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
    {
       fits_report_error(stderr, status); /* print error report */

       exit( status );    /* terminate the program, returning error status */
    }
    return;
}

void FindRampSize(ha,RAMPX,RAMPY)
float ha,*RAMPX,*RAMPY;
{
  int i;
  float x1,y1,xmin,xmax,ymin,ymax;
  double a;
  
  YA=YCENTER+(XOFF*LY+LY/2.0)/60.0;
  XA=XCENTER+(XOFF*LX+LX/2.0)/60.0/cos(YA);
  a=GetAngle(ha,XA,YA);

/* Find the extreme, rotated positions within the array at this angle */
  xmin=1.0e+20; xmax=-1.0e+20; ymin=1.0e+20; ymax=-1.0e+20;
  for(i=0; i<H.nhorns; i++) {
    x1=-(H.DXHORNS[i]*60.0/1.0e+04)*cos(a)+
        (H.DYHORNS[i]*60.0/1.0e+04)*sin(a);
    if (x1 < xmin) xmin=x1;
    if (x1 > xmax) xmax=x1;

    y1=(H.DXHORNS[i]*60.0/1.0e+04)*sin(a)+
       (H.DYHORNS[i]*60.0/1.0e+04)*cos(a);
    if (y1 < ymin) ymin=y1;
    if (y1 > ymax) ymax=y1;
  }
/* Set default ramp offset */
   RAMPX[0]=fabs(xmin);
   RAMPX[1]=fabs(xmax);
   RAMPY[0]=fabs(ymin);
   RAMPY[1]=fabs(ymax);

   RAMPY[1]=RAMPY[0];
   RAMPX[1]=RAMPX[0];
}

void convolve_0(x1,y1,ihorn,acrval2,acrpix2,acdelt2,acrval3,acrpix3,acdelt3,TT)
int ihorn;
float x1,y1,*TT;
double acrval2,acrval3,acrpix2,acrpix3,acdelt2,acdelt3;
{
 int i,ii,j,k,ix,iy,ix1,ix2,iy1,iy2;
 long ind,firstpixel;
 float XMAX,XMIN,YMAX,YMIN;
 float x,y,radius,deltax,deltay,arg,WSUM,W,FWHM=0.75;
 float *I,*S;
 float nulval;
 float noise;


 if ( (I = (float *) malloc(sizeof(float)*H.nchan))==NULL) {
   fprintf(stderr,"\n Can not allocate memory for array\n");
   exit(3);
 }
 if ( (S = (float *) malloc(sizeof(float)*H.nchan))==NULL) {
   fprintf(stderr,"\n Can not allocate memory for array\n");
   exit(3);
 }
 XMAX=x1+3.0*FWHM; XMIN=x1-3.0*FWHM;
 YMAX=y1+3.0*FWHM; YMIN=y1-3.0*FWHM;
 ix1=acrpix2+(XMAX-acrval2)/acdelt2;
 ix2=acrpix2+(XMIN-acrval2)/acdelt2;
 iy1=acrpix3+(YMIN-acrval3)/acdelt3;
 iy2=acrpix3+(YMAX-acrval3)/acdelt3;
 if (iy1 < 0) iy1=0;  if (iy2 > anaxes[2]-1) iy2=anaxes[2]-1;
 if (ix1 < 0) ix1=0;  if (ix2 > anaxes[1]-1) ix2=anaxes[1]-1;


 WSUM=0.0;
 for(k=0; k<H.nchan; k++) {
  S[k]=0.0;
 }
 for(iy=iy1; iy<=iy2; iy++) {
  y=acrval3+(iy-acrpix3)*acdelt3;
  deltay=fabs(y-y1);
  for(ix=ix1; ix<=ix2; ix++) {
   x=acrval2+(ix-acrpix2)*acdelt2;
   deltax=fabs(x-x1);
   radius=sqrt(pow(deltax,2.0)+pow(deltay,2.0));
   if (radius < 3.0*FWHM) {   
    ind=Getindex(ix,iy,anaxes[1]);
    firstpixel=ind*anaxes[0]+1;
    nulval=0.0;
    status=0;
    if (fits_read_img(ifptr,TFLOAT,firstpixel,nelements,&nulval,I,&anynul, &status
))
    printerror( status );

    arg=2.77*pow(radius/FWHM,2.0);
    W=exp(-arg);
    for(k=0; k<H.nchan; k++) S[k]+=(float)W*I[k];
    WSUM+=W;
   }
  }
 }

 noise=H.tsys[ihorn]/sqrt(dnu*TDUMP);
 if (RMS1=0.0) noise=0.0;
 noise=0.0; 
 for(k=0; k<H.nchan; k++) {
   ii=Getindex(k,ihorn,H.nchan); 
   TT[ii]=noise*(float)gasdev(&idum)+(float)ihorn+1.0;
   if (WSUM !=0.0) TT[ii]+=S[k]/WSUM;
 }
 free(I);
 free(S);
}

void DoCALMeasurement()
{
  fprintf(stderr,"\n CAL measurement");
  X=xref; Y=yref; ttotal+=tslew2ref; HA+=tslew2ref/3600.;
  ttotal+=tcal; HA+=tcal/3600.;


  H.tsys[0]=300; H.tsys[1]=307; H.tsys[2]=286; H.tsys[3]=290;
  H.tsys[4]=307; H.tsys[5]=309; H.tsys[6]=317; H.tsys[7]=304;
  H.tsys[8]=318; H.tsys[9]=341; H.tsys[10]=272; H.tsys[11]=325;
  H.tsys[12]=348; H.tsys[13]=282; H.tsys[14]=281; H.tsys[15]=329;
}
void DoREFMeasurement(TT)
float *TT;
{
 int i,ii,k; 

 fprintf(stderr,"\n REF measurement");
 if (X!=xref || Y!=yref) {
  ttotal+=tslew2ref; HA+=tslew2ref/3600.;
 }
 X=xref; Y=yref; 
 ttotal+=tref;
 HA+=tref/3600.;
 ttotal+=tslew2ref; 
 HA+=tslew2ref/3600.;
 DSUM+=H.nhorns*H.nchan*sizeof(float);
 for (i=0; i<H.nhorns; i++) {
  for(k=0; k<H.nchan; k++) {
    ii=Getindex(k,i,H.nchan); 
    TT[ii]=(float)i+1.0;
/*
    TT[ii]=(float)i+1.0-0.005*(float)k-1.5e-4*pow(k,2.0)+6.0e-7*pow(k-128,3.0);
*/
  }
 }
}

void CreateScanFile()
{
 char filename[64];
 int day,mo,year;
 long offset;

   
 day=20; mo=11; year=2001;
 if (day < 10) {
   sprintf(filename,"%1.0f%1X0%1d%4.0f.LOA",
     fmod(year,2000),mo,day,fmod(H.iscan,10000));
 }
 else   sprintf(filename,"%1.0f%1X%2d%4.0f.LOA",
     fmod(year,2000),mo,day,fmod(H.iscan,10000));
 
 if ( (otffp=fopen(filename,"wb"))==NULL) {
   fprintf(stderr,"\n Can not open scan file %s\n", filename); 
   exit(1);
 }
/* Position file beyond the header */
 offset=HEADERSIZE+POSITIONHEADERSIZE;
 fseek(otffp,offset,SEEK_SET);


}

void WriteDATA(TT)
float *TT; 
{
  int ii,ihorn;
  long nel,nmem,pos;
  

  nmem=H.nhorns*H.nchan;
  fwrite(TT,sizeof(float),nmem, otffp);
}

