#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <cpgplot.h>
#include "fitsio.h"
#include "otf.h"


FILE *fp;
fitsfile *fptr;
int anynul,nos;
int idum=-1;
int kmap;
int maxdim=3;
long naxes[3],anaxes[3];
long nelements,fpixel=1,naxis=2;
float dnu=50.0e+03;
float TDUMPMIN=0.1,MAXTIMEREF=120.0,RateMAX=60.;
float RMS,RMS1,TDUMP=0.1;
float DSUM;
float crval11,crval22,crpix1,crpix2,cdelt1,cdelt2;
float eta=2.71;
float N_c=16.0;
float tref,tslew2ref=10.0;
float tcal=15.0;
float xa[32],ya[32];
float tsysAV;
float *sigma,*WT;
float XA,YA,X,Y,XE,XS,YE,YS,xs,xe,r,DX,dx,dy,tscan,ttotal;
float xref,yref;
float SCAN_ANGLE,LX,LY,XOFF,YOFF,ROWSTEP; 
double XCENTER,YCENTER;
double LAT=0.73987416;   /* FCRAO latitude in radians */
double HA;

void DoCALMeasurement();
void DoREFMeasurement(float*);
void CreateScanFile();
void FindRampSize(float,float,float*,float*);
void printerror(int);
void convolve_1(int,float,float);
double gasdev(int*);
double GetAngle(double,float,float);

main(argc,argv)
int argc;
char *argv[];
{
 char infitsfile[64],MAPCOORD[8];
 char comment[64];
 char keyname[16];
 char apos[64],*b;
 int k,ier,i,icol,idmp,ndmp,irow,ix,iy,itdump;
 int rmsstore,iscan1,NRPO,NRPC;
 int just,axis;
 float x1,y1,x2,y2,xsv[MAXSCANSIZE],ysv[MAXSCANSIZE];
 float RAMPX[2],RAMPY[2];
 float s1,s0,xmin,xmax,ymin,ymax,scan_angle;
 float R,tcell;
 float tr[6];
 float t,x,y;
 float dxx,dyy;
 float xminmap,xmaxmap,yminmap,ymaxmap;
 float CRVAL1,CRVAL11,CRVAL2,CRVAL22;
 float BX,rampx,rampy;
 float rah,ram,ras,dd,dm,ds,RA,DEC;
 float TMAX,TMIN;
 float x0,y0;
 double a,pa,ha,HAS;
  
/* Get input parameters */
  if (argc < 16) {
    fprintf(stderr,"\n OTF PARAMETERS: ");
    fprintf(stderr,"\n RA/L position of center position in degrees");
    fprintf(stderr,"\n DEC/B position of center position in degrees ");
    fprintf(stderr,"\n SCAN COORDINATE (equa,gala) ");
    fprintf(stderr,"\n SCAN ANGLE");
    fprintf(stderr,"\n XOFF ");
    fprintf(stderr,"\n YOFF");
    fprintf(stderr,"\n LX (arcminutes)");
    fprintf(stderr,"\n LY (arcminutes)");
    fprintf(stderr,"\n RAMPX (beams)");
    fprintf(stderr,"\n RAMPY (beams)");
    fprintf(stderr,"\n NRPO rows per off (integer)");
    fprintf(stderr,"\n NRPC rows per cal (integer)");
    fprintf(stderr,"\n TSAMPLE (secs)");
    fprintf(stderr,"\n ROWSTEP (beams) ");
    fprintf(stderr,"\n Starting Hour Angle (hrs)");
    fprintf(stderr,"\n Cell size (arcsec) ");
    fprintf(stderr,"\n Tsys (K) ");
    fprintf(stderr,"\n Convolving Function (1:4) ");
    fprintf(stderr,"\n Exiting\n");
    exit(0);
  } 
  sscanf(argv[3],"%s",MAPCOORD);
  if (!strncmp(MAPCOORD,"equa",4)) {
    kmap=2;
    b=strtok(argv[1],":");
    rah=atof(b);
    b=strtok(NULL,":");
    ram=atof(b);
    b=strtok(NULL,":");
    ras=atof(b);
    RA=15.0*(rah+ram/60.0+ras/3600.);

    b=strtok(argv[2],":");
    dd=atof(b);
    b=strtok(NULL,":");
    dm=atof(b);
    b=strtok(NULL,":");
    ds=atof(b);
    DEC=dd+dm/60.+ds/3600.;
    XCENTER=RA;  YCENTER=DEC;
  }
  else if (!strncmp(MAPCOORD,"gala",4)) {
   kmap=3;
   XCENTER=atof(argv[1]);
   YCENTER=atof(argv[2]);
  }
  else {
    fprintf(stderr,"\n Unrecognized map coordinate");
    fprintf(stderr,"\n  --  only \"equa\" or \"gala\" allowed ");
    exit(1);
  } 

  fprintf(stderr,"\n %f %f", XCENTER,YCENTER); 
  SCAN_ANGLE=atof(argv[4]);
  XOFF=atof(argv[5]);
  YOFF=atof(argv[6]);
  LX=atof(argv[7]);
  LY=atof(argv[8]);
  rampx=atof(argv[9]);
  rampy=atof(argv[10]);
  NRPO=atoi(argv[11]);
  NRPC=atoi(argv[12]);
  TDUMP=atof(argv[13]);
  ROWSTEP=atof(argv[14]);
  ROWSTEP=fabs(ROWSTEP);
  HAS=atof(argv[15]);
  BX=0.738;
  THETA_N=60.0*BX/2.0;
  rmsstore=0;
  if (argc > 16) {
    THETA_N=atof(argv[16]);
    tsysAV=atof(argv[17]);
    kopt=atoi(argv[18]);
    rmsstore=1;
  }

  scan_angle=SCAN_ANGLE*3.1415927/180.;

   h.H.nhorns=16;
/* SEQUOIA positions relative to pixel 11 */
  xa[0]=-2.952; xa[1]=-1.476; xa[2]=0.0; xa[3]=1.476;
  xa[4]=-2.952; xa[5]=-1.476; xa[6]=0.0; xa[7]=1.476;
  xa[8]=-2.952; xa[9]=-1.476; xa[10]=0.0; xa[11]=1.476;
  xa[12]=-2.952; xa[13]=-1.476; xa[14]=0.0; xa[15]=1.476;
  ya[0]=2.952; ya[1]=2.952; ya[2]=2.952; ya[3]=2.952;
  ya[4]=1.476; ya[5]=1.476; ya[6]=1.476; ya[7]=1.476;
  ya[8]=0.0; ya[9]=0.0; ya[10]=0.0; ya[11]=0.0;
  ya[12]=-1.476; ya[13]=-1.476; ya[14]=-1.476; ya[15]=-1.476;



  HA=HAS;
  ha=15.0*HA*3.1415927/180.;
  RAMPX[0]=rampx*BX; RAMPX[1]=RAMPX[0]; 
  RAMPY[0]=rampy*BX; RAMPY[1]=RAMPY[0]; 

/* Do a CAL to estimate OTF paramters from mean Tsys */
  DoCALMeasurement();
  nos=2.0;
  itdump=(TDUMP/0.05);
  TDUMP=0.05*itdump;

  if (TDUMP < TDUMPMIN) {
     fprintf(stderr,"\n Cannot not run the correlators faster than 10 Hz");
     fprintf(stderr,"\n Setting TDUMP=0.1 secs"); 
     TDUMP=0.1;
     itdump=(TDUMP/0.05);
     TDUMP=0.05*itdump;
   }
   h.H.nsample=60.0*NRPO*nos*(LX+RAMPX[0]+RAMPX[1])/(THETA_N);
   if (h.H.nsample*+2 > MAXSCANSIZE) {
     fprintf(stderr,"\n Scan Length exceeds maximum (%d)",MAXSCANSIZE);
     exit(1);
   }
   if (h.H.nsample*TDUMP > MAXTIMEREF) {
     fprintf(stderr,"\n Time between OFF measurements exceeds recommended maximum%4.0f secs",MAXTIMEREF);
     fprintf(stderr,"\n Increase target rms and make repeated maps\n");
     exit(1);
   }
   tref=0.5*TDUMP*sqrt(h.H.nsample);
   R=THETA_N/(60.0*nos*TDUMP);  /* antenna rate in arcmin/sec */
   if (R > RateMAX) {
     fprintf(stderr,"\n Antenna rate exceeds max.");
     fprintf(stderr,"\n Reduce TDUMP ");
     exit(1);
   }
   if (h.H.nsample > MAXSCANSIZE) {
     fprintf(stderr,"\n Number of samples is too large (>%d) -- exitting\n",MAXSCANSIZE);
     exit(1);
   }
/* Establish some basic header information */
   strcpy(h.H.name,"otfsim");
   strcpy(h.H.bunit," K ");
   strcpy(h.H.vlabel," VLSR ");
   if (kmap == 2) {
     strcpy(h.H.xlabel,"RA--SIN");
     strcpy(h.H.ylabel,"DEC--SIN");
   }
   if (kmap == 3) {
     strcpy(h.H.xlabel,"GLON");
     strcpy(h.H.ylabel,"GLAT");
   }
   iscan1=2001;
   h.H.iscan=iscan1;
   h.H.nchan=1024; 
   h.H.NOS=nos;
   h.H.otfdatarev=1.0;
   h.H.vlsr=-10.0;
   h.H.epoch=2000.0;
   h.H.telrate=R;
   h.H.tdump=TDUMP;
   h.H.f0=110.20137;

   fprintf(stderr,"\n nos=%6.2f ",h.H.NOS);
   fprintf(stderr,"\n DUMP RATE=%4.2f HZ",1.0/TDUMP);
   fprintf(stderr,"\n R=%6.2f arcmin/sec",R);
   fprintf(stderr,"\n <TSYS>=%6.0f K",tsysAV);
   fprintf(stderr,"\n h.H.nsample=%4d ",h.H.nsample);
   fprintf(stderr,"\n tref=%6.2f sec",tref);
   fprintf(stderr,"\n Estimated time between OFF measurements=%6.2f sec",2.*tslew2ref+h.H.nsample*TDUMP);
   fprintf(stderr,"\n Estimated time between CAL measurements=%6.2f sec",2.*tslew2ref+h.H.nsample*TDUMP*(float)(NRPC/NRPO));
  
   
   xminmap=1.0e+20; xmaxmap=-1.0e+20;
   yminmap=1.0e+20; ymaxmap=-1.0e+20;
   YS=YOFF*LY+LY/2.0; YE=YOFF*LY-LY/2.0;
   XS=XOFF*LX+LX/2.0; XE=XOFF*LX-LX/2.0;
   x1=XS*cos(scan_angle)+YS*sin(scan_angle);
   y1=YS*cos(scan_angle)-XS*sin(scan_angle);
   if (x1 < xminmap) xminmap=x1;
   if (x1 > xmaxmap) xmaxmap=x1;
   if (y1 < yminmap) yminmap=y1;
   if (y1 > ymaxmap) ymaxmap=y1;
   
   x1=XS*cos(scan_angle)+YE*sin(scan_angle);
   y1=YE*cos(scan_angle)-XS*sin(scan_angle);
   if (x1 < xminmap) xminmap=x1;
   if (x1 > xmaxmap) xmaxmap=x1;
   if (y1 < yminmap) yminmap=y1;
   if (y1 > ymaxmap) ymaxmap=y1;
   x1=XE*cos(scan_angle)+YE*sin(scan_angle);
   y1=YE*cos(scan_angle)-XE*sin(scan_angle);
   if (x1 < xminmap) xminmap=x1;
   if (x1 > xmaxmap) xmaxmap=x1;
   if (y1 < yminmap) yminmap=y1;
   if (y1 > ymaxmap) ymaxmap=y1;
   x1=XE*cos(scan_angle)+YS*sin(scan_angle);
   y1=YS*cos(scan_angle)-XE*sin(scan_angle);
   if (x1 < xminmap) xminmap=x1;
   if (x1 > xmaxmap) xmaxmap=x1;
   if (y1 < yminmap) yminmap=y1;
   if (y1 > ymaxmap) ymaxmap=y1;

   crval1=xmaxmap; 
   crval11=xminmap;
   crval2=yminmap; 
   crval22=ymaxmap;
   naxes[0]=60.0*(crval1-crval11)/THETA_N;
   naxes[1]=60.0*(crval22-crval2)/THETA_N;

   xminmap=1.0e+20; xmaxmap=-1.0e+20;
   yminmap=1.0e+20; ymaxmap=-1.0e+20;
   YS=YOFF*LY+LY/2.0+RAMPY[0]; YE=YOFF*LY-LY/2.0-RAMPY[1];
   XS=XOFF*LX+LX/2.0+RAMPX[0]; XE=XOFF*LX-LX/2.0-RAMPX[1];
   x1=XS*cos(scan_angle)+YS*sin(scan_angle);
   y1=YS*cos(scan_angle)-XS*sin(scan_angle);
   if (x1 < xminmap) xminmap=x1;
   if (x1 > xmaxmap) xmaxmap=x1;
   if (y1 < yminmap) yminmap=y1;
   if (y1 > ymaxmap) ymaxmap=y1;
   
   x1=XS*cos(scan_angle)+YE*sin(scan_angle);
   y1=YE*cos(scan_angle)-XS*sin(scan_angle);
   if (x1 < xminmap) xminmap=x1;
   if (x1 > xmaxmap) xmaxmap=x1;
   if (y1 < yminmap) yminmap=y1;
   if (y1 > ymaxmap) ymaxmap=y1;
   x1=XE*cos(scan_angle)+YE*sin(scan_angle);
   y1=YE*cos(scan_angle)-XE*sin(scan_angle);
   if (x1 < xminmap) xminmap=x1;
   if (x1 > xmaxmap) xmaxmap=x1;
   if (y1 < yminmap) yminmap=y1;
   if (y1 > ymaxmap) ymaxmap=y1;
   x1=XE*cos(scan_angle)+YS*sin(scan_angle);
   y1=YS*cos(scan_angle)-XE*sin(scan_angle);
   if (x1 < xminmap) xminmap=x1;
   if (x1 > xmaxmap) xmaxmap=x1;
   if (y1 < yminmap) yminmap=y1;
   if (y1 > ymaxmap) ymaxmap=y1;

   CRVAL1=xmaxmap; 
   CRVAL11=xminmap;
   CRVAL2=yminmap; 
   CRVAL22=ymaxmap;

   if (kmap == 3) {
    CRVAL1=CRVAL1/60.;  CRVAL11=CRVAL11/60.;
    CRVAL2=CRVAL2/60.;  CRVAL22=CRVAL22/60.;
   }

   fprintf(stderr,"\n Convolved Image %d x %d", naxes[0],naxes[1]);
   fprintf(stderr,"\n Ramp Sizes: %5.2f %5.2f %5.2f %5.2f", RAMPX[0],RAMPX[1],
                   RAMPY[0],RAMPY[1]);

   if ( (sigma = (float *)malloc(sizeof(float)*naxes[0]*naxes[1]))==NULL) {
     fprintf(stderr,"\n Can not allocate memory for data array\n");
     exit(3);
   }
   if ( (WT = (float *)malloc(sizeof(float)*naxes[0]*naxes[1]))==NULL) {
     fprintf(stderr,"\n Can not allocate memory for data array\n");
     exit(3);
   }
/* Make lookup table for convolving kernel */
 MakeWeights();

   fprintf(stderr,"\n Type any key to continue: ");
   getchar();

   tr[0]=CRVAL1; tr[1]=-THETA_N/60.; tr[2]=0.0;
   tr[3]=CRVAL2; tr[4]=0.0; tr[5]=-tr[1];
   if (kmap == 3) {
    tr[1]=tr[1]/60.;  tr[5]=tr[5]/60.;
   }

/* Open graphics device. */
   cpgopen("/xser");
   just=1; axis=0;
   dx=fabs(CRVAL1-CRVAL11); dy=fabs(CRVAL2-CRVAL22);
   cpgenv(CRVAL1+0.2*dx,CRVAL11-0.2*dx,CRVAL2-0.2*dy,
        CRVAL22+0.2*dy,just,axis);
   if (kmap == 2) cpglab("x (arcmin)", "y (armin)", "OTF Points");
   if (kmap == 3) cpglab("DEL Long. (deg)", "DEL Lat. (deg)", "OTF Points");

/* Create first scan file and do the initial REF measurement */
   CreateScanFile();
   DoREFMeasurement(WT);

/* Start the map */
   y0=YS;
   irow=0;
   icol=0; 
   idmp=0;
   ttotal=0.0;
   DSUM=0.0;
   while(y0 > YE) {
     Y=y0;
     irow++;
     fprintf(stderr,"\n Row: %4d  ", irow);
     if (fmod((double)irow,2.0)==1.0) {
       xs=XS; xe=XE; r=R;
     }
     if (fmod((double)irow,2.0)==0.0) {
       xs=XE; xe=XS; r=-R;
     }
     DX=fabs(xe-xs);
     dx=0.0;
     x0=xs; 
     tscan=0.0;
     while(dx <= DX) {
      icol++;
      idmp++;
      DSUM+=h.H.nhorns*h.H.nchan*4.0;
      tscan+=TDUMP;
      x0-=r*TDUMP;
      X=x0*cos(scan_angle)+y0*sin(scan_angle);
      Y=y0*cos(scan_angle)-x0*sin(scan_angle);
      dx+=fabs(R*TDUMP);
/* Update angle  */
      HA+=TDUMP/3600.;
      ha=15.0*HA*3.1415927/180.;
      YA=YCENTER+Y/60.0;
      XA=XCENTER+X/60.0/cos(YA);
      a=GetAngle(ha,XA,YA);
      for(i=0; i<h.H.nhorns; i++) {
       dxx=BX+xa[i]-xa[10];
       dyy=-BX+ya[i]-ya[10];
       x1=X-dxx*cos(a)+dyy*sin(a);
       y1=Y+dxx*sin(a)+dyy*cos(a);
       if (rmsstore) convolve_1(i,x1,y1);
       if (kmap == 3) {
        x1=x1/60.;  y1=y1/60.;
       } 
       cpgsci(1);
       if (i==10) {
         cpgsci(2);
         xsv[idmp-1]=x1; ysv[idmp-1]=y1;
       }
       cpgpt1(x1,y1,-2);

      }  /* horn loop */

     }   /* row loop */

     if (fmod((double)irow,(double)NRPO)==0.0 && y0 > YE) {
      DoREFMeasurement(WT);

      h.H.iscan=h.H.iscan+1;
      if (h.H.iscan > 30000) h.H.iscan=1001;
      CreateScanFile();
      if (fmod((double)irow,(double)NRPC)==0.0) {
       DoCALMeasurement();
       DoREFMeasurement(WT);    /*  a new ref measure is required after a CAL */
       icol=0; 
      }
     }

     y0-=ROWSTEP*2.0*THETA_N/60.;   /*   y0-=THETA_N/60.;   */
     ndmp=idmp;
     for(idmp=0; idmp<ndmp; idmp++) cpgpt1(xsv[idmp],ysv[idmp],-2);
     idmp=0;
     ttotal+=tscan;
   }   /* MAP is finished */

/* Be sure to end with a OFF measurement  */
   if (fmod((double)irow,(double)NRPO)!=0.0) {
     DoREFMeasurement(WT);
   }

/* Draw box around LX x LY area */
   cpgsci(4);
   YS=YOFF*LY+LY/2.0; YE=YOFF*LY-LY/2.0;
   XS=XOFF*LX+LX/2.0; XE=XOFF*LX-LX/2.0;
   x1=XS*cos(scan_angle)+YS*sin(scan_angle);
   y1=YS*cos(scan_angle)-XS*sin(scan_angle);
   if (kmap == 3){
      x1=x1/60.;  y1=y1/60;
   }
   cpgmove(x1,y1);
   x2=XS*cos(scan_angle)+YE*sin(scan_angle);
   y2=YE*cos(scan_angle)-XS*sin(scan_angle);
   if (kmap == 3){
      x2=x2/60.;  y2=y2/60;
   }
   cpgdraw(x2,y2);
   x2=XE*cos(scan_angle)+YE*sin(scan_angle);
   y2=YE*cos(scan_angle)-XE*sin(scan_angle);
   if (kmap == 3){
      x2=x2/60.;  y2=y2/60;
   }
   cpgdraw(x2,y2);
   x2=XE*cos(scan_angle)+YS*sin(scan_angle);
   y2=YS*cos(scan_angle)-XE*sin(scan_angle);
   if (kmap == 3){
      x2=x2/60.;  y2=y2/60;
   }
   cpgdraw(x2,y2);
   cpgdraw(x1,y1);

   if (rmsstore) {

   for(iy=0; iy<naxes[1]; iy++) {
    for(ix=0; ix<naxes[0]; ix++) {
      i=Getindex(ix,iy,naxes[0]);
      if (sigma[i]!=0.0 && WT[i]!=0.0) {
        sigma[i]=1.0/sqrt(dnu*sigma[i]);
      }
      else sigma[i]=0.0;
    }
   }
 
   if (fits_create_file(&fptr, "!rms.fits", &status))
         printerror( status );
   if (fits_create_img(fptr,FLOAT_IMG,naxis,naxes,&status))
         printerror( status );
    fpixel=1;
    nelements=naxes[0]*naxes[1];
    if (fits_write_img(fptr, TFLOAT,fpixel,nelements,&sigma[0], &status))
     printerror( status );



 fits_write_date(fptr,&status);
 strcpy(keyname,"OBJECT"); strcpy(comment," ");
 fits_write_key(fptr,TSTRING,keyname,&h.H.name[0],comment,&status);
 strcpy(keyname,"ORIGIN"); strcpy(comment," ");
 fits_write_key(fptr,TSTRING,keyname,"FCRAO-OTF",comment,&status);
 strcpy(keyname,"TELESCOP"); strcpy(comment," ");
 fits_write_key(fptr,TSTRING,keyname,"FCRAO-14M",comment,&status);
 strcpy(keyname,"INSTRUME"); strcpy(comment," ");
 fits_write_key(fptr,TSTRING,keyname,"SEQUOIA",comment,&status);

 strcpy(keyname,"CTYPE1"); strcpy(comment," ");
 fits_write_key(fptr,TSTRING,keyname,&h.H.xlabel[0],comment,&status);
 strcpy(keyname,"CRVAL1"); strcpy(comment,"DEGREES");
 fits_write_key(fptr,TDOUBLE,keyname,&XCENTER,comment,&status);
 strcpy(keyname,"CRPIX1"); strcpy(comment," ");
 crpix1=crval1/(THETA_N/60.0);
 fits_write_key(fptr,TFLOAT,keyname,&crpix1,comment,&status);
 strcpy(keyname,"CDELT1"); strcpy(comment,"DEGREES");
 cdelt1=-THETA_N/3600.;
 fits_write_key(fptr,TFLOAT,keyname,&cdelt1,comment,&status);

 strcpy(keyname,"CTYPE2"); strcpy(comment," ");
 fits_write_key(fptr,TSTRING,keyname,h.H.ylabel,comment,&status);
 strcpy(keyname,"CRVAL2"); strcpy(comment,"DEGREES");
 fits_write_key(fptr,TDOUBLE,keyname,&YCENTER,comment,&status);
 crpix2=crval2/(THETA_N/60.0);
 strcpy(keyname,"CRPIX2"); strcpy(comment," ");
 fits_write_key(fptr,TFLOAT,keyname,&crpix2,comment,&status);
 strcpy(keyname,"CDELT2"); strcpy(comment," ");
 cdelt2=THETA_N/3600.;
 fits_write_key(fptr,TFLOAT,keyname,&cdelt2,comment,&status);

   if (fits_close_file(fptr,&status))
         printerror( status );
}  /* clse rmsstore condition */


/*
   fprintf(stderr,"\n Closing graphics window ");
   cpgclos();
*/

   fprintf(stderr,"\n\n ******************************************"); 
   fprintf(stderr,"\n TOTAL TIME        :%6.1f minutes",ttotal/60.);
   fprintf(stderr,"\n RAW DATA COLLECTED:%5.1f Mbytes ",DSUM/1.0e+06); 
   fprintf(stderr,"\n REGRIDDED DATA    :%5.1f Mbytes ",
              (float)naxes[0]*naxes[1]*4.0*h.H.nchan/1.0e+06); 
   fprintf(stderr,"\n ******************************************\n\n"); 
}

double GetAngle(HA,X,Y)
double HA;
float X,Y;
{
 double XR,YR;
 double A,ypa, xpa, PA;
 double ra1,dec1,xga,yga,dga;
 double gzra=4.649646134;
 double gzdec=-5.050315348;
 double gpra=3.3660333150;
 double gpdec=0.4734787069;
 double eqzl=1.681402682;
 double eqzb=-1.050486996;
 double eqpl=2.145566646;
 double eqpb=0.4734787069;

 XR=X/57.29578;  YR=Y/57.29578; 
 ypa = sin(HA);
 xpa = cos(YR)*tan(LAT) - sin(YR)*cos(HA);
 PA = atan2(ypa,xpa);
 switch (kmap) {
   case 2:
     A=PA;
     break;
   case 3:

/*   convert current l,b coordinates to ra,dec j2000 */
     coord(eqzl,eqzb,eqpl,eqpb,XR,YR,&ra1,&dec1);

     yga=sin(ra1-gpra);
     xga=cos(dec1)*tan(gpdec)-sin(dec1)*cos(ra1-gpra);
     dga=atan2(yga,xga);
     A=PA+dga;
     ra1=ra1*57.29578; dec1=dec1*57.29578;
     
     break;
  }

  return (A);
}

void printerror( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
    {
       fits_report_error(stderr, status); /* print error report */

       exit( status );    /* terminate the program, returning error status */
    }
    return;
}

void FindRampSize(ha,scan_angle,RAMPX,RAMPY)
float ha,scan_angle,*RAMPX,*RAMPY;
{
  int i;
  float x1,y1,xoff,yoff,xmin,xmax,ymin,ymax;
  double a;
  
  yoff=(YOFF*LY+LY/2.0)/60.0;
  xoff=(XOFF*LX+LX/2.0)/60.0/cos(0.017453293*(YCENTER+yoff));
  YA=YCENTER+(yoff*cos(scan_angle)-xoff*sin(scan_angle));
  XA=XCENTER+(xoff*cos(scan_angle)+yoff*sin(scan_angle));
  a=GetAngle(ha,XA,YA);

/* Find the extreme, rotated positions within the array at this angle */
  xmin=1.0e+20; xmax=-1.0e+20; ymin=1.0e+20; ymax=-1.0e+20;
  for(i=0; i<h.H.nhorns; i++) {
    x1=xa[i]*cos(a)-ya[i]*sin(a);
    if (x1 < xmin) xmin=x1;
    if (x1 > xmax) xmax=x1;

    y1=xa[i]*sin(a)+ya[i]*cos(a);
    if (y1 < ymin) ymin=y1;
    if (y1 > ymax) ymax=y1;
  }
/* Set default ramp offset */
/*
   RAMPX[0]=fabs(xmin);
   RAMPX[1]=fabs(xmax);
   RAMPY[0]=fabs(ymin);
   RAMPY[1]=fabs(ymax);
*/
   RAMPX[0]=5.0; RAMPX[1]=5.0; RAMPY[0]=5.0; RAMPY[1]=5.0;

}

void convolve_1(ihorn,x1,y1)
int ihorn;
float x1,y1;
{
int ii,ix,iy;
float arg,deltax,deltay,x,y,radius;
double wt;

 for(iy=0; iy<naxes[1]; iy++) {
  y=crval2+iy*THETA_N/60.;
  deltay=60.0*fabs(y-y1);
  for(ix=0; ix<naxes[0]; ix++) {
   x=crval1-ix*THETA_N/60.;
   deltax=60.0*fabs(x-x1);
   radius=sqrt(pow(deltax,2.0)+pow(deltay,2.0));
   arg=radius/(2.0*THETA_N);
   if (arg < RMAX) {
     wt=GetWeight(arg);
     ii=Getindex(ix,iy,naxes[0]);
     sigma[ii]+=(float)wt*TDUMP/pow(tsysAV,2.);
     WT[ii]+=wt;
   }
  }
 }
}

void DoCALMeasurement()
{
  fprintf(stderr,"\n CAL measurement");
  X=xref; Y=yref; ttotal+=tslew2ref; HA+=tslew2ref/3600.;
  ttotal+=tcal; HA+=tcal/3600.;
}

void DoREFMeasurement(TT)
float *TT;
{
 int i,ii,k; 

 fprintf(stderr,"\n REF measurement");
 if (X!=xref || Y!=yref) {
  ttotal+=tslew2ref; HA+=tslew2ref/3600.;
 }
 X=xref; Y=yref; 
 ttotal+=tref;
 HA+=tref/3600.;
 ttotal+=tslew2ref; 
 HA+=tslew2ref/3600.;
 DSUM+=h.H.nhorns*h.H.nchan*sizeof(float);
}

void CreateScanFile()
{
 char filename[64];

 sprintf(filename,"%s.%d.otf",h.H.name,h.H.iscan);
 fprintf(stdout,"\nCreating scan %s\n",filename);
}
