#include "otftool.h"
#include "classio.h"
#include <cpgplot.h>

extern int config_parse_defaults (void);
extern void setLO (int);
extern GtkWidget* otftool_setup_logging (otf_gui *);
extern GtkWidget* otftool_setup_status (otf_gui *);
extern GtkWidget* otftool_setup_menu (otf_gui *);
extern char* timestring (void);

COM_DEF Com;
CARCOM_DEF Carcom;
IND_DEF Ind;
RSCAN_DEF Rscan; 
CRDATA_DEF Crdata;      
CRPAR_DEF Crpar;        
SASSET_DEF Sasset; 
SASCET_DEF Sascet; 



int 
main(int  argc,
     char *argv[])
{
  int i;
  time_t t1;
  struct stat statbuf;
  char rcfile[MAXPATHLEN];
  char *rawpath;
  char *tmpstr,temppath[MAXPATHLEN];
  FILE *fplog=NULL;

  /* init threads */
  g_thread_init(NULL);


  sprintf (rcfile, "%s/otftoolrc",DATADIR);
  /*  printf("%s\n",rcfile);*/
  if ( stat(rcfile, &statbuf) == 0) {
    gtk_rc_add_default_file (rcfile);
  } else {
    fprintf(stderr, "Can't locate RC file for otftool\n");
    /*    exit (3);*/
  }

  strcpy(calfile,"");
  gtk_init(&argc, &argv);
  
  if (argc > 1) {
    i=1;
    if (!strcmp(argv[i],"-calfile")) {
      i++;
      sscanf(argv[i], "%s", calfile);
      fprintf(stderr, "Using calfile = %s\n", calfile);
    }
  }
  if  ((otffile = (char *) malloc(100*sizeof(char)))  == NULL){
    perror("otffile text error: malloc");
    exit(3);
  }

  if  ((outfile = (char *) malloc(100*sizeof(char)))  == NULL){
    perror("outfile text error: malloc");
    exit(3);
  }

  if  ((logtext_string = (char *) malloc(2000*sizeof(char)))  == NULL){
    perror("logtext_string text error: malloc");
    exit(3);
  }

  if ((infile = (char **) malloc (sizeof (char *))) == NULL) {
    perror("infile text error: malloc");
    exit(3);
  } 

  if ((sname = (char *) malloc (12*sizeof(char))) == NULL) {
    perror("sname text error: malloc");
    exit(3);
  }

  if ((linename_sav = (char *) malloc (12*sizeof(char))) == NULL) {
    perror("linename_sav text error: malloc");
    exit(3);
  }


  strcpy(outfile,"");
  strcpy(otffile,"");
  /*  otffile = NULL;
      outfile = NULL;*/


  strcpy(linename,"");
  strcpy(WTmode,"equal");
  IF=-1;
  kopt=1;
  THETA_N=0.0; 
  out_type=1;
  CmdLineMASK=-1; 
  statusint = 0; /*menu will automatically turn this on*/
  logint = 0;
  log_mode = LOG_VERBOSE;
  ORDER = 1;
  NWIND = 1;
  wind_type = 1;
  assess_mode = 0;
  rmsweight = 1;
  tsysweight = 0;
  otfedit_exit = 1;
  lo_sel = 0;
  filter.startscan = 0;
  filter.endscan = 31000;
  include_margin = 0;
  filter.f0 = 0.;
  filter.deltaf = 0.;
  bw_sel = 0;
  main_window_width = 1020;
  main_window_height = 658;
  rmsthreshold = 1.0e35;

  weight_mode = WT_EQUAL;
  IF = 0;
  out_type = 2;

  for (i = 0; i<MAXHORNS; i++) horn_is_active[i] = 1;
  for (i = 0; i<MAXWIND; i++) {
    W1[i] = -1000.0; W2[i] = -1000.0;
  }

  /* Make default start date Jan.1, 2001 */
  start_tm.tm_year = 101; start_tm.tm_mon = 0; start_tm.tm_mday = 1;
  /* Get current date*/
  t1 = time(NULL);
  end_tm = *gmtime(&t1);

  if (config_parse_defaults () == 0) {
    strcpy(filter.sname,sname);
    strcpy(linename,linename_sav);
  }

  otfgui = g_malloc(sizeof(*otfgui));
  baselinewgt = g_malloc(sizeof(*baselinewgt));

  if ((rawpath = getenv("RAWPATH")) == NULL) 
    sprintf(rawbasepath,"%s",RAWPATH);
  else 
    strcpy(rawbasepath,rawpath);
  
  /* remove any unnneccessary trailing "/" */
  tmpstr = rindex(rawbasepath, '/');
  if (strlen(tmpstr) == 1) {
    strncpy(temppath,rawbasepath,(size_t) (strlen(rawbasepath) - 1));
    strcpy(rawbasepath,temppath);
  }
  /*  printf("%s\n",pathname);*/


  setLO(lo_sel);

  otfgui->mainwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (otfgui->mainwindow, "main window");
  gtk_window_set_title (GTK_WINDOW (otfgui->mainwindow), "otftool");
  gtk_signal_connect (GTK_OBJECT (otfgui->mainwindow), "delete_event",
                      GTK_SIGNAL_FUNC (delete_event), NULL);
  gtk_signal_connect (GTK_OBJECT (otfgui->mainwindow), "destroy",
		      GTK_SIGNAL_FUNC (destroy), NULL);
  gtk_container_set_border_width (GTK_CONTAINER (otfgui->mainwindow), 10);
  gtk_window_set_default_size (GTK_WINDOW (otfgui->mainwindow), main_window_width, 
		      main_window_height );
  gtk_widget_realize(otfgui->mainwindow);   

  otfgui->vertbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (otfgui->mainwindow), otfgui->vertbox);

  otfgui->logwin = GTK_WIDGET (otftool_setup_logging(otfgui));
  otfgui->statuswin = GTK_WIDGET (otftool_setup_status(otfgui));


  otfgui->menu = GTK_WIDGET (otftool_setup_menu (otfgui));
  otftool_setup_body (otfgui);






  gtk_widget_show (otfgui->vertbox);
  gtk_widget_show (otfgui->mainwindow);

  /*simply initialize log file for output everytime otftool is run */
  if ((fplog = fopen(log_file,"w")) == NULL) {
    fprintf(stderr,"\n\n Warning - Can't open %s for writing; no logs will be written out\n", log_file);
  } else {
    fclose(fplog);
  }

  append_log(0,1,1,otfgui->logtext,(char *)timestring());
  append_log(0,1,1,otfgui->logtext,"Finished setting up otftool\n");



  /*  start_progressbar();*/
  gdk_threads_enter();
  gtk_main();
  gdk_threads_leave();
  /*  gtk_main();*/
  
  return(0);
}
