/* otftool_setup_body.c
 * Sets up the body of otftool GUI 
 * Gopal Narayanan <gopal@astro.umass.edu>
 *
 *
 * $Author: gopal $
 * $Date: 2004/02/11 15:27:18 $
 * $Log: otftool_setup_body.c,v $
 * Revision 1.20  2004/02/11 15:27:18  gopal
 * Minor changes in debug printout and malloc
 *
 * Revision 1.19  2003/01/31 05:00:18  gopal
 * Fixed bug in examine/edit scan that caused otftool to crash and burn. Thanks to John for pointing this problem
 *
 * Revision 1.18  2002/12/24 04:20:21  gopal
 * Fixed bug in Otf file list selections in Data Assessment tab where it claimed input otf file was not found
 *
 * Revision 1.17  2002/12/04 03:11:23  gopal
 * List of Old otf files no longer on disk can be saved to restore file for administrator to restore. Restore button now available in data select tab. A whole bunch of changes globally to accomodate different raw directory locations. gtk clist now stores data in each row corresponding to dirname
 *
 * Revision 1.16  2002/07/22 18:35:17  gopal
 * Fixed some bugs in otffile, outfile memory allocation; baseline window code cleaned up, more verbose logging in some places
 *
 * Revision 1.15  2002/07/18 20:50:13  gopal
 * Added f0,deltaf and BW selection criteria for Data filtering; minor changes in data filter GUI; list file selection now has a file filter
 *
 * Revision 1.14  2002/07/18 15:03:18  gopal
 * Compiled with -Wall to catch a lot of problems, fixed undefined prototypes, removed unused variable declarations, etc; ChebychevSet in baseline.c fixed to use proper indices
 *
 * Revision 1.13  2002/04/08 14:54:24  gopal
 * New 16 pixels are now made active in Set Horn Mask popup
 *
 * Revision 1.12  2002/03/12 01:02:45  gopal
 * Except for statistics structure, there is no more hardcoded upperlimit on the number of files (MAXFILES) in otflist. Should try to get rid of the statistic structure limit in a future version
 *
 * Revision 1.11  2002/02/01 19:05:33  gopal
 * Position plots now available in dataselection tab - use right mouse button
 *
 * Revision 1.10  2002/01/26 07:06:41  gopal
 * New Right click menu option in Data Assesment tab to plot the limits and xy positions of the data
 *
 * Revision 1.9  2002/01/26 04:01:24  gopal
 * More versatile context sensitive right button menu in Data assesment clist
 *
 * Revision 1.8  2002/01/24 05:32:37  gopal
 * New Abort Regrid button pops up to abort regridding process in otfmap tab
 *
 * Revision 1.7  2002/01/17 14:28:03  gopal
 * Horn mask is now obeyed in OTF edit mode; only unmasked horns will be displayed; Also only one set horn mask window is now allowed to popup
 *
 * Revision 1.6  2002/01/16 02:06:10  gopal
 * Fixed crashes in Baseline Selected - progressbar[1] was being called - hacked a workaround
 *
 * Revision 1.5  2002/01/15 20:43:20  gopal
 * Fixed bug that crashed program on Set Mask Window operations
 *
 * Revision 1.4  2002/01/15 17:43:09  gopal
 * Fixed crash when entering otflist file name directly in the regrid tab
 *
 * Revision 1.3  2002/01/07 19:53:06  gopal
 * Brought back the rmsthreshold option into the otftool gui; also pruned some of the text output during regridding
 *
 * Revision 1.2  2002/01/07 19:03:43  gopal
 * Fixed bugs that caused crashes when switching tabs occasionally; in otfmap, first bring up basewin and noisewin and then fire up everything else
 *
 * Revision 1.1.1.1  2001/12/20 20:28:21  gopal
 * Initial import of otf2.0
 *
 * Revision 1.16  2001/12/13 20:12:09  gopal
 * Added configuration file parsing, saves current conf values upon exit; looks for otftool.sav on startup; some bug fixes in otftool_setup_body
 *
 * Revision 1.15  2001/12/07 17:03:00  gopal
 * Included include_margin (ramp or no ramp) option to otftool regridding function
 *
 * Revision 1.14  2001/12/05 01:51:36  gopal
 * Added global variable rawdir that allows separation of otf raw filenames from its directory path; corresponding changes to otffp file open calls
 *
 * Revision 1.13  2001/12/03 17:46:58  gopal
 * Fixed r+ to r in fopen - avoids permission problems
 *
 * Revision 1.12  2001/11/28 17:43:14  gopal
 * List file selection browse window improved; fixed bug in baseline_all if no files present in list
 *
 * Revision 1.11  2001/11/25 06:02:47  gopal
 * Add more space for some char strings. Added major new notebook page for data selection
 *
 * Revision 1.10  2001/11/14 14:55:13  gopal
 * Print Statistics is now part of the otftool GUI, also will plot statistics
 *
 * Revision 1.9  2001/11/12 21:53:26  gopal
 * New subroutine for bringing up otfedit like pgplot windows for editing data
 *
 * Revision 1.8  2001/11/07 05:03:57  gopal
 * Lots of changes in GUI to bring up to date with new design discussed with Mark; flatfield, baseline removal, rms weighting, etc
 *
 * Revision 1.7  2001/11/05 17:15:02  heyer
 * ChebychevFit.c
 *
 * Revision 1.6  2001/11/02 19:38:15  gopal
 * Fixed bugs in header structure usage that used old IF plurality
 *
 * Revision 1.5  2001/10/13 04:23:29  gopal
 * Baseline select and all works now with progressbars
 *
 * Revision 1.4  2001/10/12 21:03:05  gopal
 * Updates for baseline window unit selection/ bug fix for MAXFILES in input list
 *
 * Revision 1.3  2001/10/11 16:39:00  gopal
 * otftool now working with progres bars, more buttons and labels and more functionality
 *
 * Revision 1.2  2001/10/05 13:23:59  gopal
 * Updates for logging, statusbar, otfedit
 *
 */

#include "otftool.h"
#include "otftool_setup_body.h"
#include <cpgplot.h>
#include <pthread.h>
#include <fnmatch.h>


void *OTFEdit(void *);
char *bin_prnt_2byte (int);
char *bin_prnt_int (int);
void entry_changed (GtkWidget *, gpointer);
void attach_list(gpointer);
extern int Baseline_AllHorns(int);
extern void error_mes (gchar *, gchar *);
extern GtkWidget* start_progressbar (int);
extern void CompileStats (int, int);
extern void PrintGtkSTATS (int, int);
extern int plotpos (int, int, char **, int);
extern int Baseline_All (void);

void cpgbox(const char *xopt, float xtick, int nxsub, const char *yopt, float ytick, int nysub);

GtkWidget *winentry[5][2];
GtkWidget *winlabel;
pthread_t thread;


char
*bin_prnt_2byte(int x)
{
   int n;
   static char txt[20];

   strcpy(txt,"");
   for(n=0; n<16; n++)
   {
      if((x & 0x8000) !=0)
      {
	strcat(txt,"1");
      }
      else
      {
	strcat(txt,"0");
      }
      /*      if (n==7)
      {
         strcat(txt," "); 
	 } */
      x = x<<1;
   }
   return &txt[0];
}

/* returns binary format integer */
char 
*bin_prnt_int(int x)
{
   int hi, lo;
   static char txt[40];

   hi=(x>>16) & 0xffff;
   lo=x&0xffff;
   strcpy(txt,bin_prnt_2byte(hi));
   strcat(txt," ");
   strcat(txt,bin_prnt_2byte(lo));
   return &txt[0];
}

void 
weightCB (int Clientdata)
{
  setWeightMode((int) Clientdata);
  switch(weight_mode) {
  case 0:
    append_log(0,1,1,otfgui->logtext,"Weight mode changed to ");
    append_log(2,1,1,otfgui->logtext,"Equal\n");    
    strcpy(WTmode, "equal");
    break;
  case 1:
    append_log(0,1,1,otfgui->logtext,"Weight mode changed to ");
    append_log(2,1,1,otfgui->logtext,"Biased\n");
    strcpy(WTmode, "biased");
    break;    
  }
#ifdef DEBUG
  printf("weight = %d\n",weight_mode);
#endif  
}

void
setWeightMode (int type)
{
  weight_mode = type;
}

int getWeightMode ()
{
  return weight_mode;
}


void
ifCB (gpointer Clientdata)
{
  setIFnum((int) Clientdata);
  append_log(0,1,1,otfgui->logtext,"Will regrid IF Number ");
  sprintf(logtext_string, "%d\n", getIFnum());
  append_log(2,1,1,otfgui->logtext,logtext_string);
#ifdef DEBUG
  printf("IF = %d\n",IF);
#endif  
}

void 
setIFnum(int num)
{
  IF = num-1;
}

int getIFnum() 
{
  return IF+1;
}

void
noiseweightCB (gpointer Clientdata)
{
  switch ((int) Clientdata) {
  case 0:
    /*rmsweighting*/
    rmsweight = 1;
    tsysweight = 0;
    if (otfgui->basewin != NULL) 
      gtk_widget_set_sensitive (otfgui->basewin, TRUE);
    append_log(0,1,1,otfgui->logtext,"Noise weight by RMS\n");
    break;
  case 1:
    /*tsysweighting*/
    tsysweight = 1;
    rmsweight = 0;
    if (!base && (otfgui->basewin != NULL)) gtk_widget_set_sensitive (otfgui->basewin, FALSE);
    append_log(0,1,1,otfgui->logtext,"Noise weight by Tsys\n");
    break;
  }
}

void
weight_combo_change (GtkWidget *widget, gpointer data)
{
  char txt[10];

  strcpy(txt,gtk_entry_get_text(GTK_ENTRY( widget)));
  if (!strcmp(txt,"Equal")) {
    weightCB(0);
  }
  if (!strcmp(txt,"Biased")) {
    weightCB(1);
  }
}


char
*get_weight_string ()
{
  static char txt[10];
  switch (weight_mode) {
  case 0:
    strcpy(txt,"Equal");
    break;
  case 1:
    strcpy(txt,"Biased");
    break;    
  }
  return &txt[0];
}

/*  
void 
weightToggleCB (GtkWidget *widget, gpointer data)
{
  if(GTK_TOGGLE_BUTTON(widget)->active){
    weightCB(data);
  }
}
*/

void 
ifToggleCB (GtkWidget *widget, gpointer data)
{
  if(GTK_TOGGLE_BUTTON(widget)->active){
    ifCB(data);
  }
}


void 
noiseweightToggleCB (GtkWidget *widget, gpointer data)
{
  if(GTK_TOGGLE_BUTTON(widget)->active){
    noiseweightCB(data);
  }
}

void
outtypeCB (gpointer Clientdata)
{
  setouttype((int) Clientdata);
  append_log(0,1,1,otfgui->logtext,"Output format set to ");
  switch (out_type) {
  case 1:
    append_log(2,1,1,otfgui->logtext,"FITS\n");
    break;
  case 2:
    append_log(2,1,1,otfgui->logtext,"CLASS\n");
    break;
  }
#ifdef DEBUG
  printf("outtype = %d\n",out_type);
#endif  

}

void 
setouttype(int num)
{
  out_type = num;
}

int getouttype()
{
  return out_type;
}

void 
outtypeToggleCB (GtkWidget *widget, gpointer data)
{
  if(GTK_TOGGLE_BUTTON(widget)->active){
    outtypeCB(data);
  }
}

/*
void 
koptCB (gpointer Clientdata)
{
  setkopt((int) Clientdata);
  append_log(0,1,1,"Convolution Kernel set to ");
  switch (kopt) {
  case 1:
    append_log(2,1,1,"jinc*gaussian\n");
    break;
  case 2:
    append_log(2,1,1,"sinc*gaussian\n");
    break;
  case 3: 
    append_log(2,1,1,"sinc\n");
    break;
  case 4:
    append_log(2,1,1, "gaussian\n");
    break;
  }
#ifdef DEBUG    
  printf("kopt = %d\n",kopt);
#endif  
}
*/

int 
setkopt(char *txt)
{
  if (!strcmp(txt, "jinc*gaussian")) 
    return 1;
  if (!strcmp(txt,"sinc*gaussian"))
    return 2;
  if (!strcmp(txt, "sinc")) 
    return 3;
  if (!strcmp(txt, "gaussian")) 
    return 4;
  return -1;
}

char
*getkopt (int num)
{
  static char txt[15];
  switch (num) {
  case 1:
    strcpy(txt,"jinc*gaussian");
    break;
  case 2:
    strcpy(txt,"sinc*gaussian");
    break;    
  case 3:
    strcpy(txt,"sinc");
    break;    
  case 4:
    strcpy(txt,"gaussian");
    break;    
  }
  return &txt[0];
}

/*
void 
koptToggleCB (GtkWidget *widget, gpointer data)
{
  if(GTK_TOGGLE_BUTTON(widget)->active){
    koptCB(data);
  }
} 
*/

void
kopt_change (GtkWidget *widget, gpointer data)
{
  append_log(0,1,1,otfgui->logtext,"Convolution Kernel set to ");
  kopt = setkopt(gtk_entry_get_text (GTK_ENTRY (widget)));
  append_log (2,1,1,otfgui->logtext,"%s\n",gtk_entry_get_text (GTK_ENTRY (widget)));
  /*  append_log(0,1,1,"\n");*/

#ifdef DEBUG    
  printf("kopt = %d\n",kopt);
#endif  
}  


void 
file_ok_sel_inp( GtkWidget        *w,
                  GtkFileSelection *fs )
{
  /*  char *cwd,*fullfile;
  char *basepath=NULL;
  char *filename;
  size_t n;
  */

  /*
 if ((basepath = (char *) calloc(1, 100*sizeof(char)))  == NULL){
    perror("basepath text error: calloc");
    exit(3);
 }
 if ((fullfile = (char *) calloc(1, 100*sizeof(char)))  == NULL){
    perror("fullfile text error: calloc");
    exit(3);
 }
 if ((filename = (char *) calloc(1, 100*sizeof(char)))  == NULL){
    perror("filename text error: calloc");
    exit(3);
 }
 if ((cwd = (char *) calloc(1, 100*sizeof(char)))  == NULL){
    perror("cwd text error: calloc");

    exit(3);
 }

  strcpy(fullfile,gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
  strcpy(basepath,"");
  
  if (strrchr(fullfile,'/')){
    filename = strrchr(fullfile,'/')+1;
    n = (size_t) (filename - fullfile -1);
    printf("%d\n",(int) n);
    strncat(basepath,fullfile,n);
  }  else {
    strcpy(filename, fullfile);
  } 
  printf("%s %s %s\n", fullfile, basepath, filename);

  getcwd(cwd,100);
  printf("cwd = %s\n",cwd);
  if (!strcmp(cwd,basepath)) {
    gtk_entry_set_text (GTK_ENTRY (otfgui->entry[2]), filename);
  } else {
    strcpy(filename,fullfile);
    gtk_entry_set_text (GTK_ENTRY (otfgui->entry[2]), filename);
  }

  strcpy(otffile,filename);
  */

  if ((otffile = (char *) realloc (otffile, 
				   ((strlen(gtk_file_selection_get_filename 
					    (GTK_FILE_SELECTION (fs)))+1) *
				    sizeof (char)))) == NULL) {
    append_log (2,1,1, otfgui->logtext, "memory allocation error otffile\n");
    return;
  }
  strcpy(otffile,gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
  append_log(0,1,1,otfgui->logtext,"Input List file set to ");
  sprintf(logtext_string,"%s \n",otffile);
  append_log(2,1,1,otfgui->logtext,otffile);
#ifdef DEBUG  
  printf("%s\n",otffile);
#endif
  gtk_entry_set_text (GTK_ENTRY (otfgui->entry[1]), otffile);
  gtk_signal_emit_by_name (GTK_OBJECT (otfgui->entry[1]), "activate", (gpointer) 1);
  /*  entry_changed (otfgui->entry[1], (gpointer) 1);*/
  /*  free(cwd);
  free(filename);
  free(fullfile);
  free(basepath);*/
  gtk_widget_destroy(GTK_WIDGET (fs));
}

/* this function is called when the file selection entry changes */
static void 
file_selection_update_file_list(GtkWidget * dialog)
{
  GtkFileSelection *fs = NULL;
  gchar *list_name;
  guint i = 0;

  g_return_if_fail(GTK_IS_FILE_SELECTION(dialog));

  fs = GTK_FILE_SELECTION(dialog);
  
  gtk_idle_remove(idle_flag);
  idle_flag = 0;
  
  if (!file_filter_text || !strlen(file_filter_text))
    return;
  
  /* stop updating the list visuals */
  gtk_clist_freeze(GTK_CLIST(fs->file_list));
  
  while (gtk_clist_get_text(GTK_CLIST(fs->file_list), i, 0, &list_name)) {
    if (fnmatch(file_filter_text, list_name, 0)) {
      gtk_clist_remove(GTK_CLIST(fs->file_list), i);
      continue;
    }
    i++;
  }
  /* enable updating list visuals */
  gtk_clist_thaw(GTK_CLIST(fs->file_list));
}



/* if user double-clicks on a directory update the file list 
   according to the current filter */
static void 
file_selection_dir_button(GtkWidget * widget, GtkWidget * dialog)
{
  if (idle_flag) {
    return;
  }
  
  idle_flag = gtk_idle_add((GtkFunction) file_selection_update_file_list, dialog);
}

/* when filter entry is changed update the file list according to the current filter */
static gint 
file_selection_filter_key_press(GtkWidget * widget, 
GdkEventKey * event, GtkWidget * dialog)
{
  GtkFileSelection *fs = NULL;
  
  if (!GTK_IS_FILE_SELECTION(dialog))
    return FALSE;

  fs = GTK_FILE_SELECTION(dialog);
  if (event->keyval == GDK_KP_Enter) {
    /* convert keypad enter key to return key (?) */
    event->keyval = GDK_Return;
  }
  if (event->keyval == GDK_Tab || event->keyval == GDK_Return) {
    file_filter_text = gtk_entry_get_text(GTK_ENTRY(widget));
    /* this will trigger the file list updating */
    gtk_file_selection_set_filename(fs, "./");
  }
  return FALSE;
}


void 
filesel_inp (void)
{
  static GtkWidget *filew = NULL;
  GtkWidget *label = NULL;

  if (!filew) {
    /* Create a new file selection widget */
    filew = gtk_file_selection_new ("Input List File selection");
    
    gtk_widget_set_name (GTK_WIDGET (filew), "aux window");
    gtk_file_selection_show_fileop_buttons (GTK_FILE_SELECTION (filew));
    gtk_signal_connect (GTK_OBJECT (filew), "destroy",
			GTK_SIGNAL_FUNC (gtk_widget_destroyed), &filew);

    label = gtk_label_new("File Filter:");
    gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION (filew)->action_area), 
		       label, FALSE, FALSE, 0);
    gtk_widget_show(label);

    file_filter_entry = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION (filew)->action_area), 
		       file_filter_entry, FALSE, FALSE, 0);
    gtk_entry_set_text(GTK_ENTRY(file_filter_entry), file_filter_text);
    gtk_widget_show(file_filter_entry);

    /* when the selection entry changes update the file list */
    /* this can be triggered also by double clicking on a directory */
    gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION(filew)->selection_entry), 
		       "changed", (GtkSignalFunc) file_selection_dir_button, 
		       (gpointer) filew);
    
    gtk_signal_connect(GTK_OBJECT(file_filter_entry), "key_press_event", 
		       (GtkSignalFunc) file_selection_filter_key_press, filew);
    
    /* set current filename (if any); this will also trigger file list 
       update, according to filter_text */
    /*gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), path);*/

    /* select the current selection entry */
    gtk_editable_select_region(GTK_EDITABLE
			       (GTK_FILE_SELECTION(filew)->selection_entry), 0, -1);

    /*    goto_flag = gtk_idle_add((GtkFunction) file_selection_goto, 
	  (gpointer) filew); */
  

  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
                      "clicked", (GtkSignalFunc) file_ok_sel_inp, filew );

  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
                                         (filew)->cancel_button),
                             "clicked", (GtkSignalFunc) gtk_widget_destroy,
                             GTK_OBJECT (filew));

  gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew),
                                   "otf.lis");
  gtk_file_selection_complete (GTK_FILE_SELECTION (filew),"*.lis");
  }

  if (!GTK_WIDGET_VISIBLE (filew))
    gtk_widget_show(filew);
  else 
    gtk_widget_destroy(filew);
}

void 
file_ok_sel_out( GtkWidget        *w,
                  GtkFileSelection *fs )
{
  if ((outfile = realloc(outfile,
			 ((strlen(gtk_file_selection_get_filename 
				  (GTK_FILE_SELECTION (fs)))+1) * 
			  sizeof(char)))) == NULL) {
    append_log (2,1,1,otfgui->logtext, "Realloc error in outfile\n");
    return;
  }
  strcpy(outfile,gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
  append_log(0,1,1,otfgui->logtext,"Output file set to ");
  sprintf(logtext_string,"%s \n",outfile);
  append_log(2,1,1,otfgui->logtext,outfile);
#ifdef DEBUG    
  printf ("%s\n", outfile);
#endif
  gtk_entry_set_text (GTK_ENTRY (otfgui->entry[2]), outfile);
  gtk_signal_emit_by_name (GTK_OBJECT (otfgui->entry[2]), "activate", (gpointer) 2);
  gtk_widget_destroy(GTK_WIDGET (fs));
}

void 
filesel_out (void)
{
  static GtkWidget *filew = NULL;
  
  if (!filew) {
    /* Create a new file selection widget */
    filew = gtk_file_selection_new ("Output File selection");
    
    /*    gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (filew));*/
    gtk_signal_connect (GTK_OBJECT (filew), "destroy",
			GTK_SIGNAL_FUNC (gtk_widget_destroyed), &filew);

  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
                      "clicked", (GtkSignalFunc) file_ok_sel_out, filew );

  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
                                         (filew)->cancel_button),
                             "clicked", (GtkSignalFunc) gtk_widget_destroy,
                             GTK_OBJECT (filew));

  }

  if (!GTK_WIDGET_VISIBLE (filew))
    gtk_widget_show(filew);
  else 
    gtk_widget_destroy(filew);
}

void
maskbtnToggleCB(GtkWidget *widget, gpointer data)
{
  int HornMASK;

  if (GTK_TOGGLE_BUTTON (widget)->active) 
    {
      /* If control reaches here, the toggle button is down */
      HornMASK = 1<<(int)data;
      CmdLineMASK = CmdLineMASK|HornMASK;
      horn_is_active[(int) data] = 1;
      append_log(0,1,1,otfgui->logtext,"Horn ");
      sprintf(logtext_string,"%2d",(int) data+1);
      append_log(2,1,1,otfgui->logtext,logtext_string);
      append_log(0,1,1,otfgui->logtext," turned ON\n");
    } else {
      HornMASK = ~(1<<(int)data);
      CmdLineMASK = CmdLineMASK&HornMASK;
      horn_is_active[(int) data] = 0;
      append_log(0,1,1,otfgui->logtext,"Horn ");
      sprintf(logtext_string,"%2d",(int) data+1);
      append_log(2,1,1,otfgui->logtext,logtext_string);
      append_log(0,1,1,otfgui->logtext," turned OFF\n");
      /* If control reaches here, the toggle button is up */
    }
  /*  for (i = 0; i<2; i++){*/
    /*    if ((GTK_IS_WIDGET (otfgui->hornmasklabel[i])) && (otfgui->hornmasklabel[i] !=NULL))*/
  if (otfgui->hornmasklabel[0] !=NULL)
    gtk_label_set_text (GTK_LABEL (otfgui->hornmasklabel[0]),bin_prnt_int(CmdLineMASK));
  
  if (log_mode == LOG_VERBOSE) {
    append_log(0,1,1,otfgui->logtext,"Horn Mask being set to ");
    append_log(2,1,1,otfgui->logtext,"%s\n",bin_prnt_int(CmdLineMASK));
    /*    append_log(0,1,1,"\n");*/
  }
#ifdef DEBUG
  printf("CmdLineMASK = %x\n",CmdLineMASK);
#endif 
}

void 
entry_modified (GtkWidget *widget, gpointer data)
{
  struct stat statbuf;
  char *errmsg;
  char tmpfile[100];

  if ((errmsg = (char *) malloc(50*sizeof(char))) == NULL) {
    perror("errmsg text error: malloc");
    exit(3);
  }

  switch ((int) data){
  case 0:
    if (sscanf (gtk_entry_get_text(GTK_ENTRY (widget)), 
		"%g", &THETA_N) != 1) {
      error_mes("Error!","Invalid Entry for Cell Size");
	return;
    } else {
      if (THETA_N <= 0) {
	error_mes("Error!","Cell Size should be greater than 0");
	return;
      }
    }
    if (log_mode == LOG_VERBOSE)
      append_log(0,1,1,otfgui->logtext,"Output Cell Size set to ");
    break;
  case 1:
    if (sscanf (gtk_entry_get_text(GTK_ENTRY (widget)), 
		"%s", &tmpfile[0]) != 1) {
      error_mes("Error!","Invalid Entry for Input List file");
      return;
    } else {
      if ((otffile = realloc (otffile, ((strlen(tmpfile)+1)*sizeof(char)))) == NULL) {
	append_log(2,1,1,otfgui->logtext, "Reallocation memory error for otffile\n");
	return;
      }
      strcpy (otffile, tmpfile);
      if ((stat(otffile,&statbuf)) == -1) {
	strcpy(errmsg,"Input List file:");
	strcat(errmsg,strerror(errno));
	error_mes("Error!", errmsg);
	return;
      } else {
	if (notebookmode == DATAEDIT) {
	  if ((GTK_IS_WIDGET (otfgui->clist)) && (otfgui->clist !=NULL))
	    attach_list(otfgui->clist);
	}
      }
    }
    if (log_mode == LOG_VERBOSE)
      append_log(0,1,1,otfgui->logtext,"Input List File set to ");
    break;
  case 2:
    if (sscanf (gtk_entry_get_text(GTK_ENTRY (widget)), 
		"%s", &tmpfile[0]) != 1) {
      error_mes("Error!","Invalid Entry for Output file");
      return;
    } else {
      if ((outfile = realloc (outfile, ((strlen(tmpfile)+1)*sizeof(char)))) == NULL) {
	append_log(2,1,1,otfgui->logtext, "Reallocation memory error for outfile\n");
	return;
      }
      strcpy (outfile, tmpfile);
    }
    if (log_mode == LOG_VERBOSE)
      append_log(0,1,1,otfgui->logtext,"Output file set to ");
    break;
  case 3:
    if (sscanf (gtk_entry_get_text(GTK_ENTRY (widget)), 
		"%s", &linename[0]) == 1) {
      if (strlen(linename) > 12) {
	error_mes("Error!","Linename can only be 12 characters long");
	return;
      }
    }
    if (log_mode == LOG_VERBOSE)    
      append_log(0,1,1,otfgui->logtext,"Linename set to ");
    break;
  case 4:
    if (sscanf (gtk_entry_get_text(GTK_ENTRY (widget)), 
		"%g", &rmsthreshold) != 1) {
      error_mes("Error!","Invalid Entry for RMS Threshold");
      return;
    } else {
      if (rmsthreshold <= 0) {
	error_mes("Error!","RMS Threshold should be greater than 0");
	return;
      }
    }
    if (log_mode == LOG_VERBOSE)
      append_log(0,1,1,otfgui->logtext,"RMS Threshold Set to ");
    break;      
  }
  if (log_mode == LOG_VERBOSE) { 
    append_log(2,1,1,otfgui->logtext,"%s\n",gtk_entry_get_text(GTK_ENTRY (widget)));
    /*    append_log(0,1,1,"\n");*/
  }
#ifdef DEBUG
  printf("otfg->entry[%1d] = %s\n",(int) data, gtk_entry_get_text(GTK_ENTRY (widget)));
#endif
  free(errmsg);
}

void
entry_changed (GtkWidget *widget, gpointer data)
{
  entry_modified (widget, data);
}

void
entry_left (GtkWidget *widget, GdkEvent *event, gpointer data)
{

  entry_modified (widget, data);
}




void
select_all_horns (GtkWidget *widget, gpointer data)
{
  int i;

  for (i=0; i<MAXHORNS; i++){
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(otfgui->maskbtn[i]), TRUE);
  }
}

void
deselect_all_horns (GtkWidget *widget, gpointer data)
{
  int i;

  for (i=0; i<MAXHORNS; i++){
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(otfgui->maskbtn[i]), FALSE);
  }
}

void
hornmask_win (GtkWidget *widget) {
  
  char *btntext;
  static GtkWidget *dialog_window = NULL;
  GtkWidget *box1;
  GtkWidget *table;
  int i;
  GtkWidget *button;
  GtkWidget *label, *labelbox;
  GtkTooltips *tooltips;

  if (!dialog_window) {
    dialog_window = gtk_dialog_new ();
    gtk_signal_connect (GTK_OBJECT (dialog_window), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			&dialog_window);
    gtk_widget_set_name (GTK_WIDGET (dialog_window), "aux window");
    gtk_window_set_title (GTK_WINDOW (dialog_window), "Set Horn Mask");
    gtk_container_set_border_width (GTK_CONTAINER (dialog_window), 0);
    /*  gtk_widget_set_usize (dialog_window, 400, 300);*/
    
    table = gtk_table_new (4, 3, TRUE);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->vbox),
			table, TRUE, TRUE, 0);
    
    if  ((btntext = (char *) malloc(2*sizeof(char)))  == NULL){
      perror("btntext text error: malloc");
      exit(3);
    }
    box1 = gtk_hbox_new (FALSE,5);
    gtk_table_attach_defaults (GTK_TABLE (table), box1, 1, 3, 0, 1); 
    
    tooltips = gtk_tooltips_new ();
    /* 1st row */
    for (i = 0; i<8; i++) {
      sprintf(btntext,"%2d",i+1);
      otfgui->maskbtn[i] = gtk_check_button_new_with_label (btntext);
      gtk_box_pack_start (GTK_BOX(box1), otfgui->maskbtn[i], TRUE, TRUE, 0);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(otfgui->maskbtn[i]), horn_is_active[i]);
      gtk_signal_connect(GTK_OBJECT(otfgui->maskbtn[i]), "toggled",
			 GTK_SIGNAL_FUNC(maskbtnToggleCB), (gpointer)i);
      gtk_widget_show (otfgui->maskbtn[i]);
    }
    gtk_widget_show(box1);
    
    
    /*2nd row */
    
    labelbox = gtk_event_box_new ();
    label = gtk_label_new ("Mask horns");
    gtk_widget_set_usize(label,15,-1);
    gtk_container_add (GTK_CONTAINER (labelbox), label);
    gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 1, 3);
    gtk_tooltips_set_tip (tooltips, labelbox, 
			"Horns to be used for regridding", NULL);
    gtk_widget_show (label);
    gtk_widget_show (labelbox);
    
    box1 = gtk_hbox_new (FALSE,5);
    gtk_table_attach_defaults (GTK_TABLE (table), box1, 1, 3, 1, 2); 
    for (i = 8; i<16; i++) {
      sprintf(btntext,"%2d",i+1);
      otfgui->maskbtn[i] = gtk_check_button_new_with_label (btntext);
      gtk_box_pack_start (GTK_BOX(box1), otfgui->maskbtn[i], TRUE, TRUE, 0);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(otfgui->maskbtn[i]), horn_is_active[i]);
      gtk_signal_connect(GTK_OBJECT(otfgui->maskbtn[i]), "toggled",
			 GTK_SIGNAL_FUNC(maskbtnToggleCB), (gpointer)i);
      gtk_widget_show (otfgui->maskbtn[i]);
    }
    gtk_widget_show(box1);
    
    /*3rd row*/
    
    box1 = gtk_hbox_new (FALSE,5);
    gtk_table_attach_defaults (GTK_TABLE (table), box1, 1, 3, 2, 3); 
    for (i = 16; i<24; i++) {
      sprintf(btntext,"%2d",i+1);
      otfgui->maskbtn[i] = gtk_check_button_new_with_label (btntext);
      gtk_box_pack_start (GTK_BOX(box1), otfgui->maskbtn[i], TRUE, TRUE, 0);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(otfgui->maskbtn[i]), horn_is_active[i]);
      gtk_signal_connect(GTK_OBJECT(otfgui->maskbtn[i]), "toggled",
			 GTK_SIGNAL_FUNC(maskbtnToggleCB), (gpointer)i);
      gtk_widget_show (otfgui->maskbtn[i]);
      /*      gtk_widget_set_sensitive (GTK_WIDGET (otfgui->maskbtn[i]), FALSE);*/
    }
    gtk_widget_show(box1);
    
    /* 4th row */
    
    box1 = gtk_hbox_new (FALSE,5);
    gtk_table_attach_defaults (GTK_TABLE (table), box1, 1, 3, 3, 4); 
    for (i = 24; i<32; i++) {
      sprintf(btntext,"%2d",i+1);
      otfgui->maskbtn[i] = gtk_check_button_new_with_label (btntext);
      gtk_box_pack_start (GTK_BOX(box1), otfgui->maskbtn[i], TRUE, TRUE, 0);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(otfgui->maskbtn[i]), horn_is_active[i]);
      gtk_signal_connect(GTK_OBJECT(otfgui->maskbtn[i]), "toggled",
			 GTK_SIGNAL_FUNC(maskbtnToggleCB), (gpointer)i);
      gtk_widget_show (otfgui->maskbtn[i]);
      /*      gtk_widget_set_sensitive (GTK_WIDGET (otfgui->maskbtn[i]), FALSE);*/
    }
    gtk_widget_show(box1);
    
    button = gtk_button_new_with_label ("Dismiss");
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area), 
			button, TRUE, TRUE, 0);
    gtk_widget_show (button);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       GTK_OBJECT (dialog_window));
    
    gtk_widget_show (table);
    
    box1 = gtk_hbox_new (FALSE,5);
    
    button = gtk_button_new_with_label("Select All Horns");
    gtk_box_pack_start (GTK_BOX (box1), button, TRUE, TRUE, 0);  
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (select_all_horns), NULL);
    gtk_widget_show(button);
    
    button = gtk_button_new_with_label("Deselect All Horns");
    gtk_box_pack_start (GTK_BOX (box1), button, TRUE, TRUE, 0);  
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (deselect_all_horns), NULL);
    gtk_widget_show(button);
    
    gtk_widget_show(box1);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area), 
			box1, TRUE, TRUE, 0);
  }
  
  if (!GTK_WIDGET_VISIBLE (dialog_window))
    gtk_widget_show(dialog_window);
  else 
    gtk_widget_destroy(dialog_window);
}


void
header_activate (gint row)
{
  char errmsg[300];
  char temp[MAXPATHLEN];
  gchar *text;
  gchar *clistdata;

  gtk_clist_get_text(GTK_CLIST(otfgui->clist), row, 0, &text);
  clistdata = gtk_clist_get_row_data (GTK_CLIST(otfgui->clist), 
                                      row);
  strcpy(temp,clistdata);
  strcat (temp, "/");
  strcat(temp,text);

  /*  strcpy(temp,rawdir);
      strcat(temp,text);*/
  if ((otffp=fopen(temp,"r"))==NULL) {
    sprintf(errmsg,"Input OTF file: %s",temp);
    strcat(errmsg,strerror(errno));
    error_mes("Error!", errmsg);
    return;
  }
  
  append_log(0,1,1,otfgui->logtext,"Reading file for header listing: %s\n", text);
  ReadHEADER();
  GtkPrintHEADER(text);
  /*  PrintHEADER();*/
}

void
examine_activate (gint row)
{
  char errmsg[300];
  char temp[MAXPATHLEN];
  short checkmask,n;
  gchar *text;
  gchar *clistdata;

  gtk_clist_get_text(GTK_CLIST(otfgui->clist), row, 0, &text);
  clistdata = gtk_clist_get_row_data (GTK_CLIST(otfgui->clist), 
                                      row);

  strcpy(temp,clistdata);
  strcat (temp, "/");
  strcat(temp,text);
  

  /*
  strcpy(temp,rawdir);
  strcat(temp,text);
  */
  if (otfedit_exit == 1) {
    if ((otffp=fopen(temp,"r"))==NULL) {
      sprintf(errmsg,"Input OTF file: %s",temp);
      strcat(errmsg,strerror(errno));
      error_mes("Error!", errmsg);
      return;
    }

    append_log(0,1,1,otfgui->logtext,"Examine activate: Reading file %s\n", text);
    ReadHEADER();

    checkmask = 0;
    last_active_horn = -1;
    for (n=0; n<H.nhorns; n++) {
      if (horn_is_active[n]) last_active_horn = n;
      checkmask += horn_is_active[n];
    }
    if (checkmask == 0) {
      append_log(2,1,1,otfgui->logtext,"All Horns masked off; Nothing to edit\n ");
      return;
    }
    otfedit_exit = 0;  /* flag to exit otfediting file */
    pthread_create(&thread, NULL, OTFEdit, (void *) temp); 
    /*  GtkPrintHEADER(text);*/
    /*  PrintHEADER();*/
  } else {
    append_log(0,1,1,otfgui->logtext,"Another OtfEdit is already active. Please exit that first\n");
  }
}

void
baseline_activate (gint row)
{

  GtkWidget *statuscontainer;
  char errmsg[50];
  char temp[MAXPATHLEN];
  gchar *text;
  gchar *clistdata;


  statuscontainer = GTK_WIDGET (start_progressbar(1));

  gtk_clist_get_text(GTK_CLIST(otfgui->clist), row, 0, &text);
  clistdata = gtk_clist_get_row_data (GTK_CLIST(otfgui->clist), 
                                      row);
  strcpy(temp,clistdata);
  strcat (temp, "/");
  strcat(temp,text);


  /*  strcpy(temp,rawdir);
      strcat(temp,infile[row]); */
  if ((otffp=fopen(temp,"r"))==NULL) {
    sprintf(errmsg,"Input OTF file: %s",temp);
    strcat(errmsg,strerror(errno));
    error_mes("Error!", errmsg);
    return;
  }

  baselinemode = BASESELECT;
  /*  append_log(0,1,1,otfgui->logtext,"Start baselining (1) all horns for file %s\n", infile[row]);  */
  if (Baseline_AllHorns(row) != 0) {
    gtk_widget_destroy(statuscontainer);
    return;
  } else {
    /*      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (baselinewgt->checkbtn), TRUE);*/
    /*  append_log(0,1,1,otfgui->logtext,"Start baselining (2) all horns for file %s\n", infile[row]);  */
    CompileStats(row,1);
    /*  append_log(0,1,1,otfgui->logtext,"Start baselining (3) all horns for file %s\n", infile[row]);  */
    PrintGtkSTATS(row,1);
    /*    printf ("%s\n",temp);*/
    /*    sprintf(logtext_string,"Finished baselining all horns for file %s\n",temp);*/
    append_log(0,1,1,otfgui->logtext,"Finished baselining all horns for file %s\n", infile[row]);
    gtk_widget_destroy (statuscontainer);
    fclose(otffp);
  }
}


void
plotpos_x11_activate (gint row)
{
  append_log(0,1,1,otfgui->logtext,"Plotting positions for %s\n",infile[row]);
  plotpos (row, 1, infile, 0);
}

void
plotpos_ps_activate (gint row)
{
  append_log(0,1,1,otfgui->logtext,"Plotting positions for %s to postscript file otfplotpos.ps\n",
	     infile[row]);
  plotpos (row, 1, infile, 1);
}

void
plotpos_all_x11_activate (gint row)
{
  append_log(0,1,1,otfgui->logtext,"Plotting positions for all files\n");
  plotpos (0, nfiles, infile, 0);
}

void
plotpos_all_ps_activate (gint row)
{
  append_log(0,1,1,otfgui->logtext,"Plotting positions for all files to postscript file otfplotpos.ps\n");
  plotpos (0, nfiles, infile, 1);
}

  
  
GtkWidget 
*make_menu_item (GtkWidget *menu, gchar *text)
{
  GtkWidget *item;


  if (!strcmp(text,"SEPARATOR")) {
    item = gtk_menu_item_new ();
    gtk_container_add ( GTK_CONTAINER (item), gtk_hseparator_new ());
  } else {
    item = gtk_menu_item_new_with_label (text);
  }
  gtk_menu_append (GTK_MENU (menu), item);
  gtk_widget_show_all (item);
  return item;
}

gint
clist_button_pressed (GtkWidget *widget, GdkEventButton * event)
{
  GtkWidget *menu, *item1, *item2;
  GtkWidget *item3, *item4, *item5, *item6;
  GtkWidget *item7, *item8;
  GtkWidget *sep;
  gint row, column;
  gint ret;
  gchar *text;

  menu = gtk_menu_new ();

  item1 = make_menu_item (menu, "File Header");
  item2 = make_menu_item (menu, "File Examine");
  item3 = make_menu_item (menu, "File Baseline");
  sep = make_menu_item (menu, "SEPARATOR");
  item4 = make_menu_item (menu, "Baseline All");
  sep = make_menu_item (menu, "SEPARATOR");
  item5 = make_menu_item (menu, "File Position Plot");
  item6 = make_menu_item (menu, "File Position Plot postscript");
  sep = make_menu_item (menu, "SEPARATOR");
  item7 = make_menu_item (menu, "Position Plot All");
  item8 = make_menu_item (menu, "Position Plot All postscript");

  if (event->type == GDK_BUTTON_PRESS) {
    /* convert GdkEvent to GdkEventButton */
    GdkEventButton *bevent = (GdkEventButton *) event;
    if (!(ret = gtk_clist_get_selection_info(GTK_CLIST (widget),
					     (gint) event->x, (gint) event->y, &row, &column)))
      {
#ifdef DEBUG	
	printf("No Selection made \n");
#endif	
	return FALSE;
      } else {
	gtk_clist_get_text(GTK_CLIST(widget), row, column, &text);
	gtk_signal_connect_object (GTK_OBJECT (item1), "activate",
                                   GTK_SIGNAL_FUNC (header_activate),
                                   (gpointer) row);
	gtk_signal_connect_object (GTK_OBJECT (item2), "activate",
                                   GTK_SIGNAL_FUNC (examine_activate),
                                   (gpointer) row);
	gtk_signal_connect_object (GTK_OBJECT (item3), "activate",
                                   GTK_SIGNAL_FUNC (baseline_activate),
                                   (gpointer) row);
	gtk_signal_connect_object (GTK_OBJECT (item4), "activate",
                                   GTK_SIGNAL_FUNC (baseline_all),
                                   NULL);
	gtk_signal_connect_object (GTK_OBJECT (item5), "activate",
                                   GTK_SIGNAL_FUNC (plotpos_x11_activate),
                                   (gpointer) row);	
	gtk_signal_connect_object (GTK_OBJECT (item6), "activate",
                                   GTK_SIGNAL_FUNC (plotpos_ps_activate),
                                   (gpointer) row);	
	gtk_signal_connect_object (GTK_OBJECT (item7), "activate",
                                   GTK_SIGNAL_FUNC (plotpos_all_x11_activate),
                                   NULL);	
	gtk_signal_connect_object (GTK_OBJECT (item7), "activate",
                                   GTK_SIGNAL_FUNC (plotpos_all_ps_activate),
                                   NULL);	
	
	
	if (event->button == 3 || event->button == 2) {
	  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
			  bevent->button, bevent->time);
	}
	return TRUE;
      }
#ifdef DEBUG
    printf("event-button %d\n",event->button);
#endif
  }
  return FALSE;
}

void
settext_baselines()
{
  static char txt[200];
  static char tmptxt[40];
  int i;

  switch(wind_type) {
  case 1:
    strcpy (txt,"Channels: ");
    break;
  case 2:
    strcpy (txt,"Velocity: ");
    break;
  }
  for (i = 0; i<NWIND; i++){
    switch(wind_type) {
    case 1:
      W1[i] = w1[i]; W2[i] = w2[i];
      break;
    case 2:
      W1[i] = (w1[i]*H.dveldin) + H.xvel1;
      W2[i] = (w2[i]*H.dveldin) + H.xvel1;
      break;
    }
    sprintf(tmptxt, "(%g, %g)", W1[i], W2[i]);
    strcat (txt, tmptxt);
    
    if (i != (NWIND-1)) strcat(txt," ; ");
  }
  gtk_entry_set_text (GTK_ENTRY (baselinewgt->window), txt);
  sprintf(txt,"%1d", get_nwind());
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (baselinewgt->nwindow)->entry), txt);
  sprintf(txt,"%1d", get_order());
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (baselinewgt->order)->entry), txt);
}

/* If we come here, then the user has selected a row in the clist. */
void 
otffile_selection_made( GtkWidget      *clist,
			gint            row,
			gint            column,
			GdkEventButton *event,
			gpointer        data )
{
    gchar *text;
    char errmsg[50];
    char temp[MAXPATHLEN];
    gchar *clistdata;

    
    /* Get the text that is stored in the selected row and column
     * which was clicked in. We will receive it as a pointer in the
     * argument text.
     */
    gtk_clist_get_text(GTK_CLIST(clist), row, column, &text);

    /* Just prints some information about the selected row */

    /*    g_print("You selected row %d. More specifically you clicked in "
            "column %d, and the text in this cell is %s\n\n",
            row, column, text);
	    printf("event-button %d\n",event->button); */

    clistdata = gtk_clist_get_row_data (GTK_CLIST(clist), row);
    
    strcpy(temp,clistdata);
    strcat (temp, "/");
    strcat(temp,text);

    

    if ((otffp=fopen(temp,"r"))==NULL) {
      sprintf(errmsg,"Input OTF file: %s",temp);
      strcat(errmsg,strerror(errno));
      error_mes("Error!", errmsg);
      return;
    }
    ReadHEADER();
    append_log(0,1,1,otfgui->logtext,"Selection Made: Reading file %s for source = %s\n", text, H.name);


    /*
    REDUCED=0;
    if (REDUCED) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (baselinewgt->checkbtn), TRUE);
      sprintf(logtext_string,"%d", order);
      ORDER = order;
      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (baselinewgt->order)->entry), 
			  logtext_string);
      sprintf(logtext_string,"%d", nwind);
      NWIND = nwind;
      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (baselinewgt->nwindow)->entry), 
			  logtext_string);
      gtk_entry_set_text (GTK_ENTRY (baselinewgt->window), settext_main_window_entry());
    } else {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (baselinewgt->checkbtn), FALSE);
    }
    */

    fclose(otffp);
    return;
}

void
attach_list(gpointer data)
{

  int i;
  struct stat statbuf;
  /*  char *errmsg;*/
  char *rowentry[1];
  char tmpfile[100];
  char *basec, *dirc, *tempfile, *tempfile2;
  /*
  if ((errmsg = (char *) malloc(50*sizeof(char))) == NULL) {
    perror("errmsg text error: malloc");
    exit(3);
  }
  */

  if ((rowentry[0] = (char *) malloc(50*sizeof(char))) == NULL) {
    perror("row entry text error: malloc");
    exit(3);
  }


  if (sscanf (gtk_entry_get_text(GTK_ENTRY (otfgui->entry[1])), 
              "%s", &tmpfile[0]) != 1) {
    error_mes("Error!","Invalid Entry for Input List file");
    return;
  } else {
    if ((otffile = realloc (otffile, ((strlen(tmpfile)+1)*sizeof(char)))) == NULL) {
      append_log(2,1,1,otfgui->logtext, "Reallocation memory error for otffile\n");
      return;
    }
    strcpy (otffile, tmpfile);
    if ((stat(otffile,&statbuf)) == -1) {
      strcpy(logtext_string,"Input List file:");
      strcat(logtext_string,strerror(errno));
      error_mes("Error!", logtext_string);
      return;
    }
  }
  /*  printf("%s\n", otffile);*/
  if (strcmp(otffile, "")) {
    if (ReadList(otffile) == -1) {
      error_mes ("Error!", "Cannot Read Input List File");
      append_log(2,1,1,otfgui->logtext,"Cannot Read Input List File\n");
    } else {
      gtk_clist_clear( (GtkCList *) data );
      for (i = 0; i<nfiles; i++) {
	tempfile = strdup (infile[i]);
	tempfile2 = strdup (infile[i]);
	dirc = dirname(tempfile);
	basec = basename (tempfile2);
	strcpy(rowentry[0],basec);
	/*	printf("%s\n", rowentry[i][0]);*/
	gtk_clist_append( (GtkCList *) data, (char **) rowentry);
	gtk_clist_set_row_data (GTK_CLIST (data), i, (gpointer) dirc); 

	/*	gtk_clist_set_text ((GtkCList *) data, i, 0, infile[i]);*/
      }
    }
  }
  /*  free(errmsg);*/
}

void 
set_order (short num)
{
  ORDER = num;
}

int
get_order ()
{
  return ORDER;
}

void
order_change (GtkWidget *widget, gpointer data)
{
  /*  set_order((short) atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(baselinewgt->order)->entry))));*/
  set_order((short) atoi(gtk_entry_get_text(GTK_ENTRY(widget))));
  if (log_mode == LOG_VERBOSE) {
    sprintf(logtext_string,"Baseline order changed to %d\n",ORDER);
    append_log(0,1,1,otfgui->logtext,logtext_string);
  }
}

void 
set_nwind (short num)
{
  NWIND = num;
}

int
get_nwind ()
{
  return NWIND;
}

void
nwindow_change (GtkWidget *widget, gpointer data)
{
  /*  set_nwind((short) atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(baselinewgt->nwindow)->entry)))); */
  set_nwind((short) atoi(gtk_entry_get_text(GTK_ENTRY(widget))));
  if (log_mode == LOG_VERBOSE) {
    sprintf(logtext_string,"Baseline number of windows changed to %d\n",NWIND);
    append_log(0,1,1,otfgui->logtext,logtext_string);
  }
}

void 
*pgwindow (void *args)
{
  int id1;
  float x[] = {0.1, 0.2, 0.3, 0.4, 0.5};
  float y[] = {1, 2, 3, 4, 5};

  id1=cpgopen("/XSERVE");
  if (id1 < 1) exit(1); 
  /*  cpgbeg(0,"/xserve", 1, 1);*/
  /*  cpgenv(0,1,0,5,0,1);*/
  cpgscf(2); cpgslw(1); cpgsch(0.9);
  cpgsvp(0.15, 0.85, 0.15, 0.85);
  cpgwnad(0.0, 1.0, 0.0, 5.0);
  /*  cpgbbuf();
  cpgpage();
  cpgvstd();
  cpgswin(0.0, 1.0, 0.0, 5.0);*/

  /*  cpgbox  ("ABCN", 0.0, 0, "ABCN", 0.0, 0);
      cpglab("x","y", "title");*/
  cpgbox("BCNST", 0,0, "BCNST", 0,0);
  cpgpt(5,x,y,9);

  /*  cpgebuf();*/

  /*   cpgslct(id1);
       cpgclos();*/
  cpgend();
  pthread_exit(0);
}

/*void 
pgwindow_wrap (GtkWidget *widget, gpointer data) 
{
  pgwindow();
}
*/

void 
edit_selected (GtkWidget *widget, GtkWidget *event, gpointer data)
{
  GList *l;
  GtkCList *clist;
  char *filename;

  if (otfedit_exit == 1) {
    clist = GTK_CLIST (otfgui->clist);
    l = clist->selection;
    if (l != NULL) {
      gtk_clist_get_text (GTK_CLIST (clist), GPOINTER_TO_INT (l->data), 0, &filename);
      append_log(0,1,1,otfgui->logtext,"Editing file %s \n", filename);
#ifdef DEBUG   
      printf("%s\n", filename);
#endif
      examine_activate (GPOINTER_TO_INT (l->data));
    }
  } else {
    append_log(0,1,1,otfgui->logtext,"Another OtfEdit is already active. Please exit that first\n");
  }
  
}

void
baseline_selected (GtkWidget *widget, gpointer data)
{
  GList *l;
  GtkCList *clist;
  int row;
  /*
  GtkWidget *statuscontainer;
  char errmsg[50];
  char temp[MAXPATHLEN];
  */
  clist = GTK_CLIST (otfgui->clist);
  l = clist->selection;
  if (l != NULL) {
    row = GPOINTER_TO_INT(l->data);
    if ((W1[0] == -1000.0) || (W2[0] == -1000.0)) {
      error_mes("Error!","No baseline windows specified or Bad Baseline Windows");
      return;
    } 
    
    append_log (0,1,1,otfgui->logtext, "Baselining file number %d\n",row);
    baseline_activate (row);

    /*
    statuscontainer = GTK_WIDGET (start_progressbar(1));
    strcpy(temp,rawdir);
    strcat(temp,infile[row]);
    if ((otffp=fopen(temp,"r"))==NULL) {
      sprintf(errmsg,"Input OTF file: %s",temp);
      strcat(errmsg,strerror(errno));
      error_mes("Error!", errmsg);
      return;
    }

    baselinemode = BASESELECT;

    if (Baseline_AllHorns(row) != 0) {
      gtk_widget_destroy(statuscontainer);
      return;
    } else {
      CompileStats(row,1);
      PrintGtkSTATS(row,1);
      sprintf(logtext_string,"Finished baselining all horns for file %s\n",infile[row]);
      append_log(0,1,1,otfgui->logtext,logtext_string);
      gtk_widget_destroy (statuscontainer);
      fclose(otffp);
    }
    */
  } else {
    error_mes("Error!","Select a file to Baseline from list on right");
  }
}
     

void
baseline_all (GtkWidget *widget, gpointer data)
{
  GtkWidget *statuscontainer;
  int gett;
  
  gett = gtk_clist_get_text (GTK_CLIST (otfgui->clist), 0,0,NULL);
  if (gett == 0) {
    error_mes ("Error!", "No files to process");
    return;
  }

  if ((W1[0] == -1000.0) || (W2[0] == -1000.0)) {
    error_mes("Error!","No baseline windows specified");
    return;
  } 
  statuscontainer = GTK_WIDGET (start_progressbar(2));
  baselinemode = BASEALL;
  append_log (0,1,1,otfgui->logtext, "Baselining all Horns for all Files\n");
  if (Baseline_All() != 0) {
    gtk_widget_destroy(statuscontainer);
    return;
  } else {
    CompileStats(0,nfiles);
    PrintGtkSTATS(0,nfiles);
    sprintf(logtext_string,"Finished baselining all horns for all files \n");
    append_log(0,1,1,otfgui->logtext,logtext_string);
    gtk_widget_destroy (statuscontainer);
  }
}

void
settext_winentry (void)
{
  short i, j, nwin;
  char txt[30];

  /*  sscanf(gtk_entry_get_text (GTK_ENTRY ( GTK_COMBO (baselinewgt->nwindow)->entry)),
      "%hd",&nwin);  */
  nwin = get_nwind();
  for (i = 0; i<nwin; i++){
    for (j = 0; j < 2; j++) {
      switch (j) {
      case 0:
	sprintf(txt,"%g",W1[i]);
	gtk_entry_set_text (GTK_ENTRY (winentry[i][j]), txt);
	break;
      case 1:
	sprintf(txt,"%g",W2[i]);
	gtk_entry_set_text (GTK_ENTRY (winentry[i][j]), txt);
	break;
      }
    }
  }
}


void
window_unit_change (GtkWidget *widget, gpointer data)
{
  char txt[10];
  short i;

  strcpy(txt,gtk_entry_get_text(GTK_ENTRY( widget)));
  if (!strcmp(txt,"Channels")) {
    wind_type = 1;
    for (i = 0; i<NWIND; i++){
      W1[i] = w1[i]; W2[i] = w2[i];
    }
  } else {
    if (H.dveldin != 0.0) {
      wind_type = 2;
      for (i = 0; i<NWIND; i++){
	W1[i] = (w1[i]*H.dveldin) + H.xvel1;
	W2[i] = (w2[i]*H.dveldin) + H.xvel1;
      }
    } else {
      wind_type = 1;
      append_log (2,1,1, otfgui->logtext, 
		  "VLSR or velocity resolution unknown. No files read yet?\n");
      append_log (2,1,1, otfgui->logtext, "Resetting to Channels\n");
      for (i = 0; i<NWIND; i++){
	W1[i] = w1[i]; W2[i] = w2[i];
      }
      gtk_entry_set_text (GTK_ENTRY (widget), "Channels");
      strcpy(txt, "Channels");
    }
  }
  settext_winentry();

  if (log_mode == LOG_VERBOSE) {
    sprintf(logtext_string,"Window Unit changed to %s\n",txt);
    append_log(0,1,1,otfgui->logtext,logtext_string);
  }
}

void
set_windows (GtkWidget *widget, gpointer data)
{
  int nwin, i;
  char txt[200], tmptxt[20];
  float wi1,wi2;
  
  /*  sscanf(gtk_entry_get_text (GTK_ENTRY ( GTK_COMBO (baselinewgt->nwindow)->entry)),
  "%d",&nwin); */
  nwin = get_nwind ();

  switch(wind_type) {
  case 1:
    strcpy (txt,"Channels: ");
    break;
  case 2:
    strcpy (txt,"Velocity: ");
    break;
  }
  for (i=0; i<nwin; i++){
    if ((sscanf(gtk_entry_get_text (GTK_ENTRY (winentry[i][0])), "%g", &wi1) != 1) ||
	(sscanf(gtk_entry_get_text (GTK_ENTRY (winentry[i][1])), "%g", &wi2) != 1)) {
      gtk_label_set_text (GTK_LABEL (winlabel), "Invalid Entries");
      return;
    } else {
      gtk_label_set_text (GTK_LABEL (winlabel), "");
      /*
      if (wind_type == 1) {
	W1[i] = (w1-H.xvel1[IF])/H.dveldin[IF];
	W2[i] = (w2-H.xvel1[IF])/H.dveldin[IF];
      } else {
	W1[i] = w1;
	W2[i] = w2;
	}*/
      W1[i] = wi1;
      W2[i] = wi2;
      sprintf(tmptxt, "(%g, %g)", wi1, wi2);
      strcat (txt, tmptxt);
      if (i != (nwin-1)) strcat(txt," ; ");
      switch (wind_type) {
      case 1:
	w1[i] = wi1;
	w2[i] = wi2;
	break;
      case 2:
	w1[i] = (wi1-H.xvel1)/H.dveldin;
	w2[i] = (wi2-H.xvel1)/H.dveldin;
	break;
      }

    }
  }
  gtk_entry_set_text (GTK_ENTRY (baselinewgt->window), txt);
  append_log (0,1,1, otfgui->logtext, "Setting baseline window to %s\n",txt);
  gtk_widget_destroy (GTK_WIDGET (data));
  
}

void 
edit_windows (GtkWidget *widget, gpointer data)
{

  static GtkWidget *dialog_window = NULL;
  GtkWidget *button, *label;
  GtkWidget *hbox;
  int nwin, i, j;
  char txt[9];
  GList *glist = NULL;
  GtkWidget *combo;

  if (!dialog_window) {
    dialog_window = gtk_dialog_new ();
    gtk_signal_connect (GTK_OBJECT (dialog_window), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			&dialog_window);
    gtk_window_set_title (GTK_WINDOW (dialog_window), "Set Windows");
    gtk_container_set_border_width (GTK_CONTAINER (dialog_window), 0);
    label = gtk_label_new ("Windows");
    
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->vbox),
			label, TRUE, TRUE, 0);
    /*    sscanf(gtk_entry_get_text (GTK_ENTRY ( GTK_COMBO (baselinewgt->nwindow)->entry)),
	  "%d",&nwin);*/
    nwin = get_nwind();
    
#ifdef DEBUG  
    printf("%d\n",nwin);
#endif
    hbox = gtk_hbox_new (FALSE, 5);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->vbox),
			hbox, TRUE, TRUE, 0);
    label = gtk_label_new ("Windows Units");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    combo = gtk_combo_new ();
    glist = g_list_append (glist, "Channels");  glist = g_list_append (glist, "Velocity");
    gtk_combo_set_popdown_strings( GTK_COMBO(combo), glist) ;
    gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (combo)->entry), FALSE);
    if (wind_type == 1) 
      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), "Channels");
    else
      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), "Velocity");
    gtk_signal_connect (GTK_OBJECT(GTK_COMBO (combo)->entry), "changed",
			GTK_SIGNAL_FUNC (window_unit_change), NULL);
    gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);
    gtk_widget_show (label);   gtk_widget_show (combo);   gtk_widget_show (hbox); 
    /*  wind_type = 1;*/
    
    for (i = 0; i<MAXWIND; i++) {
      hbox = gtk_hbox_new (FALSE, 10);
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->vbox),
			  hbox, FALSE, FALSE, 0);
      sprintf(txt,"Window %1d",i+1);
      label = gtk_label_new (txt);
      gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
      gtk_widget_show (label);
      for (j=0; j<2; j++) {
	winentry[i][j] = gtk_entry_new();
	gtk_box_pack_start (GTK_BOX (hbox), winentry[i][j], FALSE, FALSE, 0);
	gtk_widget_set_usize (GTK_WIDGET (winentry[i][j]), 75, -1);
	gtk_widget_show (winentry[i][j]);
	if (i >= nwin) gtk_widget_set_sensitive (GTK_WIDGET (winentry[i][j]), FALSE);
      }
      gtk_widget_show (hbox);
    }
    
    
    settext_winentry();
    
    winlabel = gtk_label_new ("");
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->vbox),
			winlabel, TRUE, TRUE, 0);
    gtk_widget_show (winlabel);
    
    hbox = gtk_hbox_new (FALSE, 5);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area),
			hbox, TRUE, TRUE, 0);
    button = gtk_button_new_with_label ("Set");
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (set_windows), dialog_window);
    gtk_widget_show (button);
    
    
    button = gtk_button_new_with_label ("Cancel");
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (dialog_window));
    gtk_widget_show (button);
    
    gtk_widget_show (hbox);
    gtk_widget_show (label);
  }
  if (!GTK_WIDGET_VISIBLE (dialog_window)) 
    gtk_widget_show(dialog_window);
  else 
    gtk_widget_destroy (dialog_window);
}

void
assessment_change (GtkWidget *widget, gpointer data)
{
  char txt[10];

  strcpy(txt,gtk_entry_get_text(GTK_ENTRY( widget)));
  if (!strcmp(txt,"RMS")) {
    assess_mode = 0;
  }
  if (log_mode == LOG_VERBOSE) {
    sprintf(logtext_string,"Assessment Mode changed to %s\n",txt);
    append_log(0,1,1,otfgui->logtext,logtext_string);
  }
} 

GtkWidget
*setup_baseline (int mode)
{

  GtkWidget *frame;
  GtkWidget *vbox2;
  GtkWidget *box1;
  GList *glist = NULL;
  GList *glist1 = NULL;
  GtkTooltips *tooltips;
  GtkWidget *labelbox;
  GtkWidget *button;
  GtkWidget *label;
  GtkWidget *combo;


  frame = gtk_frame_new (NULL);

  vbox2 = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (frame), vbox2);
  
  if (mode == 0) {
    box1 = gtk_hbox_new (FALSE, 5);
    gtk_box_pack_start (GTK_BOX (vbox2), box1, FALSE, FALSE, 0);
    
    label = gtk_label_new ("Reference Weighting Scheme");
    gtk_box_pack_start (GTK_BOX (box1), label, FALSE, TRUE, 0);
    gtk_widget_show (label);
    combo = gtk_combo_new ();
    glist1 = g_list_append(glist1, "Equal");  
    glist1 = g_list_append(glist1, "Biased");
    gtk_combo_set_popdown_strings (GTK_COMBO (combo), glist1);
    gtk_entry_set_editable( GTK_ENTRY (GTK_COMBO (combo)->entry), FALSE);
    gtk_signal_connect (GTK_OBJECT (GTK_COMBO (combo)->entry), "changed",
			GTK_SIGNAL_FUNC (weight_combo_change), NULL);
    gtk_box_pack_start (GTK_BOX (box1), combo, FALSE, FALSE, 0);
    gtk_widget_set_usize (GTK_WIDGET (combo), 80, -1);
    sprintf(logtext_string, "%s",get_weight_string());
    gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), logtext_string);  
    gtk_widget_show (combo); gtk_widget_show (box1);
  }

  box1 = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox2), box1, FALSE, FALSE, 0);

  label = gtk_label_new ("Baseline Order");
  gtk_box_pack_start (GTK_BOX (box1), label, FALSE, TRUE, 0);

  baselinewgt->order = gtk_combo_new ();
  glist = g_list_append (glist, NULL);
  glist = g_list_append (glist, "0");  glist = g_list_append (glist, "1");
  glist = g_list_append (glist, "2");  glist = g_list_append (glist, "3");
  glist = g_list_append (glist, "4");  glist = g_list_append (glist, "5");  


  gtk_combo_set_popdown_strings( GTK_COMBO(baselinewgt->order), glist) ;
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (baselinewgt->order)->entry), FALSE);
  gtk_signal_connect (GTK_OBJECT(GTK_COMBO (baselinewgt->order)->entry), "changed",
                           GTK_SIGNAL_FUNC (order_change), NULL);

  gtk_box_pack_start (GTK_BOX (box1), baselinewgt->order, FALSE, FALSE, 0);
  gtk_widget_set_usize (GTK_WIDGET (baselinewgt->order), 50, -1);
  /*  sprintf(logtext_string, "%1d",get_order());
      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (baselinewgt->order)->entry), logtext_string);*/
  
  gtk_widget_show (baselinewgt->order);
  gtk_widget_show (label);


  tooltips = gtk_tooltips_new ();
  labelbox = gtk_event_box_new ();
  label = gtk_label_new ("# Windows");
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_tooltips_set_tip (tooltips, labelbox, 
                        "Number of baseline windows", NULL);

  gtk_widget_show (label);  gtk_widget_show (labelbox);

  baselinewgt->nwindow = gtk_combo_new();
  glist = NULL; glist = g_list_append (glist, "1");
  glist = g_list_append (glist, "2");glist = g_list_append (glist, "3");
  glist = g_list_append (glist, "4");glist = g_list_append (glist, "5");  
  gtk_combo_set_popdown_strings( GTK_COMBO(baselinewgt->nwindow), glist) ;
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (baselinewgt->nwindow)->entry), FALSE);
  gtk_signal_connect (GTK_OBJECT(GTK_COMBO (baselinewgt->nwindow)->entry), "changed",
                           GTK_SIGNAL_FUNC (nwindow_change), NULL);
  /*  sprintf(logtext_string, "%1d",get_nwind());*/

  gtk_box_pack_end (GTK_BOX (box1), baselinewgt->nwindow, FALSE, FALSE, 0);
  gtk_box_pack_end (GTK_BOX (box1), labelbox, FALSE, FALSE, 0);  
  gtk_widget_set_usize (GTK_WIDGET (baselinewgt->nwindow), 40, -1);

  gtk_widget_show (baselinewgt->nwindow);
  /*  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (baselinewgt->nwindow)->entry), logtext_string);*/
  gtk_widget_show (box1);

  box1 = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox2), box1, FALSE, FALSE, 0);

  labelbox = gtk_event_box_new ();
  label = gtk_label_new ("Windows:");
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
  gtk_tooltips_set_tip (tooltips, labelbox, 
                        "Comma-separated Channel pairs; use Edit to set", NULL);
  gtk_box_pack_start (GTK_BOX (box1), labelbox, FALSE, FALSE, 0);  

  /*  gtk_widget_realize (frame);*/

  button = gtk_button_new_with_label ("Edit");
  gtk_box_pack_start (GTK_BOX (box1), button, FALSE, FALSE, 0);  
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (edit_windows), NULL);
  gtk_widget_show (button);

  baselinewgt->window = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (box1), baselinewgt->window, FALSE, FALSE, 0);  

  gtk_widget_show (label); gtk_widget_show (labelbox); gtk_widget_show (box1);
  gtk_widget_show (baselinewgt->window);
  gtk_widget_set_usize (GTK_WIDGET (baselinewgt->window), 300, -1);
  gtk_entry_set_editable (GTK_ENTRY (baselinewgt->window), FALSE);
  gtk_widget_show (box1);


  gtk_widget_show (vbox2);
  gtk_widget_show (frame);
  /*  gtk_widget_realize (frame);*/
  return (frame);

}

GtkWidget
*setup_noise ()
{
  GtkWidget *frame;
  GtkWidget *hbox;
  GtkWidget *button;
  GSList *group;
  /*  GtkWidget *vbox1, *vbox2;
      GtkWidget *label; */

  frame = gtk_frame_new ("Noise Weighting Method");

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (frame), hbox);
  
  
  button = gtk_radio_button_new_with_label (NULL, "RMS");
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  gtk_signal_connect(GTK_OBJECT(button), "toggled",
		     GTK_SIGNAL_FUNC(noiseweightToggleCB), (gpointer)0);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), 
			      rmsweight == 1 ? TRUE : FALSE);


  group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
  button = gtk_radio_button_new_with_label(group, "TSYS");
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  gtk_signal_connect(GTK_OBJECT(button), "toggled",
		     GTK_SIGNAL_FUNC(noiseweightToggleCB), (gpointer)1);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), 
			      tsysweight == 1 ? TRUE : FALSE);
  gtk_widget_show (hbox);
  gtk_widget_show (frame);
  
  return(frame);
}


  
  
void
flatfieldbtnToggleCB(GtkWidget *widget, gpointer data)
{

  if (GTK_TOGGLE_BUTTON (widget)->active) 
    {
      /* If control reaches here, the toggle button is down */
      flatfield = 1;
      gtk_label_set_text (GTK_LABEL (GTK_BIN (widget)->child), "Yes");
      append_log(0,1,1,otfgui->logtext,"Flat fielding turned");
      append_log(2,1,1,otfgui->logtext," ON\n");
    } else {
      /* If control reaches here, the toggle button is up */
      flatfield = 0;
      gtk_label_set_text (GTK_LABEL (GTK_BIN (widget)->child), "No");
      append_log(0,1,1,otfgui->logtext,"Flat fielding turned");
      append_log(2,1,1,otfgui->logtext," OFF\n");
    }
}

void
remove_baseline_btnToggleCB(GtkWidget *widget, GtkWidget *vbox)
{

  if (GTK_TOGGLE_BUTTON (widget)->active) 
    {
      /* If control reaches here, the toggle button is down */
      base = 1;
      gtk_label_set_text (GTK_LABEL (GTK_BIN (widget)->child), "Yes");
      append_log(0,1,1,otfgui->logtext,"Baselines will be removed \n");
      /*      if ((otfgui->basewin != NULL) && (GTK_IS_WIDGET (otfgui->basewin))) */
      if (GTK_IS_WIDGET (otfgui->basewin))
	gtk_widget_set_sensitive (otfgui->basewin, TRUE);
    } else {
      /* If control reaches here, the toggle button is up */
      base = 0;
      gtk_label_set_text (GTK_LABEL (GTK_BIN (widget)->child), "No");
      append_log(0,1,1,otfgui->logtext,"Baselines will not be removed \n");
      if (GTK_WIDGET_SENSITIVE (otfgui->basewin)) {
	if (!noise_weight) {
	  if (otfgui->basewin != NULL)
	    gtk_widget_set_sensitive(otfgui->basewin, FALSE);
	} else {
	  if (!rmsweight) {
	    if (otfgui->basewin != NULL)
	    gtk_widget_set_sensitive(otfgui->basewin, FALSE);
	  }
	}
      }
    }
}

void
noise_weighting_btnToggleCB(GtkWidget *widget, GtkWidget *vbox)
{

  if (GTK_TOGGLE_BUTTON (widget)->active) 
    {
      /* If control reaches here, the toggle button is down */
      noise_weight = 1;
      gtk_label_set_text (GTK_LABEL (GTK_BIN (widget)->child), "Yes");
      append_log(0,1,1,otfgui->logtext,"Data will be weighted by its noise before regridding \n");
      if (GTK_IS_WIDGET (otfgui->noisewin))
	gtk_widget_set_sensitive (otfgui->noisewin, TRUE);
      if (rmsweight) {
	if (GTK_IS_WIDGET (otfgui->basewin))
	  gtk_widget_set_sensitive(otfgui->basewin, TRUE);
      }
    } else {
      /* If control reaches here, the toggle button is up */
      noise_weight = 0;
      gtk_label_set_text (GTK_LABEL (GTK_BIN (widget)->child), "No");
      append_log(0,1,1,otfgui->logtext,"No noise weighting \n");
      if (GTK_IS_WIDGET( otfgui->noisewin))
	gtk_widget_set_sensitive (otfgui->noisewin, FALSE);
      if (!base) {
	if (GTK_IS_WIDGET(otfgui->basewin))
	  gtk_widget_set_sensitive (otfgui->basewin, FALSE);
      }
    }
}
  

void
include_margin_btnToggleCB(GtkWidget *widget, GtkWidget *vbox)
{

  if (GTK_TOGGLE_BUTTON (widget)->active) 
    {
      /* If control reaches here, the toggle button is down */
      include_margin = 1;
      gtk_label_set_text (GTK_LABEL (GTK_BIN (widget)->child), "Yes");
      append_log(0,1,1,otfgui->logtext,"Ramp Margins will be included. \n");
    } else {
      /* If control reaches here, the toggle button is up */
      include_margin = 0;
      gtk_label_set_text (GTK_LABEL (GTK_BIN (widget)->child), "No");
      append_log(0,1,1,otfgui->logtext,"Ramp Margins will NOT be included. \n");      
    }
}

void
otftool_setup_otfedit(otf_gui *otfg, GtkWidget *vbox)
{
  GtkWidget *hbox, *hbox1;
  GtkWidget *vbox2;
  GtkWidget *vbox_left, *vbox_right;
  GtkWidget *box1;
  GtkWidget *frame;
  GtkWidget *button;
  /*  GSList *group;*/
  GtkWidget *table;
  GtkTooltips *tooltips;
  GtkWidget *labelbox, *label;
  GtkWidget *scrolled_window;
  /*  GtkWidget *clist;*/
  gchar *titles[1] = { "OTF File List" };
  GtkWidget *separator;
  GList *glist = NULL;
  GtkWidget *combo;
  GtkWidget *vvbox;

  notebookmode = DATAEDIT;

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);  


  vbox_left = gtk_vbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (hbox), vbox_left, TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox_left), hbox1, FALSE, TRUE, 0);  

  frame = gtk_frame_new ("Required Parameters");
  gtk_box_pack_start (GTK_BOX (hbox1), frame, TRUE, TRUE, 0);
  /*  gtk_widget_set_usize(hbox1, -1, 80);*/

  vbox2 = gtk_vbox_new (FALSE,5);
  gtk_container_add (GTK_CONTAINER (frame), vbox2);

  box1 = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox2), box1, FALSE, FALSE, 0);
  
  table = gtk_table_new (1, 3, TRUE);
  gtk_container_add( GTK_CONTAINER (box1), table);
  
  tooltips = gtk_tooltips_new ();
  labelbox = gtk_event_box_new ();
  label = gtk_label_new ("Input list file");
  /*  gtk_widget_set_usize(label,15,-1);*/
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 0, 1);
  gtk_tooltips_set_tip (tooltips, labelbox, 
                        "Input file containing list of raw otf files", NULL);
  gtk_widget_show (label);
  gtk_widget_show (labelbox);

  otfg->entry[1] = gtk_entry_new();
  /*  gtk_widget_set_usize(otfg->entry[1],50,-1);*/
  gtk_table_attach_defaults (GTK_TABLE (table), otfg->entry[1],
                             1,2,0,1);
  gtk_widget_show (otfg->entry[1]);
  gtk_signal_connect (GTK_OBJECT (otfg->entry[1]), "activate",
		      GTK_SIGNAL_FUNC (entry_changed), (gpointer) 1);
  /*  gtk_signal_connect (GTK_OBJECT (otfg->entry[1]), "focus-out-event",
      GTK_SIGNAL_FUNC (entry_left), (gpointer) 1);*/

  if (strcmp(otffile,""))
    gtk_entry_set_text (GTK_ENTRY (otfg->entry[1]), otffile);

  button = gtk_button_new_with_label("Browse");
  gtk_table_attach_defaults (GTK_TABLE (table), button,
                             2,3,0,1);  
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (filesel_inp), NULL);


  gtk_widget_show (table);
  gtk_widget_show (box1);
  gtk_widget_show (frame);
  gtk_widget_show (vbox2);
  gtk_widget_show (hbox1);

  separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (vbox2), separator, FALSE, FALSE, 0);
  gtk_widget_show (separator);
  hbox1 = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, TRUE, 0);  
  

  vvbox = gtk_vbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (hbox1), vvbox, FALSE, FALSE, 0);

  box1 = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vvbox), box1, FALSE, FALSE, 0);

  vbox2 = gtk_vbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox2, FALSE, FALSE, 0);
  
  label = gtk_label_new ("Assessment Mode");
  gtk_box_pack_start (GTK_BOX (box1), label, FALSE, TRUE, 0);
  combo = gtk_combo_new ();
  glist = g_list_append (glist, "RMS"); 


  gtk_combo_set_popdown_strings ( GTK_COMBO(combo), glist) ;
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (combo)->entry), FALSE);
  gtk_signal_connect (GTK_OBJECT(GTK_COMBO (combo)->entry), "changed",
		      GTK_SIGNAL_FUNC (assessment_change), NULL);

  gtk_box_pack_start (GTK_BOX (box1), combo, FALSE, FALSE, 0);
  gtk_widget_set_usize (GTK_WIDGET (combo), 70, -1);

  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), "RMS");
  gtk_widget_show (label);
  gtk_widget_show (combo);
  gtk_widget_show (box1);
  gtk_widget_show (vbox2);
  gtk_widget_show (hbox1);
  gtk_widget_show (vvbox);

  box1 = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vvbox), box1, FALSE, FALSE, 0);

  labelbox = gtk_event_box_new ();
  label = gtk_label_new ("RMS Threshold");
  /*  gtk_widget_set_usize(label,15,-1);*/
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_box_pack_start (GTK_BOX (box1), labelbox, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Spectra with RMS above this value will not be included in baseline statistics", NULL);
  gtk_widget_show (label);
  gtk_widget_show (labelbox);

  otfg->entry[4] = gtk_entry_new();
  gtk_box_pack_start (GTK_BOX (box1), otfg->entry[4], FALSE, FALSE, 0);
  gtk_widget_show (otfg->entry[4]);
  gtk_signal_connect (GTK_OBJECT (otfg->entry[4]), "activate",
                        GTK_SIGNAL_FUNC (entry_changed), (gpointer) 4);
  sprintf(logtext_string, "%g",rmsthreshold);
  gtk_entry_set_text (GTK_ENTRY (otfg->entry[4]), logtext_string);

  gtk_widget_show (box1);


  if (assess_mode == 0 ) {
    otfg->basewin = setup_baseline (0);
    gtk_box_pack_start (GTK_BOX (vbox2), otfg->basewin, FALSE, FALSE, 0);
    settext_baselines();
  }


  gtk_widget_show (vbox_left);


  /* Now the right hand side clist widget */
  vbox_right = gtk_vbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (hbox), vbox_right, TRUE, TRUE, 0);
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  gtk_box_pack_start(GTK_BOX(vbox_right), scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show (scrolled_window);

  /* Create the CList . For now we use just 1 column */
  otfg->clist = gtk_clist_new_with_titles (1, titles);

  /* When a selection is made, we want to know about it. */
  gtk_signal_connect(GTK_OBJECT(otfg->clist), "select_row",
		     GTK_SIGNAL_FUNC(otffile_selection_made),
		     NULL);

  gtk_signal_connect_object (GTK_OBJECT(otfg->clist), "button_press_event",
		     GTK_SIGNAL_FUNC(clist_button_pressed),
		     GTK_OBJECT (otfg->clist));

  gtk_clist_set_shadow_type (GTK_CLIST(otfg->clist), GTK_SHADOW_ETCHED_IN);
  gtk_clist_set_column_width (GTK_CLIST(otfg->clist), 0, 150);
  gtk_clist_column_titles_passive (GTK_CLIST (otfg->clist) );
  gtk_clist_set_selection_mode (GTK_CLIST (otfg->clist), GTK_SELECTION_SINGLE);


  gtk_container_add(GTK_CONTAINER(scrolled_window), otfg->clist);
  gtk_widget_show(otfg->clist);
  gtk_widget_set_usize (scrolled_window, 200, -1);

  separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (vbox_right), separator, FALSE, TRUE, 0);
  gtk_widget_show (separator);
  
  button = gtk_button_new_with_label("Load File List");
  gtk_box_pack_start (GTK_BOX (vbox_right), button, FALSE, FALSE, 0);
  gtk_widget_show (button);
  gtk_signal_connect_object (GTK_OBJECT(button), "clicked",
			     GTK_SIGNAL_FUNC(attach_list),
			     (gpointer) otfg->clist);

  /* Now add the file list into the window */

  if (strcmp(otffile,""))
      attach_list(otfg->clist);


  /*back to the left side */
  frame = gtk_frame_new ("Optional Parameters");
  gtk_box_pack_start (GTK_BOX (vbox_left), frame, TRUE, TRUE, 0);

  vbox2 = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (frame), vbox2);
  
  table = gtk_table_new (1, 3, TRUE);
  gtk_container_add( GTK_CONTAINER (vbox2), table);

  /*
  label = gtk_label_new ("Horn Mask");
  gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 1, 0, 1);
  gtk_widget_show (label);
  */

  button = gtk_button_new_with_label ("Set Horn Mask");
  gtk_tooltips_set_tip (tooltips, button, 
			"Pops up window to select horns to be used for assessment", NULL);  
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (hornmask_win), NULL);
  gtk_table_attach_defaults(GTK_TABLE (table), button, 0, 1, 0, 1);


  labelbox = gtk_event_box_new ();
  otfg->hornmasklabel[0] = gtk_label_new(bin_prnt_int(CmdLineMASK));
  gtk_container_add (GTK_CONTAINER (labelbox), otfg->hornmasklabel[0]);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"current horn mask: 1 is ON, 0 is off", NULL);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox,
			     1,3,0,1);
  gtk_widget_show (otfg->hornmasklabel[0]); gtk_widget_show (labelbox);

  gtk_widget_show(table);
  gtk_widget_show(frame);
  gtk_widget_show(vbox2);


  vbox2 = gtk_vbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox_left), vbox2, FALSE, FALSE, 0);

  table = gtk_table_new (1, 3, TRUE);
  gtk_container_add( GTK_CONTAINER (vbox2), table);
  
  button = gtk_button_new_with_label ("Edit Selected");
  gtk_tooltips_set_tip (tooltips, button, "Edit or Examine the selected OTF File from the list on right", NULL);
  gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 0, 1);
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
      GTK_SIGNAL_FUNC (edit_selected), NULL);
  /*  gtk_signal_connect (GTK_OBJECT (button), "clicked",
      GTK_SIGNAL_FUNC (edit_selected), NULL);*/




  button = gtk_button_new_with_label ("Baseline Selected");
  gtk_tooltips_set_tip (tooltips, button, "Baseline all horns and all spectra in selected file from the list on right", NULL);
  gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 0, 1);
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
		      GTK_SIGNAL_FUNC (baseline_selected), NULL);


  button = gtk_button_new_with_label ("Baseline All");
  gtk_tooltips_set_tip (tooltips, button, "Baseline all horns and all spectra for all files in the list on right", NULL);
  gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
		      GTK_SIGNAL_FUNC (baseline_all), NULL);

  gtk_widget_show (table);
  
  gtk_widget_show (vbox2);
  gtk_widget_show (frame);
  
  gtk_widget_show (scrolled_window);
  gtk_widget_show (vbox_right);

  gtk_widget_show (hbox);
  gtk_widget_show (vbox);
}


void
otftool_setup_otfmap(otf_gui *otfg, GtkWidget *vbox)
{
  GtkWidget *hbox1;
  GtkWidget *frame;
  GtkWidget *vbox1, *vbox2;
  GtkWidget *button;
  GSList *group;
  GtkWidget *table;
  GtkTooltips *tooltips;
  GtkWidget *labelbox, *label;
  /*  GtkWidget *tablabel1, *tablabel2;*/
  int i = 0;
  GList *glist = NULL;
  GList *glist1 = NULL;
  GtkWidget *combo;
  GtkWidget *vboxup, *vboxdown;

  notebookmode = DATAGRID;
  /* Now put together the box with required parameters */
  vbox1 = gtk_vbox_new (FALSE, 5);

  gtk_box_pack_start (GTK_BOX (vbox), vbox1, TRUE, TRUE, 0);
  frame = gtk_frame_new ("Required Parameters");
  gtk_container_add (GTK_CONTAINER (vbox1), frame);


  /* Make a table of 3 rows and 3 columns */
  table = gtk_table_new (6, 3, TRUE);
  gtk_container_add( GTK_CONTAINER (frame), table);

  tooltips = gtk_tooltips_new ();
  
  /*1st row */

  labelbox = gtk_event_box_new ();
  label = gtk_label_new ("Input list file");
  /*  gtk_widget_set_usize(label,15,-1);*/
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 0, 1);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Input file containing list of raw otf files", NULL);
  gtk_widget_show (label);
  gtk_widget_show (labelbox);

  otfg->entry[1] = gtk_entry_new();
  /*  gtk_widget_set_usize(otfg->entry[1],50,-1);*/
  gtk_table_attach_defaults (GTK_TABLE (table), otfg->entry[1],
			     1,2, 0, 1);
  gtk_widget_show (otfg->entry[1]);
  gtk_signal_connect (GTK_OBJECT (otfg->entry[1]), "activate",
                        GTK_SIGNAL_FUNC (entry_changed), (gpointer) 1);  
  /*  gtk_signal_connect (GTK_OBJECT (otfg->entry[1]), "focus-out-event",
      GTK_SIGNAL_FUNC (entry_left), (gpointer) 1);*/
  if (strcmp(otffile,""))
    gtk_entry_set_text (GTK_ENTRY (otfg->entry[1]), otffile);

  button = gtk_button_new_with_label("Browse");
  gtk_table_attach_defaults (GTK_TABLE (table), button,
			     2,3, 0, 1);  
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (filesel_inp), NULL);


  /* 2nd row */


  labelbox = gtk_event_box_new ();
  label = gtk_label_new ("Output File");
  /*  gtk_widget_set_usize(label,15,-1);*/
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 1, 2);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Output FITS or CLASS filename", NULL);
  gtk_widget_show (label);
  gtk_widget_show (labelbox);

  otfg->entry[2] = gtk_entry_new();
  /*  gtk_widget_set_usize(otfg->entry[2],50,-1);*/
  gtk_table_attach_defaults (GTK_TABLE (table), otfg->entry[2],
			     1,2,1,2);
  gtk_widget_show (otfg->entry[2]);
  gtk_signal_connect (GTK_OBJECT (otfg->entry[2]), "activate",
                        GTK_SIGNAL_FUNC (entry_changed), (gpointer) 2);  
  /*  gtk_signal_connect (GTK_OBJECT (otfg->entry[2]), "focus-out-event",
      GTK_SIGNAL_FUNC (entry_left), (gpointer) 2);*/
  if (strcmp(outfile,""))
    gtk_entry_set_text (GTK_ENTRY (otfg->entry[2]), outfile);

  button = gtk_button_new_with_label("Browse");
  gtk_table_attach_defaults (GTK_TABLE (table), button,
			     2,3,1,2);  
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (filesel_out), NULL);


  /* 3rd row */
  labelbox = gtk_event_box_new ();
  label = gtk_label_new ("Output File Type");
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 2, 3);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Format of output file", NULL);
  gtk_widget_show (label);
  gtk_widget_show (labelbox);

  hbox1 = gtk_hbox_new (FALSE, 5);
  gtk_table_attach_defaults (GTK_TABLE (table), hbox1, 1, 2, 2, 3);

  button = gtk_radio_button_new_with_label (NULL, "FITS");
  gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, FALSE, 0);
  gtk_widget_show (button);
  gtk_signal_connect(GTK_OBJECT(button), "toggled",
		     GTK_SIGNAL_FUNC(outtypeToggleCB), (gpointer)1);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), 
			      getouttype() == 1 ? TRUE : FALSE);


  group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
  button = gtk_radio_button_new_with_label(group, "CLASS");
  gtk_box_pack_end (GTK_BOX (hbox1), button, FALSE, FALSE, 0);
  gtk_widget_show (button);
  gtk_signal_connect(GTK_OBJECT(button), "toggled",
		     GTK_SIGNAL_FUNC(outtypeToggleCB), (gpointer)2);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), 
			      getouttype() == 2 ? TRUE : FALSE);
  gtk_widget_show (hbox1);




  /* 4th row */
  labelbox = gtk_event_box_new ();
  label = gtk_label_new ("Output Cell Size");
  /*  gtk_widget_set_usize(label,15,-1);*/
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 3, 4);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Output step size in arcseconds", NULL);
  gtk_widget_show (label);
  gtk_widget_show (labelbox);

  otfg->entry[0] = gtk_entry_new();
  gtk_table_attach_defaults (GTK_TABLE (table), otfg->entry[0],
			     1,2,3,4);
  gtk_widget_show (otfg->entry[0]);
  gtk_signal_connect (GTK_OBJECT (otfg->entry[0]), "activate",
                        GTK_SIGNAL_FUNC (entry_changed), (gpointer) 0);
  /*  gtk_signal_connect (GTK_OBJECT (otfg->entry[0]), "focus-out-event",
      GTK_SIGNAL_FUNC (entry_left), (gpointer) 0);*/

  if (THETA_N != 0) {
    sprintf(logtext_string, "%g",THETA_N);
    gtk_entry_set_text (GTK_ENTRY (otfg->entry[0]), logtext_string);
  }



  label = gtk_label_new("in arcseconds");
  gtk_widget_show (label);
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     2,3,3,4);
  /*  gtk_entry_set_text (GTK_ENTRY (otfg->entry[0]), "25");*/


  /* 5th row */
  label = gtk_label_new ("Reference Weighting Scheme");
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     0,1,4,5);
  gtk_widget_show (label);

  combo = gtk_combo_new ();
  glist1 = g_list_append(glist1, "Equal");  
  glist1 = g_list_append(glist1, "Biased");
  gtk_combo_set_popdown_strings (GTK_COMBO (combo), glist1);
  gtk_entry_set_editable( GTK_ENTRY (GTK_COMBO (combo)->entry), FALSE);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO (combo)->entry), "changed",
		      GTK_SIGNAL_FUNC (weight_combo_change), NULL);
  gtk_table_attach_defaults (GTK_TABLE (table), combo,
			     1, 2, 4, 5);
  gtk_widget_set_usize (GTK_WIDGET (combo), 80, -1);
  sprintf(logtext_string, "%s",get_weight_string());
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), logtext_string);  
  gtk_widget_show (combo); 
  

  /* 6th row */

  
  labelbox = gtk_event_box_new();
  label = gtk_label_new ("Spatial Filtering Kernel");

  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 5, 6); 
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Spatial Convolution Function for regridding - jinc*gaussian is default", NULL);
  gtk_widget_show (label);
  gtk_widget_show (labelbox);

  combo = gtk_combo_new();
  glist = NULL; glist = g_list_append (glist, "jinc*gaussian");
  glist = g_list_append (glist, "sinc*gaussian");
  glist = g_list_append (glist, "sinc");
  glist = g_list_append (glist, "gaussian");
  gtk_combo_set_popdown_strings( GTK_COMBO(combo), glist) ;
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (combo)->entry), FALSE);
  gtk_signal_connect (GTK_OBJECT(GTK_COMBO (combo)->entry), "changed",
      GTK_SIGNAL_FUNC (kopt_change), NULL);
  gtk_widget_set_usize (GTK_WIDGET (combo), 125, -1);
  gtk_table_attach_defaults (GTK_TABLE (table), combo, 1, 2, 5, 6); 
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry),getkopt(kopt));

  gtk_widget_show (combo);


  
  gtk_widget_show(table);
  gtk_widget_show(frame);
  gtk_widget_show(vbox1);


  /* Now put together the lower box for optional parameters */
  hbox1 = gtk_hbox_new (FALSE, 5);
  vbox1 = gtk_vbox_new (FALSE, 5);
  vbox2 = gtk_vbox_new (FALSE, 5);
  vboxup = gtk_vbox_new (FALSE, 5);
  vboxdown = gtk_vbox_new (FALSE, 5);

  gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox1, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox2), vboxup, TRUE, TRUE, 0);
  gtk_box_pack_end (GTK_BOX (vbox2), vboxdown, TRUE, TRUE, 0);

  gtk_widget_show (vbox2); 
  
  /* optional right frames */
  otfg->basewin = setup_baseline(1);
  settext_baselines();
  gtk_box_pack_start (GTK_BOX (vboxup), otfgui->basewin, TRUE, TRUE, 0);
  gtk_widget_show (vboxup);
  if (!base) {
    if (GTK_IS_WIDGET (otfgui->basewin))
      gtk_widget_set_sensitive (otfg->basewin, FALSE);
  }
  otfg->noisewin = setup_noise();
  gtk_box_pack_start (GTK_BOX (vboxdown), otfg->noisewin, TRUE, TRUE, 0);
  gtk_widget_show (vboxdown);
  if (!noise_weight) {
    if (GTK_IS_WIDGET (otfgui->noisewin))
	gtk_widget_set_sensitive (otfg->noisewin, FALSE);
  }


  frame = gtk_frame_new ("Optional Parameters");
  gtk_container_add (GTK_CONTAINER (vbox1), frame);

  /* Make a table of 7 rows and 2 columns */
  table = gtk_table_new (7, 2, TRUE);
  gtk_container_add( GTK_CONTAINER (frame), table);

  /* 1st row */
  labelbox = gtk_event_box_new ();
  label = gtk_label_new ("Linename");
  /*  gtk_widget_set_usize(label,15,-1);*/
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 0, 1);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Linename string to be added to Class or Fits output file", NULL);
  gtk_widget_show (label);
  gtk_widget_show (labelbox);

  otfg->entry[3] = gtk_entry_new();
  gtk_table_attach_defaults (GTK_TABLE (table), otfg->entry[3],
			     1,2,0,1);
  gtk_widget_show (otfg->entry[3]);
  gtk_signal_connect (GTK_OBJECT (otfg->entry[3]), "activate",
                        GTK_SIGNAL_FUNC (entry_changed), (gpointer) 3);
  /*  gtk_signal_connect (GTK_OBJECT (otfg->entry[3]), "focus-out-event",
      GTK_SIGNAL_FUNC (entry_left), (gpointer) 3);*/
  if (strcmp(linename,""))
    gtk_entry_set_text(GTK_ENTRY (otfg->entry[3]), linename);


  /* 2nd row */

  labelbox = gtk_event_box_new ();
  label = gtk_label_new ("RMS Threshold");
  /*  gtk_widget_set_usize(label,15,-1);*/
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 1, 2);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Spectra with RMS above this value will not be included in regridding", NULL);
  gtk_widget_show (label);
  gtk_widget_show (labelbox);

  otfg->entry[4] = gtk_entry_new();
  gtk_table_attach_defaults (GTK_TABLE (table), otfg->entry[4],
			     1,2,1,2);
  gtk_widget_show (otfg->entry[4]);
  gtk_signal_connect (GTK_OBJECT (otfg->entry[4]), "activate",
                        GTK_SIGNAL_FUNC (entry_changed), (gpointer) 4);
  /*  gtk_signal_connect (GTK_OBJECT (otfg->entry[3]), "focus-out-event",
      GTK_SIGNAL_FUNC (entry_left), (gpointer) 3);*/
  sprintf(logtext_string, "%g",rmsthreshold);
  gtk_entry_set_text (GTK_ENTRY (otfg->entry[4]), logtext_string);
  



  /* 3rd row */

  button = gtk_button_new_with_label ("Set Horn Mask");
  gtk_tooltips_set_tip (tooltips, button, 
			"Pops up window to select horns to be used for regridding", NULL);  
  gtk_widget_show (button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (hornmask_win), NULL);
  gtk_table_attach_defaults(GTK_TABLE (table), button, 0, 1, 2, 3);


  labelbox = gtk_event_box_new ();
  otfg->hornmasklabel[0] = gtk_label_new(bin_prnt_int(CmdLineMASK));
  gtk_container_add (GTK_CONTAINER (labelbox), otfg->hornmasklabel[0]);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"current horn mask: 1 is ON, 0 is off", NULL);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox,
			     1,2, 2,3);
  gtk_widget_show (otfg->hornmasklabel[0]); gtk_widget_show (labelbox);


  /* 4th row */
  labelbox = gtk_event_box_new ();
  label = gtk_label_new("Flat Field");
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 3, 4);
  gtk_widget_show (label); gtk_widget_show (labelbox);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Apply relative beam efficiency for array horns before regridding", NULL);

  if (flatfield)  button = gtk_check_button_new_with_label ("Yes");
  else button = gtk_check_button_new_with_label ("No");
  gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 3, 4);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), flatfield);
  gtk_signal_connect(GTK_OBJECT(button), "toggled",
		     GTK_SIGNAL_FUNC(flatfieldbtnToggleCB), (gpointer)i);
  gtk_widget_show (button);

  /* 5th row */

  labelbox = gtk_event_box_new ();
  label = gtk_label_new("Remove Baselines");
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 4, 5);
  gtk_widget_show (label); gtk_widget_show (labelbox);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Remove Baselines prior to regridding?", NULL);
  
  if (base) button = gtk_check_button_new_with_label ("Yes");
  else button = gtk_check_button_new_with_label ("No");
  gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 4, 5);
  gtk_signal_connect(GTK_OBJECT(button), "toggled",
		     GTK_SIGNAL_FUNC(remove_baseline_btnToggleCB), vboxup);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), base);
  gtk_widget_show (button);


  /*6th row */
  
  labelbox = gtk_event_box_new ();
  label = gtk_label_new("Noise Weighting");
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 5, 6);
  gtk_widget_show (label); gtk_widget_show (labelbox);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Should noise weighting be considered in addition to spatial weights?", NULL);
  
  if (noise_weight) button = gtk_check_button_new_with_label ("Yes");
  else button = gtk_check_button_new_with_label ("No");
  gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 5, 6);
  gtk_signal_connect(GTK_OBJECT(button), "toggled",
		     GTK_SIGNAL_FUNC(noise_weighting_btnToggleCB), vboxdown);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), noise_weight);
  gtk_widget_show (button);


  /*7th row */
  
  labelbox = gtk_event_box_new ();
  label = gtk_label_new("Include Ramps ?");
  gtk_container_add (GTK_CONTAINER (labelbox), label);
  gtk_table_attach_defaults (GTK_TABLE (table), labelbox, 0, 1, 6, 7);
  gtk_widget_show (label); gtk_widget_show (labelbox);
  gtk_tooltips_set_tip (tooltips, labelbox, 
			"Should the ramp margins on either side of the real map be included in making map?", NULL);
  
  if (include_margin) button = gtk_check_button_new_with_label ("Yes");
  else button = gtk_check_button_new_with_label ("No");
  gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 6, 7);
  gtk_signal_connect(GTK_OBJECT(button), "toggled",
		     GTK_SIGNAL_FUNC(include_margin_btnToggleCB), NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), include_margin);
  gtk_widget_show (button);
  
  

  
  gtk_widget_show(table);
  gtk_widget_show(frame);
  gtk_widget_show(vbox1);
  gtk_widget_show (hbox1);


  hbox1 = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);

  stop_regrid = 0;

  button = gtk_button_new_with_label ("Compute OTFMAP");
  gtk_tooltips_set_tip (tooltips, button, "Compute the Regridded output map", NULL);
  gtk_box_pack_start (GTK_BOX (hbox1), button, TRUE, TRUE, 0);  
  gtk_widget_show (button);

  /*
  exitbutton = gtk_button_new_with_label ("Exit");
  gtk_tooltips_set_tip (tooltips, exitbutton, "Exit otftool", NULL);
  gtk_box_pack_start (GTK_BOX (hbox1), exitbutton, TRUE, TRUE, 0);  
  gtk_signal_connect (GTK_OBJECT (exitbutton), "clicked",
		      GTK_SIGNAL_FUNC (destroy), NULL);
  gtk_widget_show (exitbutton);
  */

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (otftool_regrid), NULL);

  
  gtk_widget_show(hbox1);


  gtk_widget_show(vbox);
  /*  gtk_widget_show(hbox);*/

}

static void
turn_page (GtkWidget *widget, GtkNotebookPage *page, gint page_num,
	   otf_gui *otfg)
{
  /*  GtkNotebookPage *oldpage;*/
  int oldpagenum;

  oldpagenum = gtk_notebook_get_current_page (GTK_NOTEBOOK (widget));
  /*      printf("%d %d\n", oldpagenum, page_num);*/
  switch (oldpagenum) {
  case -1:
    break;
  case 0:
    otfg->page[page_num] = gtk_vbox_new (FALSE, 5);
    gtk_box_pack_start (GTK_BOX (otfg->vbox[page_num]), otfg->page[page_num], FALSE, FALSE, 0);
    gtk_widget_show (otfg->page[page_num]);
    switch (page_num) {
    case 1:
      otftool_setup_otfedit(otfg, otfg->page[page_num]);
      append_log(0,1,1,otfgui->logtext,"Switching to Data Assessment tab\n");
      break;
    case 2:
      otftool_setup_otfmap(otfg, otfg->page[page_num]);
      append_log(0,1,1,otfgui->logtext,"Switching to Regrid Data tab\n");
      break;
    default:
      break;
    }
    gtk_widget_destroy (GTK_WIDGET (otfg->page[0]));
    break;
  case 1:
    otfg->page[page_num] = gtk_vbox_new (FALSE, 5);
    gtk_box_pack_start (GTK_BOX (otfg->vbox[page_num]), otfg->page[page_num], FALSE, FALSE, 0);
    gtk_widget_show (otfg->page[page_num]);
    switch (page_num) {
    case 0: 
      otftool_setup_datasel (otfg, otfg->page[page_num]);
      append_log(0,1,1,otfgui->logtext,"Switching to Data Selection tab\n");
      break;
    case 2:
      otftool_setup_otfmap(otfg, otfg->page[page_num]);
      append_log(0,1,1,otfgui->logtext,"Switching to Regrid Data tab\n");
      break;
    default:
      break;
    }
    gtk_widget_destroy (GTK_WIDGET (otfg->page[1]));
    break;    
  case 2:
    otfg->page[page_num] = gtk_vbox_new (FALSE, 5);
    gtk_box_pack_start (GTK_BOX (otfg->vbox[page_num]), otfg->page[page_num], FALSE, FALSE, 0);
    gtk_widget_show (otfg->page[page_num]);
    switch (page_num) {
    case 0: 
      otftool_setup_datasel (otfg, otfg->page[page_num]);
      append_log(0,1,1,otfgui->logtext,"Switching to Data Selection tab\n");
      break;
    case 1:
      otftool_setup_otfedit(otfg, otfg->page[page_num]);
      append_log(0,1,1,otfgui->logtext,"Switching to Data Assessment tab\n");
      break;
    default:
      break;
    }
    gtk_widget_destroy (GTK_WIDGET (otfg->page[2]));
    break;    

  }
}


void
otftool_setup_body(otf_gui *otfg)
{

  /*  GtkWidget *vbox_map,*vbox_edit;*/
  GtkWidget *tablabel1, *tablabel2, *tablabel0;


  /* Add a vbox and hbox to start the packing */
  otfg->vbox[0] = gtk_vbox_new (FALSE, 5);
  otfg->vbox[1] = gtk_vbox_new (FALSE, 5);
  otfg->vbox[2] = gtk_vbox_new (FALSE, 5);

  /*  otfg->page[1] = gtk_vbox_new (FALSE, 5);*/
  otfg->page[0] = gtk_vbox_new (FALSE, 5);

  otfg->notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (otfg->notebook), GTK_POS_TOP);
  gtk_box_pack_start (GTK_BOX (otfg->vertbox), otfg->notebook, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (otfg->notebook), 10);

  tablabel0 = gtk_label_new ("Data File Selection");
  gtk_notebook_append_page(GTK_NOTEBOOK (otfg->notebook), otfg->vbox[0], tablabel0);
  gtk_box_pack_start (GTK_BOX (otfg->vbox[0]), otfg->page[0], FALSE, FALSE, 0);  

  tablabel1 = gtk_label_new("Data Assessment");
  gtk_notebook_append_page(GTK_NOTEBOOK (otfg->notebook), otfg->vbox[1], tablabel1);
  /*  gtk_box_pack_start (GTK_BOX (otfg->vbox[0]), otfg->page[1], FALSE, FALSE, 0);*/

  tablabel2 = gtk_label_new("Regrid Data");
  gtk_notebook_append_page(GTK_NOTEBOOK (otfg->notebook), otfg->vbox[2], tablabel2);
  /*gtk_box_pack_start (GTK_BOX (otfg->vbox[1]), otfg->page[2], TRUE, TRUE, 0);*/
 
  gtk_widget_show(otfg->vbox[0]);
  gtk_widget_show(otfg->vbox[1]);
  gtk_widget_show(otfg->vbox[2]);

  /*  gtk_widget_set_usize (otfg->notebook, 600, 390);*/
  gtk_widget_show(otfg->notebook);

  /*  gtk_container_add (GTK_CONTAINER (otfg->vertbox), vbox);*/


  /*  gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);*/


  otftool_setup_datasel (otfg, otfg->page[0]);
  /*  otftool_setup_otfedit(otfg,otfg->page[0]);*/

  gtk_signal_connect (GTK_OBJECT (otfg->notebook), "switch_page",
		      GTK_SIGNAL_FUNC (turn_page), otfg);

  /*  otftool_setup_otfmap(otfg, vbox);*/

  /*  gtk_widget_show (otfg->page[1]);*/
  gtk_widget_show (otfg->page[0]);

}


  











