/* setup_menu.c by Gopal Narayanan
 * This file containes the program that builds the menu bar
 */

#include "otftool.h"
#include <gdk/gdkkeysyms.h>

#define HELPF1 0
#define HELPKEYWORD 1
#define HELPOTFGEN 2
#define HELPDATAFILTER 3
#define HELPDATAASSESS 4
#define HELPDATAEDIT 5
#define HELPDATAREGRID 6
#define HELPCHANGES 7

/*extern static void turn_page (GtkWidget *, GtkNotebookPage *, gint, otf_gui *);*/

extern int save_defaults_as (char *);
extern int save_defaults (void);
extern int config_parse (char *);
extern void otftool_setup_datasel (otf_gui *, GtkWidget *);
extern void otftool_setup_otfmap (otf_gui *, GtkWidget *);
extern void otftool_setup_otfedit (otf_gui *, GtkWidget *);
extern gint show_mozilla_url (gchar *);
extern void error_mes (gchar *, gchar *);

void
filesave_ok_sel_inp (GtkWidget *w, GtkFileSelection *fs)
{
  save_defaults_as (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
  gtk_widget_destroy(GTK_WIDGET (fs));
}

void
fileload_ok_sel_inp (GtkWidget *w, GtkFileSelection *fs)
{
  int pagenum;
  /*  GtkNotebookPage *page;*/
  /*  GtkNotebook *nbook = GTK_NOTEBOOK (otfgui->notebook);*/

  config_parse (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
  gtk_widget_destroy(GTK_WIDGET (fs));
  pagenum = gtk_notebook_get_current_page (GTK_NOTEBOOK (otfgui->notebook));
  gtk_widget_destroy (GTK_WIDGET (otfgui->page[pagenum]));
  otfgui->page[pagenum] = gtk_vbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (otfgui->vbox[pagenum]), 
		      otfgui->page[pagenum], FALSE, FALSE, 0);
  gtk_widget_show (otfgui->page[pagenum]);
  switch (pagenum) {
  case -1:
    break;
  case 0:
    otftool_setup_datasel (otfgui, otfgui->page[pagenum]);
    break;
  case 1:
    otftool_setup_otfedit (otfgui, otfgui->page[pagenum]);
    break;
  case 2:
    otftool_setup_otfmap (otfgui, otfgui->page[pagenum]);
    break;
  default:
    break;
  }
  /*
  printf("%d \n",pagenum);
  page = nbook->cur_page;
  gtk_signal_emit_by_name (GTK_OBJECT (otfgui->notebook), "switch_page",
			   page, pagenum);
  */
  /*
  turn_page (otfgui->notebook, page, pagenum, otfgui);
  */
}

void 
loadCB (GtkWidget *parent)
{
  static GtkWidget *filew = NULL;
  
  if (!filew) {
    /* Create a new file selection widget */
    filew = gtk_file_selection_new ("Load configuration defaults");
    
    gtk_widget_set_name (GTK_WIDGET (filew), "aux window");
    gtk_file_selection_show_fileop_buttons (GTK_FILE_SELECTION (filew));
    gtk_signal_connect (GTK_OBJECT (filew), "destroy",
                        GTK_SIGNAL_FUNC (gtk_widget_destroyed), &filew);

    /* Connect the ok_button to filesave_ok_sel function */
    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
			"clicked", (GtkSignalFunc) fileload_ok_sel_inp, filew );
    
    /* Connect the cancel_button to destroy the widget */
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					   (filew)->cancel_button),
			       "clicked", (GtkSignalFunc) gtk_widget_destroy,
			       GTK_OBJECT (filew));

    gtk_file_selection_complete (GTK_FILE_SELECTION (filew),"*.sav");
  }

  if (!GTK_WIDGET_VISIBLE (filew))
    gtk_widget_show(filew);
  else 
    gtk_widget_destroy(filew);
}

void 
saveCB (GtkWidget *parent)
{
  /*  gtk_signal_emit_by_name (GTK_OBJECT (twin->Synbutton), "clicked",
      twin); */
  save_defaults ();
}

void 
saveasCB (GtkWidget *parent)
{
  /*  gtk_signal_emit_by_name (GTK_OBJECT (twin->Synbutton), "clicked",
      twin); */
  static GtkWidget *filew = NULL;
  
  if (!filew) {
    /* Create a new file selection widget */
    filew = gtk_file_selection_new ("Save configuration defaults");
    
    gtk_widget_set_name (GTK_WIDGET (filew), "aux window");
    gtk_file_selection_show_fileop_buttons (GTK_FILE_SELECTION (filew));
    gtk_signal_connect (GTK_OBJECT (filew), "destroy",
                        GTK_SIGNAL_FUNC (gtk_widget_destroyed), &filew);

    /* Connect the ok_button to filesave_ok_sel function */
    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
			"clicked", (GtkSignalFunc) filesave_ok_sel_inp, filew );
    
    /* Connect the cancel_button to destroy the widget */
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					   (filew)->cancel_button),
			       "clicked", (GtkSignalFunc) gtk_widget_destroy,
			       GTK_OBJECT (filew));

    gtk_file_selection_complete (GTK_FILE_SELECTION (filew),"*.sav");
  }

  if (!GTK_WIDGET_VISIBLE (filew))
    gtk_widget_show(filew);
  else 
    gtk_widget_destroy(filew);
}

void 
loggingCB (GtkWidget *parent, gpointer data)
{
  /*  gtk_signal_emit_by_name (GTK_OBJECT (twin->Synbutton), "clicked",
      twin); */
  log_mode = (int) data;
  switch ((int) data) {
  case LOG_VERBOSE:
    append_log(0,1,1,otfgui->logtext,"Logging set to verbose mode\n");
    break;
  case LOG_TERSE:
    append_log(0,1,1,otfgui->logtext,"Logging set to terse mode\n");
    break;
  }    
#ifdef DEBUG
  printf("loggingCB %d\n", (int) data);
#endif
}

void 
helpCB (GtkWidget *parent, gpointer data)
{
  int helpmode = (int) data;
  short int status=0;

  append_log(0,1,1,otfgui->logtext,"Launching netscape help - please wait\n");
  switch (helpmode) {
  case HELPF1:
    status = show_mozilla_url("http://swift.astro.umass.edu/otftool/otf.html");
    break;
  case HELPKEYWORD:
    status = show_mozilla_url("http://swift.astro.umass.edu/otftool/otf.html#keywords");
    break;
  case HELPOTFGEN:
    status = show_mozilla_url("http://swift.astro.umass.edu/otftool/otf.html#otftool_general");
    break; 
  case HELPDATAFILTER:
    status = show_mozilla_url("http://swift.astro.umass.edu/otftool/otf.html#datasel");
    break;
  case HELPDATAASSESS:
    status = show_mozilla_url("http://swift.astro.umass.edu/otftool/otf.html#data_assesment");
    break;
  case HELPDATAEDIT:
    status = show_mozilla_url("http://swift.astro.umass.edu/otftool/otf.html#data_editing");
    break;
  case HELPDATAREGRID:
    status = show_mozilla_url("http://swift.astro.umass.edu/otftool/otf.html#data_regrid");
    break;
  case HELPCHANGES:
    status = show_mozilla_url("http://swift.astro.umass.edu/otftool/ChangeLog");
    break;
  }
  if (status != 0) 
    append_log(0,1,1,otfgui->logtext,"Problem launching netscape help\n");
}

void 
keywordshelp (GtkWidget *parent)
{
  short int status;
  /*  append_log(0,1,1,otfgui->logtext,"Launching netscape help - please wait\n");*/
  status = show_mozilla_url("http://swift.astro.umass.edu/otftool/otf.html#keywords");
  if (status != 0) 
    append_log(0,1,1,otfgui->logtext,"Problem launching netscape help\n");
}

void 
aboutCB (GtkWidget *parent)
{
  char text[1024];

  sprintf (text, "OTFTOOL\n Version: (%g) \n\n", OTFREV);
  strcat (text, "Authors:");
  sprintf ( text + strlen ( text ), 
	    "\nGopal Narayanan <gopal@astro.umass.edu>\nMark Heyer <heyer@astro.umass.edu>\n\n");
  error_mes ("About", text);

}

void 
statusCB (GtkWidget *parent)
{
  if (statusint) {
    statusint = 0;
    if ((GTK_IS_WIDGET (otfgui->statuswin)) && (otfgui->statuswin !=NULL))
      gtk_widget_hide (GTK_WIDGET (otfgui->statuswin));
  } else {
    statusint = 1;
    /*    if (!GTK_WIDGET_VISIBLE (otfgui->statuswin))*/
    if ((GTK_IS_WIDGET (otfgui->statuswin)) && (otfgui->statuswin !=NULL))
      gtk_widget_show (GTK_WIDGET (otfgui->statuswin));
  }
#ifdef DEBUG
  printf ("status = %d\n", statusint);
#endif
}


void 
logwindowCB (GtkWidget *parent)
{
  if (logint) {
    logint = 0;
    if ((GTK_IS_WIDGET (otfgui->logwin)) && (otfgui->logwin !=NULL))
      gtk_widget_hide (GTK_WIDGET (otfgui->logwin));
  } else {
    logint = 1;
    if ((GTK_IS_WIDGET (otfgui->logwin)) && (otfgui->logwin !=NULL))
      gtk_widget_show (GTK_WIDGET (otfgui->logwin));
  }
#ifdef DEBUG
  printf ("Logging = %d\n", logint);
#endif
}

/* This is the GtkItemFactoryEntry structure used to generate new menus.
        Item 1: The menu path. The letter after the underscore indicates an
                accelerator key once the menu is open.
        Item 2: The accelerator key for the entry
        Item 3: The callback function.
        Item 4: The callback action.  This changes the parameters with
                which the function is called.  The default is 0.
        Item 5: The item type, used to define what kind of an item it is.
                Here are the possible values:

                NULL               -> "<Item>"
                ""                 -> "<Item>"
                "<Title>"          -> create a title item
                "<Item>"           -> create a simple item
                "<CheckItem>"      -> create a check item
                "<ToggleItem>"     -> create a toggle item
                "<RadioItem>"      -> create a radio item
                <path>             -> path of a radio item to link against
                "<Separator>"      -> create a separator
                "<Branch>"         -> create an item to hold sub items (optional)
                "<LastBranch>"     -> create a right justified branch 
     */

     static GtkItemFactoryEntry menu_items[] = {
       { "/_File",         NULL,         NULL, 0, "<Branch>" },
       { "/File/_Load", "<control>L", loadCB, 0, NULL },
       { "/File/_Save",     "<control>S", saveCB, 0, NULL },
       { "/File/Save _As",     NULL, saveasCB, 0, NULL },
       { "/File/sep1",        NULL,  NULL,       0, "<Separator>" },
       { "/File/_Quit", "<control>Q", destroy, 0, NULL },
       { "/_Options",      NULL,         NULL, 0, "<Branch>" },
       { "/_Options/_Logging", NULL, NULL, 0, "<Branch>" },
       { "/_Options/Logging/Verbose", NULL, loggingCB, LOG_VERBOSE, "<RadioItem>" },       
       { "/_Options/Logging/Terse", NULL, loggingCB, LOG_TERSE, "/Options/Logging/Verbose" },      
       { "/_Options/Status Window", NULL, statusCB, 0, "<CheckItem>" },
       { "/_Options/Logging Window", NULL, logwindowCB, 0, "<CheckItem>" },
       { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
       { "/_Help/_OTF Memo",   "F1",  helpCB, HELPF1, NULL },
       { "/_Help/OTF Keywords",  NULL,  helpCB, HELPKEYWORD, NULL },     
       { "/_Help/Otftool General",  NULL, helpCB, HELPOTFGEN, NULL },       
       { "/_Help/Data Filtering",  NULL, helpCB, HELPDATAFILTER, NULL },
       { "/_Help/Data Assessment",  NULL, helpCB, HELPDATAASSESS, NULL },
       { "/_Help/Data Editing",  NULL, helpCB, HELPDATAEDIT, NULL },
       { "/_Help/Data Regriding",  NULL, helpCB, HELPDATAREGRID, NULL },
       { "/_Help/Recent Changes",  NULL, helpCB, HELPCHANGES, NULL },
       { "/_Help/About Otftool",   NULL,         aboutCB, 0, NULL },
     };


void 
make_main_menu (GtkWidget  *window,
		     GtkWidget *parent,
		     GtkWidget **menubar )
     {
       GtkItemFactory *item_factory;
       GtkAccelGroup *accel_group;
       gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
       GtkWidget *radioitem;


       accel_group = gtk_accel_group_new ();
       
       /* This function initializes the item factory.
          Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
                   or GTK_TYPE_OPTION_MENU.
          Param 2: The path of the menu.
          Param 3: A pointer to a gtk_accel_group.  The item factory sets up
                   the accelerator table while generating menus.
       */

       item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", 
                                            accel_group);

       /* This function generates the menu items. Pass the item factory,
          the number of items in the array, the array itself, and any
          callback data for the the menu items. */
       gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, parent);

       /* Attach the new accelerator group to the window. */
       gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

       radioitem = gtk_item_factory_get_item(GTK_ITEM_FACTORY (item_factory), 
					     "/Options/Status Window");
       gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(radioitem), TRUE);

       radioitem = gtk_item_factory_get_item(GTK_ITEM_FACTORY (item_factory), 
					     "/Options/Logging Window");
       gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(radioitem), TRUE);

       if (menubar)
         /* Finally, return the actual menu bar created by the item factory. */ 
         *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
     }





/* create the menubar */

/*GtkWidget *setup_menu (GtkWidget *top,
  GtkWidget *parent)*/
GtkWidget 
*otftool_setup_menu (otf_gui *otfg)
{
  GtkWidget *menubarT;


  make_main_menu (otfg->mainwindow, otfg->vertbox, &menubarT);

  /* Add the menu bar to the parent window */

  gtk_box_pack_start (GTK_BOX (otfg->vertbox), menubarT, 
		      FALSE, FALSE, 0);
  gtk_widget_show (menubarT);
  
  return menubarT;

}




