#ifdef GUI
#include "otftool.h"
extern error_mes (gchar *, gchar *);
#else
#include "otf.h"
#endif

int 
findxylimits (int jfile, int NFILES, short ramp, char** otflist, float *xmin,
	      float *xmax, float *ymin, float *ymax)
{
  int i,j;
  int ih,idmp;
  char temp[MAXPATHLEN];
  char errmsg[300];
  OTFHeader_t C;
  float dx,dy,XP[4],YP[4];
  float expxmin, expxmax, expymin, expymax;
  double A;

  *xmin=1.0e+30;  *xmax=-1.0e+30;  *ymin=1.0e+30; *ymax=-1.0e+30;
  expxmin=1.0e+30;  expxmax=-1.0e+30;  expymin=1.0e+30; expymax=-1.0e+30;

  for (i = jfile; i<(jfile+NFILES); i++) {

    /*    strcpy(temp,rawdir);
	  strcat(temp,otflist[i]); */
    strcpy (temp, otflist[i]);

    if ((otffp=fopen(temp,"r"))==NULL) {
#ifdef GUI      
      sprintf(errmsg,"Input OTF file: %s",temp);
      strcat(errmsg,strerror(errno));
      error_mes("Error!", errmsg);
      return -1;
#else
      fprintf(stderr,"\n Cannot open file %s\n", otflist[i]);
      return -1;
#endif         
    }
    
    ReadHEADER();

   if (i==jfile) {
     strcpy(C.name,H.name);
     C.nchan=H.nchan;
     C.xvel1=H.xvel1;
     C.dveldin=H.dveldin;
     C.XCEN=H.XCEN;
     C.YCEN=H.YCEN;
   }
   else {
     if (strcmp(H.name,C.name)) {
#ifdef GUI       
       append_log(0,1,1,otfgui->logtext,
		  "WARNING: source names are not consistent in file %s\n", otflist[i]);
#else 
       fprintf(stderr,"\n WARNING: source names are not consistent\n"); 
#endif
     }
     if (fabs(H.xvel1-C.xvel1) > 0.2*fabs(H.dveldin)) {
#ifdef GUI
	append_log(0,1,1,otfgui->logtext,"ERROR: the spectroscopic axes are not aligned\n");
	return -1;
#else 
        fprintf(stderr,"\n ERROR: the spectroscopic axes are not aligned\n");
	return -1;
#endif
     }
      if (fabs(H.dveldin-C.dveldin) > 0.01) {
#ifdef GUI	
	append_log(0,1,1,otfgui->logtext,"ERROR: the spectroscopic axes are not sampled the same\n");
	return -1;
#else 
        fprintf(stderr,"\n ERROR: the spectroscopic axes are not sampled the same\n");
	return -1;
#endif
      }
      if (fabs(H.XCEN-C.XCEN) > 1.e-5) {
#ifdef GUI
	append_log(0,1,1,otfgui->logtext,"ERROR: the central coordindates are not consistent\n");
	return -1;
#else
        fprintf(stderr,"\n ERROR: the central coordindates are not consistent\n");
	return -1;
#endif
      }
      if (fabs(H.YCEN-C.YCEN) > 1.e-5) {
#ifdef GUI
	append_log(0,1,1,otfgui->logtext,"ERROR: the central coordindates are not consistent\n");
	return -1;
#else
        fprintf(stderr,"\n ERROR: the central coordindates are not consistent\n");
	return -1;
#endif
      }
   }

   /* Find endpoints in coordinate system */
   if (!ramp) {
     A=H.rotang*DEG2RAD;
     dx=H.XSIZE*0.5+H.XOFFMAP;
     dy=H.YSIZE*0.5+H.YOFFMAP;
     XP[0]=dx*cos(A)+dy*sin(A);
     YP[0]=dy*cos(A)-dx*sin(A);

     dx=H.XSIZE*0.5+H.XOFFMAP;
     dy=-H.YSIZE*0.5+H.YOFFMAP;
     XP[1]=dx*cos(A)+dy*sin(A);
     YP[1]=dy*cos(A)-dx*sin(A);

     dx=-H.XSIZE*0.5+H.XOFFMAP;
     dy=H.YSIZE*0.5+H.YOFFMAP;
     XP[2]=dx*cos(A)+dy*sin(A);
     YP[2]=dy*cos(A)-dx*sin(A);

     dx=-H.XSIZE*0.5+H.XOFFMAP;
     dy=-H.YSIZE*0.5+H.YOFFMAP;
     XP[3]=dx*cos(A)+dy*sin(A);
     YP[3]=dy*cos(A)-dx*sin(A);
     for(j=0; j<4; j++) {
       if (XP[j] < expxmin) expxmin=XP[j];
       if (XP[j] > expxmax) expxmax=XP[j];
       if (YP[j] < expymin) expymin=YP[j];
       if (YP[j] > expymax) expymax=YP[j];
     }
   }
   else {
    for (idmp = 0; idmp<H.nsample; idmp++) {
      if (OTFPOS.XOFF[idmp] < 21600) {
        for (ih=0; ih<H.nhorns; ih++) {
	  if (horn_is_active[ih]) {
	    if (XPOS[ih][idmp] < expxmin) expxmin = XPOS[ih][idmp];
	    if (XPOS[ih][idmp] > expxmax) expxmax = XPOS[ih][idmp];
	    if (YPOS[ih][idmp] < expymin) expymin = YPOS[ih][idmp];
	    if (YPOS[ih][idmp] > expymax) expymax = YPOS[ih][idmp];
	  }
        }
      }
    }
   }
   fclose(otffp);
  }
   *xmin=expxmin;
   *xmax=expxmax;
   *ymin=expymin;
   *ymax=expymax;
   fprintf(stderr,"\n %6.2f %6.2f %6.2f %6.2f", *xmin,*xmax,*ymin,*ymax);
  return 0;
}

int
plotpos (int jfile, int NFILES, char ** otflist, int plotdev)
{
  float xmin, xmax, ymin, ymax;
  float xrampmin, xrampmax, yrampmin, yrampmax;
  /*  float dxrampmin, dxrampmax, dyrampmin, dyrampmax;*/
  float xramp[5], yramp[5];
  float dxramp[5], dyramp[5];
  int i, j, k, num;
  char temp[MAXPATHLEN];
  char errmsg[300];
  float x[MAXSCANSIZE*MAXHORNS],y[MAXSCANSIZE*MAXHORNS];
  double A;

  if (findxylimits (jfile, NFILES, 1, otflist, &xmin, &xmax, &ymin, &ymax) == -1) {
#ifdef GUI    
    append_log (2,1,1,otfgui->logtext, "Problem plotting positions\n");
    return -1;
#else
    fprintf(stderr,"\nProblem plotting positions\n");
    return -1;
#endif
  }

  if (findxylimits (jfile, NFILES, 0, otflist, &xrampmin, &xrampmax, 
		    &yrampmin, &yrampmax) == -1) {
#ifdef GUI    
    append_log (2,1,1,otfgui->logtext, "Problem plotting positions\n");
    return -1;
#else
    fprintf(stderr,"\nProblem plotting positions\n");
    return -1;
#endif
  }

  A=H.rotang*DEG2RAD;
  xramp[0] = xrampmax; yramp[0] = yrampmin;
  xramp[1] = xrampmin; yramp[1] = yrampmin;
  xramp[2] = xrampmin; yramp[2] = yrampmax;
  xramp[3] = xrampmax; yramp[3] = yrampmax;
  xramp[4] = xrampmax; yramp[4] = yrampmin;

  for (i=0; i<5; i++) {
    dxramp[i] = xramp[i]*cos(A) - yramp[i]*sin(A);
    dyramp[i] = xramp[i]*sin(A) + yramp[i]*cos(A);
  }

#ifdef GUI
  append_log (0,1,1,otfgui->logtext, "Plot limits: %6.2f %6.2f %6.2f %6.2f\n", xmax,xmin,ymin,ymax);
#endif

#ifdef GUI    
  if (id1 != 0 ) {
    append_log(0,1,1,otfgui->logtext,"Pgplot window in use. Quit otfedit first\n");
    error_mes("Error!","Pgplot window in use. Quit otfedit first");
    return -1;
  }
#endif

  if (plotdev == 0)
    id1 = cpgopen("/xserv");
  else 
    id1 = cpgopen("otfplotpos.ps/ps");
  if (id1 < 1) {
#ifdef GUI
    append_log(0,1,1,otfgui->logtext,"Error opening pgplot device\n");    
    return -1;
#else
    fprintf(stderr,"Error opening pgplot device\n");    
#endif
  }
  cpgenv(xmax, xmin, ymin, ymax,1,0);
  cpglab ("X (arcmin)", "Y (arcmin)", "Positions");
  cpgsci(1); cpgsls(1);
  
  for (i = jfile; i<(jfile+NFILES); i++) {
    
    /*    strcpy(temp,rawdir);
	  strcat(temp,otflist[i]); */
    strcpy (temp, otflist[i]);
    
    if ((otffp=fopen(temp,"r"))==NULL) {
#ifdef GUI      
      sprintf(errmsg,"Input OTF file: %s",temp);
      strcat(errmsg,strerror(errno));
      error_mes("Error!", errmsg);
      return -1;
#else
      fprintf(stderr,"\n Cannot open file %s\n", otflist[i]);
	return -1;
#endif         
      }
    
    ReadHEADER();
    
    num = 0;
    for (j = 0; j<H.nsample; j++) {
      for (k=0; k<H.nhorns; k++) {
	if (horn_is_active[k]) {
	  x[num] = XPOS[k][j];
	  y[num] = YPOS[k][j];
	  num++;
	}
      }
    }
    cpgpt (num, x, y, 1);
    cpgsci(2); cpgsls(2); cpgslw(3);
    cpgline(5,dxramp,dyramp);
    cpgsci(1); cpgsls(1); cpgslw(1);
  }
  cpgslct(id1);
  cpgclos();
  id1 = 0;
  return 0;
}
