/*
 * structure_par.h
 * Derived from GAG/linux-intel/astro/class/structure.par
 * fortran include file
 * Gopal Narayanan <gopal@astro.umass.edu>
 * Sept 4, 2001
 */

#ifndef __STRUCTURE_PAR_H_
#define __STRUCTURE_PAR_H_

#include <cfortran.h>

#define MPAD 6
#define MTPAR 512
#define MGEN 9
#define MGEN2 11
#define MPOS 11
#define MSPEC 18
#define MWIND 20
#define MBASE 4+2*MWIND
#define MSEQ 100
#define MORIG 2*MSEQ+1
#define MPLOT 4
#define MXPHAS 8


#define MCALIB 25
#define MSKY 10
#define MSKYDIP 10+4*MSKY
#define MXGAUS 5
#define NGAUS 3*MXGAUS
#define MFIT 3+6*MXGAUS
#define MSHELL 43
#define MNH3 27
#define MABS 35
#define MCONT 16
#define MBEAM 5
#define MFCONT 19
#define MDESC 4

typedef struct {
  int RALIG1, RNUM, RVER;
  union {
    int RTELES[3];
    char TELNAME[12];
  } tel;
  int RDOBS, RDRED, RTYPEC, RKIND, RQUAL, RSCAN;
  double RUT, RST;
  float RAZ, REL, RTAU, RTSYS, RTIME;
  char RCDOBS[12], RCDRED[12];
  int RALIG2;
  union {
    int RSOURC[3];
    char SOURCENAME[12];
  } source;
  float REPOCH;
  double RLAM, RBET;
  float RLAMOF, RBETOF;
  int RPROJ;
  union {
    int RLINE[3];
    char LINENAME[12];
  } line;
  double RRESTF;
  int RNCHAN;
  float RRCHAN, RFRES, RFOFF, RVRES, RVOFF, RBAD;
  int RALIG3;
  double RIMAGE;
  int RVTYPE, RALIG4;
  double RSKYFR;
  float RVTELES;
  int RDEG;
  float RSIGFI, RAIRE;
  int RNWIND;
  float RW1[MWIND], RW2[MWIND];
  int RNSEQ;
  int RSTART[MSEQ], REND[MSEQ];
  float RAMIN, RAMAX, RVMIN, RVMAX;
  int RALIG5, RNPHAS;
  double RDECAL[MXPHAS];
  float RDUREE[MXPHAS], RPOIDS[MXPHAS];
  int RSWMOD;
  float RLDECAL[MXPHAS], RBDECAL[MXPHAS];
  float RBEEFF, RFOEFF, RGAINI, RH2OMM, RPAMB, RTAMB, RTATMS;
  float RTCHOP, RTCOLD, RTAUS, RTAUI, RTATMI, RTREC;
  int RCMODE;
  float RATFAC, RALTI, RCOUNT[3], RLCALOF, RBCALOF;
  double RGEOLONG, RGEOLAT;
  int RALIG6;
  int RSLINE[3];
  double RSREST, RSIMAG;
  int RNSKY, RNCHOP, RNCOLD;
  float RELEV[MSKY], REMISS[MSKY], RCHOPP[MSKY], RCOLD[MSKY];
  int R_NDUMP, R_LDPAR, R_LDATL, R_LDUMP;
  int MRPAR;
} CRPAR_DEF;

#define Crpar COMMON_BLOCK(CRPAR,crpar)
COMMON_BLOCK_DEF(CRPAR_DEF,Crpar);

#define RTELES tel.RTELES
#define RSOURC source.RSOURC
#define RLINE line.RLINE

typedef struct {
  int RNLINE;
  float RSIGBA, RSIGRA;
  float RNFIT[NGAUS], RNERR[NGAUS];
} CRGAUS_DEF;

#define Crgaus COMMON_BLOCK(CRGAUS,crgaus)
COMMON_BLOCK_DEF(CRGAUS_DEF,Crgaus);

typedef struct {
  int RLSHEL;
  float RBSHEL, RRSHEL, RNSHEL[20], RESHEL[20];
} CRSHEL_DEF;

#define Crshel COMMON_BLOCK(CRSHEL,crshel)
COMMON_BLOCK_DEF(CRSHEL_DEF,Crshel);

typedef struct {
  int RLNH3;
  float  RBNH3, RRNH3, RNNH3[12], RENH3[12];
} CRNH3_DEF;

#define Crnh3 COMMON_BLOCK(CRNH3,crnh3)
COMMON_BLOCK_DEF(CRNH3_DEF,Crnh3);

typedef struct {
  int RLABS;
  float RBABS, RRABS, RNABS[16], REABS[16];
} CRABS_DEF;

#define Crabs COMMON_BLOCK(CRABS,crabs)
COMMON_BLOCK_DEF(CRABS_DEF,Crabs);

typedef struct {
  double RFREQ;
  float RWIDTH;
  int RNPOIN;
  float RRPOIN, RTREF, RAREF, RAPOS, RTRES, RARES, RCBAD;
  int RCTYPE;
  double RCIMAG;
  float RCOLLA, RCOLLE, RCAZIM, RCELEV, RSPACE, RBPOS;
  int RBTYPE;
} CRCONT_DEF;

#define Crcont COMMON_BLOCK(CRCONT,crcont)
COMMON_BLOCK_DEF(CRCONT_DEF,Crcont);

typedef struct {
  int RLCONT;
  float RBCONT, RRCONT, RNCONT[8], RECONT[8];
} CRPOINT_DEF;

#define Crpoint COMMON_BLOCK(CRPOINT,crpoint)
COMMON_BLOCK_DEF(CRPOINT_DEF,Crpoint);

#endif
